@@ -7,12 +7,12 @@ ds = yt.load("IsolatedGalaxy/galaxy0030/galaxy0030")
 p = yt.ProjectionPlot(ds, "z", "density", width=(100, 'kpc'))
 p.save()
 
-# Change the colormap to 'jet' and save again.  We must specify 
-# a different filename here or it will save it over the top of 
+# Change the colormap to 'dusk' and save again.  We must specify
+# a different filename here or it will save it over the top of
 # our first projection.
-p.set_cmap(field="density", cmap='jet')
-p.save('proj_with_jet_cmap.png')
+p.set_cmap(field="density", cmap='dusk')
+p.save('proj_with_dusk_cmap.png')
 
-# Change the colormap to 'hot' and save again.  
+# Change the colormap to 'hot' and save again.
 p.set_cmap(field="density", cmap='hot')
 p.save('proj_with_hot_cmap.png')
