@@ -12,27 +12,17 @@ hc = HaloCatalog(data_ds=data_ds, halos_ds=halos_ds)
 # Filter out less massive halos
 hc.add_filter("quantity_value", "particle_mass", ">", 1e14, "Msun")
 
-# attach a sphere object to each halo whose radius extends
-#   to twice the radius of the halo
-hc.add_callback("sphere", factor=2.0)
-
-# use the sphere to calculate radial profiles of gas density
-# weighted by cell volume in terms of the virial radius
-hc.add_callback("profile", ["radius"],
-                [("gas", "overdensity")],
-                weight_field="cell_volume",
-                accumulation=True,
-                storage="virial_quantities_profiles")
-
-
-hc.add_callback("virial_quantities", ["radius"],
-                profile_storage="virial_quantities_profiles")
-hc.add_callback('delete_attribute', 'virial_quantities_profiles')
+# This recipe creates a spherical data container, computes
+# radial profiles, and calculates r_200 and M_200.
+hc.add_recipe("calculate_virial_quantities", ["radius", "matter_mass"])
 
+# Create a sphere container with radius 5x r_200.
 field_params = dict(virial_radius=('quantity', 'radius_200'))
 hc.add_callback('sphere', radius_field='radius_200', factor=5,
                 field_parameters=field_params)
-hc.add_callback('profile', ['virial_radius_fraction'], 
+
+# Compute profiles of T vs. r/r_200
+hc.add_callback('profile', ['virial_radius_fraction'],
                 [('gas', 'temperature')],
                 storage='virial_profiles',
                 weight_field='cell_mass',
