@@ -365,7 +365,7 @@ class AbsorptionSpectrum(object):
                                    this_wavelength[lixel]), \
                               continuum['index']) * \
                     (column_density[lixel] / continuum['normalization'])
-                self.tau_field[left_index[lixel]:right_index[lixel]] += cont_tau
+                self.tau_field[left_index[lixel]:right_index[lixel]] += cont_tau.d
                 pbar.update(i)
             pbar.finish()
 
@@ -495,8 +495,8 @@ class AbsorptionSpectrum(object):
                 window_width_in_bins = 2
 
                 while True:
-                    left_index = (center_index[i] - window_width_in_bins/2)
-                    right_index = (center_index[i] + window_width_in_bins/2)
+                    left_index = (center_index[i] - window_width_in_bins//2)
+                    right_index = (center_index[i] + window_width_in_bins//2)
                     n_vbins = (right_index - left_index) * n_vbins_per_bin[i]
 
                     # the array of virtual bins in lambda space
