@@ -27,6 +27,9 @@
     #define Py_TYPE(ob) (((PyObject*)(ob))->ob_type)
 #endif
 
+void PrepareKD(KD kd);
+int kdMedianJst(KD kd, int d, int l, int u);
+void kdUpPass(KD kd, int iCell);
 void initgrouplist(Grouplist *g);
 void hop_main(KD kd, HC *my_comm, float densthres);
 void regroup_main(float dens_outer, HC *my_comm);
