@@ -16,23 +16,21 @@ from __future__ import absolute_import
 
 from yt.config import ytcfg
 from yt.data_objects.time_series import \
-     DatasetSeries
+    DatasetSeries
+from yt.extern import \
+    six
 from yt.funcs import \
-     is_root
+    is_root, mylog
 from yt.utilities.parallel_tools.parallel_analysis_interface import \
-    ParallelAnalysisInterface, ProcessorPool, Communicator
-from yt.analysis_modules.halo_finding.halo_objects import * #Halos & HaloLists
+    ParallelAnalysisInterface, ProcessorPool
 from yt.utilities.exceptions import YTRockstarMultiMassNotSupported
 
 from . import rockstar_interface
 
 import socket
 import time
-import threading
-import signal
 import os
-from os import environ
-from os import mkdir
+import numpy as np
 from os import path
 
 class InlineRunner(ParallelAnalysisInterface):
@@ -118,29 +116,29 @@ class RockstarHaloFinder(ParallelAnalysisInterface):
 
     Parameters
     ----------
-    ts   : DatasetSeries, Dataset
+    ts : DatasetSeries, ~yt.data_objects.static_output.Dataset
         This is the data source containing the DM particles. Because 
         halo IDs may change from one snapshot to the next, the only
         way to keep a consistent halo ID across time is to feed 
         Rockstar a set of snapshots, ie, via DatasetSeries.
-    num_readers: int
+    num_readers : int
         The number of reader can be increased from the default
         of 1 in the event that a single snapshot is split among
         many files. This can help in cases where performance is
         IO-limited. Default is 1. If run inline, it is
         equal to the number of MPI threads.
-    num_writers: int
+    num_writers : int
         The number of writers determines the number of processing threads
         as well as the number of threads writing output data.
         The default is set to comm.size-num_readers-1. If run inline,
         the default is equal to the number of MPI threads.
-    outbase: str
+    outbase : str
         This is where the out*list files that Rockstar makes should be
         placed. Default is 'rockstar_halos'.
-    particle_type: str
+    particle_type : str
         This is the "particle type" that can be found in the data.  This can be
         a filtered particle or an inherent type.
-    force_res: float
+    force_res : float
         This parameter specifies the force resolution that Rockstar uses
         in units of Mpc/h.
         If no value is provided, this parameter is automatically set to
@@ -169,13 +167,12 @@ class RockstarHaloFinder(ParallelAnalysisInterface):
     --------
     
     To use the script below you must run it using MPI:
-    mpirun -np 4 python run_rockstar.py --parallel
+    mpirun -np 4 python run_rockstar.py
 
     >>> import yt
+    >>> yt.enable_parallelism()
     >>> from yt.analysis_modules.halo_finding.rockstar.api import \
-    ... RockstarHaloFinder
-    >>> from yt.data_objects.particle_filters import \
-    ... particle_filter
+    ...     RockstarHaloFinder
 
     >>> # create a particle filter to remove star particles
     >>> @yt.particle_filter("dark_matter", requires=["creation_time"])
@@ -202,7 +199,7 @@ class RockstarHaloFinder(ParallelAnalysisInterface):
             mylog.info("http://adsabs.harvard.edu/abs/2013ApJ...762..109B")
         ParallelAnalysisInterface.__init__(self)
         # Decide how we're working.
-        if ytcfg.getboolean("yt", "inline") == True:
+        if ytcfg.getboolean("yt", "inline") is True:
             self.runner = InlineRunner()
         else:
             self.runner = StandardRunner(num_readers, num_writers)
@@ -247,8 +244,6 @@ class RockstarHaloFinder(ParallelAnalysisInterface):
 
         dd = tds.all_data()
         # Get DM particle mass.
-        all_fields = set(tds.derived_field_list + tds.field_list)
-        has_particle_type = ("particle_type" in all_fields)
 
         particle_mass = self.particle_mass
         if particle_mass is None:
@@ -332,7 +327,9 @@ class RockstarHaloFinder(ParallelAnalysisInterface):
             self.ts._pre_outputs = self.ts._pre_outputs[restart_num:]
         else:
             restart_num = 0
-        self.handler.setup_rockstar(self.server_address, self.port,
+        self.handler.setup_rockstar(
+                    six.b(self.server_address),
+                    six.b(self.port),
                     num_outputs, self.total_particles, 
                     self.particle_type,
                     particle_mass = self.particle_mass,
@@ -341,7 +338,7 @@ class RockstarHaloFinder(ParallelAnalysisInterface):
                     num_writers = self.num_writers,
                     writing_port = -1,
                     block_ratio = block_ratio,
-                    outbase = self.outbase,
+                    outbase = six.b(self.outbase),
                     force_res = self.force_res,
                     callbacks = callbacks,
                     restart_num = restart_num,
@@ -370,10 +367,3 @@ class RockstarHaloFinder(ParallelAnalysisInterface):
             self.runner.run(self.handler, self.workgroup)
         self.comm.barrier()
         self.pool.free_all()
-    
-    def halo_list(self,file_name='out_0.list'):
-        """
-        Reads in the out_0.list file and generates RockstarHaloList
-        and RockstarHalo objects.
-        """
-        return RockstarHaloList(self.ts[0], self.outbase+'/%s'%file_name)
