@@ -1,6 +1,7 @@
 """
 API for ppv_cube
 """
+from __future__ import absolute_import
 #-----------------------------------------------------------------------------
 # Copyright (c) 2013, yt Development Team.
 #
@@ -9,4 +10,4 @@ API for ppv_cube
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-from ppv_cube import PPVCube
+from .ppv_cube import PPVCube
