@@ -13,6 +13,14 @@ API for two_point_functions
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
+
+from yt.funcs import issue_deprecation_warning
+
+issue_deprecation_warning(
+    "The two_point_functions module has been deprecated. This code has been "
+    "moved to the yt attic (https://bitbucket.org/yt_analysis/yt_attic) and "
+    "will be removed in a future release.")
+
 from .two_point_functions import \
     TwoPointFunctions, \
     FcnSet
