@@ -13,27 +13,31 @@ Python-based grid handler, not to be confused with the SWIG-handler
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-import pdb
+import warnings
 import weakref
-import itertools
 import numpy as np
+from six import string_types
 
-from yt.funcs import *
-
+from yt.config import ytcfg
 from yt.data_objects.data_containers import \
-    YTFieldData, \
-    YTDataContainer, \
     YTSelectionContainer
-from yt.fields.field_exceptions import \
-    NeedsGridType, \
-    NeedsOriginalGrid, \
-    NeedsDataField, \
-    NeedsProperty, \
-    NeedsParameter
+from yt.data_objects.field_data import \
+    YTFieldData
+from yt.funcs import iterable
 from yt.geometry.selection_routines import convert_mask_to_indices
 import yt.geometry.particle_deposit as particle_deposit
-from yt.utilities.lib.Interpolators import \
+from yt.units.yt_array import YTArray
+from yt.utilities.exceptions import \
+    YTFieldTypeNotFound, \
+    YTParticleDepositionNotImplemented
+from yt.utilities.lib.interpolators import \
     ghost_zone_interpolate
+from yt.utilities.lib.mesh_utilities import \
+    clamp_edges
+from yt.utilities.nodal_data_utils import \
+    get_nodal_slices
+
+RECONSTRUCT_INDEX = bool(ytcfg.get('yt', 'reconstruct_index'))
 
 class AMRGridPatch(YTSelectionContainer):
     _spatial = True
@@ -95,7 +99,9 @@ class AMRGridPatch(YTSelectionContainer):
             return tr
         finfo = self.ds._get_field_info(*fields[0])
         if not finfo.particle_type:
-            return tr.reshape(self.ActiveDimensions)
+            num_nodes = 2**sum(finfo.nodal_flag)
+            new_shape = list(self.ActiveDimensions) + [num_nodes]
+            return np.squeeze(tr.reshape(new_shape))
         return tr
 
     def convert(self, datatype):
@@ -135,19 +141,22 @@ class AMRGridPatch(YTSelectionContainer):
         # So first we figure out what the index is.  We don't assume
         # that dx=dy=dz, at least here.  We probably do elsewhere.
         id = self.id - self._id_offset
+        ds = self.ds
+        index = self.index
         if self.Parent is not None:
             if not hasattr(self.Parent, 'dds'):
                 self.Parent._setup_dx()
-            self.dds = self.Parent.dds.ndarray_view() / self.ds.refine_by
+            self.dds = self.Parent.dds.d / self.ds.refine_by
         else:
-            LE, RE = self.index.grid_left_edge[id,:], \
-                     self.index.grid_right_edge[id,:]
+            LE, RE = (index.grid_left_edge[id, :].d,
+                      index.grid_right_edge[id, :].d)
             self.dds = (RE - LE) / self.ActiveDimensions
-        if self.ds.dimensionality < 2:
-            self.dds[1] = self.ds.domain_right_edge[1] - self.ds.domain_left_edge[1]
         if self.ds.dimensionality < 3:
-            self.dds[2] = self.ds.domain_right_edge[2] - self.ds.domain_left_edge[2]
-        self.dds = self.ds.arr(self.dds, "code_length")
+            self.dds[2] = ds.domain_right_edge[2] - ds.domain_left_edge[2]
+        elif self.ds.dimensionality < 2:
+            self.dds[1] = ds.domain_right_edge[1] - ds.domain_left_edge[1]
+        self.dds = self.dds.view(YTArray)
+        self.dds.units = self.index.grid_left_edge.units
 
     def __repr__(self):
         return "AMRGridPatch_%04i" % (self.id)
@@ -174,6 +183,18 @@ class AMRGridPatch(YTSelectionContainer):
         self.ActiveDimensions = h.grid_dimensions[my_ind]
         self.LeftEdge = h.grid_left_edge[my_ind]
         self.RightEdge = h.grid_right_edge[my_ind]
+        # This can be expensive so we allow people to disable this behavior
+        # via a config option
+        if RECONSTRUCT_INDEX:
+            if iterable(self.Parent) and len(self.Parent) > 0:
+                p = self.Parent[0]
+            else:
+                p = self.Parent
+            if p is not None and p != []:
+                # clamp grid edges to an integer multiple of the parent cell
+                # width
+                clamp_edges(self.LeftEdge, p.LeftEdge, p.dds)
+                clamp_edges(self.RightEdge, p.RightEdge, p.dds)
         h.grid_levels[my_ind, 0] = self.Level
         # This might be needed for streaming formats
         #self.Time = h.gridTimes[my_ind,0]
@@ -189,8 +210,8 @@ class AMRGridPatch(YTSelectionContainer):
         if dlevel != 1:
             rf = rf**dlevel
         gi, cgi = self.get_global_startindex(), child.get_global_startindex()
-        startIndex = np.maximum(0, cgi / rf - gi)
-        endIndex = np.minimum((cgi + child.ActiveDimensions) / rf - gi,
+        startIndex = np.maximum(0, cgi // rf - gi)
+        endIndex = np.minimum((cgi + child.ActiveDimensions) // rf - gi,
                               self.ActiveDimensions)
         endIndex += (startIndex == endIndex)
         mask[startIndex[0]:endIndex[0],
@@ -234,15 +255,12 @@ class AMRGridPatch(YTSelectionContainer):
         # We will attempt this by creating a datacube that is exactly bigger
         # than the grid by nZones*dx in each direction
         nl = self.get_global_startindex() - n_zones
-        nr = nl + self.ActiveDimensions + 2 * n_zones
         new_left_edge = nl * self.dds + self.ds.domain_left_edge
-        new_right_edge = nr * self.dds + self.ds.domain_left_edge
 
         # Something different needs to be done for the root grid, though
         level = self.Level
         if all_levels:
             level = self.index.max_level + 1
-        args = (level, new_left_edge, new_right_edge)
         kwargs = {'dims': self.ActiveDimensions + 2*n_zones,
                   'num_ghost_zones':n_zones,
                   'use_pbar':False, 'fields':fields}
@@ -262,33 +280,51 @@ class AMRGridPatch(YTSelectionContainer):
         cube._base_grid = self
         return cube
 
-    def get_vertex_centered_data(self, field, smoothed=True, no_ghost=False):
-        new_field = np.zeros(self.ActiveDimensions + 1, dtype='float64')
+    def get_vertex_centered_data(self, fields, smoothed=True, no_ghost=False):
+        _old_api = isinstance(fields, (string_types, tuple))
+        if _old_api:
+            message = (
+                'get_vertex_centered_data() requires list of fields, rather than '
+                'a single field as an argument.'
+            )
+            warnings.warn(message, DeprecationWarning, stacklevel=2)
+            fields = [fields]
 
+        # Make sure the field list has only unique entries
+        fields = list(set(fields))
+        new_fields = {}
+        for field in fields:
+            finfo = self.ds._get_field_info(field)
+            new_fields[field] = self.ds.arr(
+                np.zeros(self.ActiveDimensions + 1), finfo.output_units)
         if no_ghost:
-            # Ensure we have the native endianness in this array.  Avoid making
-            # a copy if possible.
-            old_field = np.asarray(self[field], dtype="=f8")
-            # We'll use the ghost zone routine, which will naturally
-            # extrapolate here.
-            input_left = np.array([0.5, 0.5, 0.5], dtype="float64")
-            output_left = np.array([0.0, 0.0, 0.0], dtype="float64")
-            # rf = 1 here
-            ghost_zone_interpolate(1, old_field, input_left,
-                                   new_field, output_left)
+            for field in fields:
+                # Ensure we have the native endianness in this array.  Avoid making
+                # a copy if possible.
+                old_field = np.asarray(self[field], dtype="=f8")
+                # We'll use the ghost zone routine, which will naturally
+                # extrapolate here.
+                input_left = np.array([0.5, 0.5, 0.5], dtype="float64")
+                output_left = np.array([0.0, 0.0, 0.0], dtype="float64")
+                # rf = 1 here
+                ghost_zone_interpolate(1, old_field, input_left,
+                                       new_fields[field], output_left)
         else:
-            cg = self.retrieve_ghost_zones(1, field, smoothed=smoothed)
-            np.add(new_field, cg[field][1: ,1: ,1: ], new_field)
-            np.add(new_field, cg[field][:-1,1: ,1: ], new_field)
-            np.add(new_field, cg[field][1: ,:-1,1: ], new_field)
-            np.add(new_field, cg[field][1: ,1: ,:-1], new_field)
-            np.add(new_field, cg[field][:-1,1: ,:-1], new_field)
-            np.add(new_field, cg[field][1: ,:-1,:-1], new_field)
-            np.add(new_field, cg[field][:-1,:-1,1: ], new_field)
-            np.add(new_field, cg[field][:-1,:-1,:-1], new_field)
-            np.multiply(new_field, 0.125, new_field)
+            cg = self.retrieve_ghost_zones(1, fields, smoothed=smoothed)
+            for field in fields:
+                np.add(new_fields[field], cg[field][1: ,1: ,1: ], new_fields[field])
+                np.add(new_fields[field], cg[field][:-1,1: ,1: ], new_fields[field])
+                np.add(new_fields[field], cg[field][1: ,:-1,1: ], new_fields[field])
+                np.add(new_fields[field], cg[field][1: ,1: ,:-1], new_fields[field])
+                np.add(new_fields[field], cg[field][:-1,1: ,:-1], new_fields[field])
+                np.add(new_fields[field], cg[field][1: ,:-1,:-1], new_fields[field])
+                np.add(new_fields[field], cg[field][:-1,:-1,1: ], new_fields[field])
+                np.add(new_fields[field], cg[field][:-1,:-1,:-1], new_fields[field])
+                np.multiply(new_fields[field], 0.125, new_fields[field])
 
-        return new_field
+        if _old_api:
+            return new_fields[fields[0]]
+        return new_fields
 
     def select_icoords(self, dobj):
         mask = self._get_selector_mask(dobj.selector)
@@ -331,17 +367,22 @@ class AMRGridPatch(YTSelectionContainer):
     def particle_operation(self, *args, **kwargs):
         raise NotImplementedError
 
-    def deposit(self, positions, fields = None, method = None):
+    def deposit(self, positions, fields = None, method = None,
+                kernel_name = 'cubic'):
         # Here we perform our particle deposition.
         cls = getattr(particle_deposit, "deposit_%s" % method, None)
         if cls is None:
             raise YTParticleDepositionNotImplemented(method)
-        op = cls(self.ActiveDimensions.prod()) # We allocate number of zones, not number of octs
+        # We allocate number of zones, not number of octs. Everything inside
+        # this is Fortran ordered because of the ordering in the octree deposit
+        # routines, so we reverse it here to match the convention there
+        op = cls(tuple(self.ActiveDimensions[::-1]), kernel_name)
         op.initialize()
         op.process_grid(self, positions, fields)
         vals = op.finalize()
         if vals is None: return
-        return vals.reshape(self.ActiveDimensions, order="C")
+        # Fortran-ordered, so transpose.
+        return vals.transpose()
 
     def select_blocks(self, selector):
         mask = self._get_selector_mask(selector)
@@ -365,7 +406,13 @@ class AMRGridPatch(YTSelectionContainer):
         mask = self._get_selector_mask(selector)
         count = self.count(selector)
         if count == 0: return 0
-        dest[offset:offset+count] = source[mask]
+        nodal_flag = source.shape - self.ActiveDimensions
+        if sum(nodal_flag) == 0:
+            dest[offset:offset+count] = source[mask]
+        else:
+            slices = get_nodal_slices(source.shape, nodal_flag)
+            for i , sl in enumerate(slices):
+                dest[offset:offset+count, i] = source[sl][mask]
         return count
 
     def count(self, selector):
