@@ -14,16 +14,14 @@ This is a library for defining and using particle filters.
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-import numpy as np
 import copy
+from collections import defaultdict
 
 from contextlib import contextmanager
-from functools import wraps
 
 from yt.fields.field_info_container import \
     NullFunc, TranslationFunc
 from yt.utilities.exceptions import YTIllDefinedFilter
-from yt.funcs import *
 
 # One to many mapping
 filter_registry = defaultdict(list)
@@ -71,9 +69,13 @@ class ParticleFilter(object):
         # fields are implicitly "all" or something.
         return all((self.filtered_type, field) in field_list for field in self.requires)
 
+    def missing(self, field_list):
+        return list((self.filtered_type, field) for field in self.requires if
+                    (self.filtered_type, field) not in field_list)
+
     def wrap_func(self, field_name, old_fi):
         new_fi = copy.copy(old_fi)
-        new_fi.name = (self.filtered_type, field_name[1])
+        new_fi.name = (self.name, field_name[1])
         if old_fi._function == NullFunc:
             new_fi._function = TranslationFunc(old_fi.name)
         return new_fi
