@@ -34,7 +34,7 @@ def test_compose_no_overlap():
     empty = np.array([])
     for n in [1, 2, 4, 8]:
         ds = fake_random_ds(64, nprocs=n)
-        ds.add_field(("index", "ID"), function=_IDFIELD)
+        ds.add_field(("index", "ID"), sampling_type='cell', function=_IDFIELD)
 
         # position parameters for initial region
         center = [0.25]*3
@@ -56,24 +56,24 @@ def test_compose_no_overlap():
         # subselect non-overlapping 0, 1, 2, 3D regions
         for data1 in sources:
             data2 = ds.sphere(center, radius, data_source=data1)
-            yield assert_array_equal, data2['index', 'ID'], empty
+            assert_array_equal(data2['index', 'ID'], empty)
 
             data2 = ds.region(center, left_edge, right_edge, data_source=data1)
-            yield assert_array_equal, data2['index', 'ID'], empty  
+            assert_array_equal(data2['index', 'ID'], empty  )
 
             data2 = ds.disk(center, normal, radius, height, data_source=data1)
-            yield assert_array_equal, data2['index', 'ID'], empty
+            assert_array_equal(data2['index', 'ID'], empty)
 
             for d in range(3):
                 data2 = ds.slice(d, center[d], data_source=data1)
-                yield assert_array_equal, data2['index', 'ID'], empty
+                assert_array_equal(data2['index', 'ID'], empty)
 
             for d in range(3):
                 data2 = ds.ortho_ray(d, center[0:d] + center[d+1:], data_source=data1)
-                yield assert_array_equal, data2['index', 'ID'], empty
+                assert_array_equal(data2['index', 'ID'], empty)
 
             data2 = ds.point(center, data_source=data1)
-            yield assert_array_equal, data2['index', 'ID'], empty
+            assert_array_equal(data2['index', 'ID'], empty)
 
 def test_compose_overlap():
     r"""Test to make sure that composed data objects that do
@@ -81,7 +81,7 @@ def test_compose_overlap():
     """
     for n in [1, 2, 4, 8]:
         ds = fake_random_ds(64, nprocs=n)
-        ds.add_field(("index", "ID"), function=_IDFIELD)
+        ds.add_field(("index", "ID"), sampling_type='cell', function=_IDFIELD)
 
         # position parameters for initial region
         center = [0.4, 0.5, 0.5]
@@ -109,21 +109,21 @@ def test_compose_overlap():
             id2 = data2['index', 'ID']
             id3 = data3['index', 'ID']
             id3.sort()
-            yield assert_array_equal, uintersect1d(id1, id2), id3
+            assert_array_equal(uintersect1d(id1, id2), id3)
 
             data2 = ds.region(center, left_edge, right_edge)
             data3 = ds.region(center, left_edge, right_edge, data_source=data1)
             id2 = data2['index', 'ID']
             id3 = data3['index', 'ID']
             id3.sort()
-            yield assert_array_equal, uintersect1d(id1, id2), id3
+            assert_array_equal(uintersect1d(id1, id2), id3)
 
             data2 = ds.disk(center, normal, radius, height)
             data3 = ds.disk(center, normal, radius, height, data_source=data1)
             id2 = data2['index', 'ID']
             id3 = data3['index', 'ID']
             id3.sort()
-            yield assert_array_equal, uintersect1d(id1, id2), id3
+            assert_array_equal(uintersect1d(id1, id2), id3)
 
             for d in range(3):
                 data2 = ds.slice(d, center[d])
@@ -131,7 +131,7 @@ def test_compose_overlap():
                 id2 = data2['index', 'ID']
                 id3 = data3['index', 'ID']
                 id3.sort()
-                yield assert_array_equal, uintersect1d(id1, id2), id3
+                assert_array_equal(uintersect1d(id1, id2), id3)
 
             for d in range(3):
                 data2 = ds.ortho_ray(d, center[0:d] + center[d+1:])
@@ -139,11 +139,11 @@ def test_compose_overlap():
                 id2 = data2['index', 'ID']
                 id3 = data3['index', 'ID']
                 id3.sort()
-                yield assert_array_equal, uintersect1d(id1, id2), id3
+                assert_array_equal(uintersect1d(id1, id2), id3)
 
             data2 = ds.point(center)
             data3 = ds.point(center, data_source=data1)
             id2 = data2['index', 'ID']
             id3 = data3['index', 'ID']
             id3.sort()
-            yield assert_array_equal, uintersect1d(id1, id2), id3
+            assert_array_equal(uintersect1d(id1, id2), id3)
