@@ -5,7 +5,8 @@ import shutil
 import unittest
 from yt.data_objects.image_array import ImageArray
 from yt.testing import \
-    assert_equal
+    assert_equal, \
+    requires_module
 
 
 def setup():
@@ -16,8 +17,8 @@ def setup():
 
 def dummy_image(kstep, nlayers):
     im = np.zeros([64, 128, nlayers])
-    for i in xrange(im.shape[0]):
-        for k in xrange(im.shape[2]):
+    for i in range(im.shape[0]):
+        for k in range(im.shape[2]):
             im[i, :, k] = np.linspace(0.0, kstep * k, im.shape[1])
     return im
 
@@ -26,14 +27,14 @@ def test_rgba_rescale():
     im_arr = ImageArray(dummy_image(10.0, 4))
 
     new_im = im_arr.rescale(inline=False)
-    yield assert_equal, im_arr[:, :, :3].max(), 2 * 10.
-    yield assert_equal, im_arr[:, :, 3].max(), 3 * 10.
-    yield assert_equal, new_im[:, :, :3].sum(axis=2).max(), 1.0
-    yield assert_equal, new_im[:, :, 3].max(), 1.0
+    assert_equal(im_arr[:, :, :3].max(), 2 * 10.)
+    assert_equal(im_arr[:, :, 3].max(), 3 * 10.)
+    assert_equal(new_im[:, :, :3].sum(axis=2).max(), 1.0)
+    assert_equal(new_im[:, :, 3].max(), 1.0)
 
     im_arr.rescale()
-    yield assert_equal, im_arr[:, :, :3].sum(axis=2).max(), 1.0
-    yield assert_equal, im_arr[:, :, 3].max(), 1.0
+    assert_equal(im_arr[:, :, :3].sum(axis=2).max(), 1.0)
+    assert_equal(im_arr[:, :, 3].max(), 1.0)
 
 
 class TestImageArray(unittest.TestCase):
@@ -46,27 +47,38 @@ class TestImageArray(unittest.TestCase):
         self.curdir = os.getcwd()
         os.chdir(self.tmpdir)
 
+    def test_image_arry_units(self):
+        im_arr = ImageArray(dummy_image(0.3, 3), input_units='cm')
+
+        assert str(im_arr.units) == 'cm'
+
+        new_im = im_arr.in_units('km')
+
+        assert str(new_im.units) == 'km'
+
+    @requires_module('h5py')
     def test_image_array_hdf5(self):
         myinfo = {'field': 'dinosaurs', 'east_vector': np.array([1., 0., 0.]),
                   'north_vector': np.array([0., 0., 1.]),
                   'normal_vector': np.array([0., 1., 0.]),
-                  'width': 0.245, 'units': 'cm', 'type': 'rendering'}
+                  'width': 0.245, 'type': 'rendering'}
 
-        im_arr = ImageArray(dummy_image(0.3, 3), info=myinfo)
+        im_arr = ImageArray(dummy_image(0.3, 3), input_units='cm', info=myinfo)
         im_arr.save('test_3d_ImageArray')
 
         im = np.zeros([64, 128])
-        for i in xrange(im.shape[0]):
+        for i in range(im.shape[0]):
             im[i, :] = np.linspace(0., 0.3 * 2, im.shape[1])
 
         myinfo = {'field': 'dinosaurs', 'east_vector': np.array([1., 0., 0.]),
                   'north_vector': np.array([0., 0., 1.]),
                   'normal_vector': np.array([0., 1., 0.]),
-                  'width': 0.245, 'units': 'cm', 'type': 'rendering'}
+                  'width': 0.245, 'type': 'rendering'}
 
-        im_arr = ImageArray(im, info=myinfo)
+        im_arr = ImageArray(im, info=myinfo, input_units='cm')
         im_arr.save('test_2d_ImageArray')
 
+
     def test_image_array_rgb_png(self):
         im_arr = ImageArray(dummy_image(10.0, 3))
         im_arr.write_png('standard.png')
