@@ -59,15 +59,15 @@ def setup_angular_momentum(registry, ftype = "gas", slice_info = None):
         rv = data.ds.arr(rv, input_units = data["index", "x"].units)
         return xv * rv[...,1] - yv * rv[...,0]
 
-    registry.add_field((ftype, "specific_angular_momentum_x"),
+    registry.add_field((ftype, "specific_angular_momentum_x"), sampling_type="cell", 
                         function=_specific_angular_momentum_x,
                         units=unit_system["specific_angular_momentum"],
                         validators=[ValidateParameter("center")])
-    registry.add_field((ftype, "specific_angular_momentum_y"),
+    registry.add_field((ftype, "specific_angular_momentum_y"), sampling_type="cell", 
                         function=_specific_angular_momentum_y,
                         units=unit_system["specific_angular_momentum"],
                         validators=[ValidateParameter("center")])
-    registry.add_field((ftype, "specific_angular_momentum_z"),
+    registry.add_field((ftype, "specific_angular_momentum_z"), sampling_type="cell", 
                         function=_specific_angular_momentum_z,
                         units=unit_system["specific_angular_momentum"],
                         validators=[ValidateParameter("center")])
@@ -78,7 +78,7 @@ def setup_angular_momentum(registry, ftype = "gas", slice_info = None):
     def _angular_momentum_x(field, data):
         return data[ftype, "cell_mass"] \
              * data[ftype, "specific_angular_momentum_x"]
-    registry.add_field((ftype, "angular_momentum_x"),
+    registry.add_field((ftype, "angular_momentum_x"), sampling_type="cell", 
                        function=_angular_momentum_x,
                        units=unit_system["angular_momentum"],
                        validators=[ValidateParameter('center')])
@@ -86,7 +86,7 @@ def setup_angular_momentum(registry, ftype = "gas", slice_info = None):
     def _angular_momentum_y(field, data):
         return data[ftype, "cell_mass"] \
              * data[ftype, "specific_angular_momentum_y"]
-    registry.add_field((ftype, "angular_momentum_y"),
+    registry.add_field((ftype, "angular_momentum_y"), sampling_type="cell", 
                        function=_angular_momentum_y,
                        units=unit_system["angular_momentum"],
                        validators=[ValidateParameter('center')])
@@ -94,7 +94,7 @@ def setup_angular_momentum(registry, ftype = "gas", slice_info = None):
     def _angular_momentum_z(field, data):
         return data[ftype, "cell_mass"] \
              * data[ftype, "specific_angular_momentum_z"]
-    registry.add_field((ftype, "angular_momentum_z"),
+    registry.add_field((ftype, "angular_momentum_z"), sampling_type="cell", 
                        function=_angular_momentum_z,
                        units=unit_system["angular_momentum"],
                        validators=[ValidateParameter('center')])
