@@ -13,42 +13,39 @@ Derived field base class.
 
 import contextlib
 import inspect
+import re
+import warnings
 
+from yt.extern.six import string_types, PY2
 from yt.funcs import \
     ensure_list
-from yt.units.yt_array import \
-    YTArray
 from .field_exceptions import \
-    ValidationException, \
     NeedsGridType, \
     NeedsOriginalGrid, \
     NeedsDataField, \
     NeedsProperty, \
     NeedsParameter, \
+    NeedsParameterValue, \
     FieldUnitsError
 from .field_detector import \
     FieldDetector
 from yt.units.unit_object import \
     Unit
+import yt.units.dimensions as ytdims
+from yt.utilities.exceptions import \
+    YTFieldNotFound
 
-def derived_field(**kwargs):
-    def inner_decorator(function):
-        if 'name' not in kwargs:
-            kwargs['name'] = function.__name__
-        kwargs['function'] = function
-        add_field(**kwargs)
-        return function
-    return inner_decorator
 
 def TranslationFunc(field_name):
     def _TranslationFunc(field, data):
         # We do a bunch of in-place modifications, so we will copy this.
         return data[field_name].copy()
+    _TranslationFunc.alias_name = field_name
     return _TranslationFunc
 
 def NullFunc(field, data):
     raise YTFieldNotFound(field.name)
- 
+
 class DerivedField(object):
     """
     This is the base class used to describe a cell-by-cell derived field.
@@ -62,15 +59,23 @@ class DerivedField(object):
        A function handle that defines the field.  Should accept
        arguments (field, data)
     units : str
-       A plain text string encoding the unit.  Powers must be in
-       python syntax (** instead of ^). If set to "auto" the units will be
-       inferred from the units of the return value of the field function.
+       A plain text string encoding the unit, or a query to a unit system of
+       a dataset. Powers must be in python syntax (** instead of ^). If set
+       to "auto" the units will be inferred from the units of the return
+       value of the field function, and the dimensions keyword must also be
+       set (see below).
     take_log : bool
        Describes whether the field should be logged
     validators : list
        A list of :class:`FieldValidator` objects
+    sampling_type : string, default = "cell"
+        How is the field sampled?  This can be one of the following options at
+        present: "cell" (cell-centered), "discrete" (or "particle") for
+        discretely sampled data.
     particle_type : bool
-       Is this a particle (1D) field?
+       (Deprecated) Is this a particle (1D) field?  This is deprecated. Use
+       sampling_type = "discrete" or sampling_type = "particle".  This will
+       *override* sampling_type.
     vector_field : bool
        Describes the dimensionality of the field.  Currently unused.
     display_field : bool
@@ -83,18 +88,42 @@ class DerivedField(object):
        For fields that exist on disk, which we may want to convert to other
        fields or that get aliased to themselves, we can specify a different
        desired output unit than the unit found on disk.
+    dimensions : str or object from yt.units.dimensions
+       The dimensions of the field, only needed if units="auto" and only used
+       for error checking.
+    nodal_flag : array-like with three components
+       This describes how the field is centered within a cell. If nodal_flag
+       is [0, 0, 0], then the field is cell-centered. If any of the components
+       of nodal_flag are 1, then the field is nodal in that direction, meaning
+       it is defined at the lo and hi sides of the cell rather than at the center.
+       For example, a field with nodal_flag = [1, 0, 0] would be defined at the
+       middle of the 2 x-faces of each cell. nodal_flag = [0, 1, 1] would mean the
+       that the field defined at the centers of the 4 edges that are normal to the
+       x axis, while nodal_flag = [1, 1, 1] would be defined at the 8 cell corners.
     """
-    def __init__(self, name, function, units=None,
+    def __init__(self, name, sampling_type, function, units=None,
                  take_log=True, validators=None,
-                 particle_type=False, vector_field=False, display_field=True,
-                 not_in_all=False, display_name=None, output_units = None):
+                 particle_type=None, vector_field=False, display_field=True,
+                 not_in_all=False, display_name=None, output_units=None,
+                 dimensions=None, ds=None, nodal_flag=None):
         self.name = name
         self.take_log = take_log
         self.display_name = display_name
         self.not_in_all = not_in_all
         self.display_field = display_field
-        self.particle_type = particle_type
+        if particle_type is True:
+            warnings.warn("particle_type for derived fields "
+                          "has been replaced with sampling_type = 'particle'",
+                          DeprecationWarning)
+            sampling_type = "particle"
+        self.sampling_type = sampling_type
         self.vector_field = vector_field
+        self.ds = ds
+
+        if nodal_flag is None:
+            self.nodal_flag = [0, 0, 0]
+        else:
+            self.nodal_flag = nodal_flag
 
         self._function = function
 
@@ -106,13 +135,18 @@ class DerivedField(object):
         # handle units
         if units is None:
             self.units = ''
-        elif isinstance(units, str):
+        elif isinstance(units, string_types):
             if units.lower() == 'auto':
+                if dimensions is None:
+                    raise RuntimeError("To set units='auto', please specify the dimensions "
+                                       "of the field with dimensions=<dimensions of field>!")
                 self.units = None
             else:
                 self.units = units
         elif isinstance(units, Unit):
             self.units = str(units)
+        elif isinstance(units, bytes):
+            self.units = units.decode("utf-8")
         else:
             raise FieldUnitsError("Cannot handle units '%s' (type %s)." \
                                   "Please provide a string or Unit " \
@@ -121,26 +155,40 @@ class DerivedField(object):
             output_units = self.units
         self.output_units = output_units
 
+        if isinstance(dimensions, string_types):
+            dimensions = getattr(ytdims, dimensions)
+        self.dimensions = dimensions
+
     def _copy_def(self):
         dd = {}
         dd['name'] = self.name
         dd['units'] = self.units
         dd['take_log'] = self.take_log
         dd['validators'] = list(self.validators)
-        dd['particle_type'] = self.particle_type
+        dd['sampling_type'] = self.sampling_type
         dd['vector_field'] = self.vector_field
         dd['display_field'] = True
         dd['not_in_all'] = self.not_in_all
         dd['display_name'] = self.display_name
         return dd
 
+    @property
+    def particle_type(self):
+        return self.sampling_type in ("discrete", "particle")
+
     def get_units(self):
-        u = Unit(self.units)
+        if self.ds is not None:
+            u = Unit(self.units, registry=self.ds.unit_registry)
+        else:
+            u = Unit(self.units)
         return u.latex_representation()
 
     def get_projected_units(self):
-        u = Unit(self.units)*Unit('cm')
-        return u.latex_representation()
+        if self.ds is not None:
+            u = Unit(self.units, registry=self.ds.unit_registry)
+        else:
+            u = Unit(self.units)
+        return (u*Unit('cm')).latex_representation()
 
     def check_available(self, data):
         """
@@ -179,7 +227,7 @@ class DerivedField(object):
 
     def __call__(self, data):
         """ Return the value of the field in a given *data* object. """
-        ii = self.check_available(data)
+        self.check_available(data)
         original_fields = data.keys() # Copy
         if self._function is NullFunc:
             raise RuntimeError(
@@ -213,7 +261,10 @@ class DerivedField(object):
         if projected:
             raise NotImplementedError
         else:
-            units = Unit(self.units)
+            if self.ds is not None:
+                units = Unit(self.units, registry=self.ds.unit_registry)
+            else:
+                units = Unit(self.units)
         # Add unit label
         if not units.is_dimensionless:
             data_label += r"\ \ (%s)" % (units.latex_representation())
@@ -221,18 +272,93 @@ class DerivedField(object):
         data_label += r"$"
         return data_label
 
+    def __repr__(self):
+        if PY2:
+            func_name = self._function.func_name
+        else:
+            func_name = self._function.__name__
+        if self._function == NullFunc:
+            s = "On-Disk Field "
+        elif func_name == "_TranslationFunc":
+            s = "Alias Field for \"%s\" " % (self._function.alias_name,)
+        else:
+            s = "Derived Field "
+        if isinstance(self.name, tuple):
+            s += "(%s, %s): " % self.name
+        else:
+            s += "%s: " % (self.name)
+        s += "(units: %s" % self.units
+        if self.display_name is not None:
+            s += ", display_name: '%s'" % (self.display_name)
+        if self.particle_type:
+            s += ", particle field"
+        s += ")"
+        return s
+
+    def _is_ion(self):
+        p = re.compile("_p[0-9]+_")
+        result = False
+        if p.search(self.name[1]) is not None:
+            result = True
+        return result
+
+    def _ion_to_label(self):
+        pnum2rom = {
+            "0":"I", "1":"II", "2":"III", "3":"IV", "4":"V",
+            "5":"VI", "6":"VII", "7":"VIII", "8":"IX", "9":"X",
+            "10":"XI", "11":"XII", "12":"XIII", "13":"XIV", "14":"XV",
+            "15":"XVI", "16":"XVII", "17":"XVIII", "18":"XIX", "19":"XX"}
+
+        p = re.compile("_p[0-9]+_")
+        m = p.search(self.name[1])
+        if m is not None:
+            pstr = m.string[m.start()+1:m.end()-1]
+            segments = self.name[1].split("_")
+            for i,s in enumerate(segments):
+                segments[i] = s.capitalize()
+                if s == pstr:
+                    ipstr = i
+            element = segments[ipstr-1]
+            roman = pnum2rom[pstr[1:]]
+            label = element + '\ ' + roman + '\ ' + \
+                '\ '.join(segments[ipstr+1:])
+        else:
+            label = self.name[1]
+        return label
+
+    def get_latex_display_name(self):
+        label = self.display_name
+        if label is None:
+            if self._is_ion():
+                fname = self._ion_to_label()
+                label = r'$\rm{'+fname.replace('_','\ ')+r'}$'
+            else:
+                label = r'$\rm{'+self.name[1].replace('_','\ ').title()+r'}$'
+        elif label.find('$') == -1:
+            label = label.replace(' ','\ ')
+            label = r'$\rm{'+label+r'}$'
+        return label
+
+
 class FieldValidator(object):
     pass
 
 class ValidateParameter(FieldValidator):
-    def __init__(self, parameters):
+    def __init__(self, parameters, parameter_values=None):
         """
         This validator ensures that the dataset has a given parameter.
+
+        If *parameter_values* is supplied, this will also ensure that the field
+        is available for all permutations of the field parameter.
         """
         FieldValidator.__init__(self)
         self.parameters = ensure_list(parameters)
+        self.parameter_values = parameter_values
     def __call__(self, data):
         doesnt_have = []
+        if self.parameter_values is not None:
+            if isinstance(data, FieldDetector):
+                raise NeedsParameterValue(self.parameter_values)
         for p in self.parameters:
             if not data.has_field_parameter(p):
                 doesnt_have.append(p)
@@ -283,6 +409,7 @@ class ValidateSpatial(FieldValidator):
         FieldValidator.__init__(self)
         self.ghost_zones = ghost_zones
         self.fields = fields
+
     def __call__(self, data):
         # When we say spatial information, we really mean
         # that it has a three-dimensional data structure
