@@ -59,14 +59,14 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
     def _cell_mass(field, data):
         return data[ftype, "density"] * data[ftype, "cell_volume"]
 
-    registry.add_field((ftype, "cell_mass"),
+    registry.add_field((ftype, "cell_mass"), sampling_type="cell", 
         function=_cell_mass,
         units=unit_system["mass"])
 
     def _sound_speed(field, data):
         tr = data.ds.gamma * data[ftype, "pressure"] / data[ftype, "density"]
         return np.sqrt(tr)
-    registry.add_field((ftype, "sound_speed"),
+    registry.add_field((ftype, "sound_speed"), sampling_type="cell", 
              function=_sound_speed,
              units=unit_system["velocity"])
 
@@ -74,7 +74,7 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
         """ Radial component of M{|v|/c_sound} """
         tr = data[ftype, "radial_velocity"] / data[ftype, "sound_speed"]
         return np.abs(tr)
-    registry.add_field((ftype, "radial_mach_number"),
+    registry.add_field((ftype, "radial_mach_number"), sampling_type="cell", 
              function=_radial_mach_number,
              units = "")
 
@@ -82,14 +82,14 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
         return 0.5*data[ftype, "density"] * ( data[ftype, "velocity_x"]**2.0
                                               + data[ftype, "velocity_y"]**2.0
                                               + data[ftype, "velocity_z"]**2.0 )
-    registry.add_field((ftype, "kinetic_energy"),
+    registry.add_field((ftype, "kinetic_energy"), sampling_type="cell", 
                        function = _kin_energy,
                        units = unit_system["pressure"])
 
     def _mach_number(field, data):
         """ M{|v|/c_sound} """
         return data[ftype, "velocity_magnitude"] / data[ftype, "sound_speed"]
-    registry.add_field((ftype, "mach_number"),
+    registry.add_field((ftype, "mach_number"), sampling_type="cell", 
             function=_mach_number,
             units = "")
 
@@ -103,7 +103,7 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
         tr = np.minimum(np.minimum(t1, t2), t3)
         return tr
 
-    registry.add_field((ftype, "courant_time_step"),
+    registry.add_field((ftype, "courant_time_step"), sampling_type="cell", 
              function=_courant_time_step,
              units=unit_system["time"])
 
@@ -113,13 +113,13 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
            * (data[ftype, "density"] * data[ftype, "thermal_energy"])
         return tr
 
-    registry.add_field((ftype, "pressure"),
+    registry.add_field((ftype, "pressure"), sampling_type="cell", 
              function=_pressure,
              units=unit_system["pressure"])
 
     def _kT(field, data):
         return (kboltz*data[ftype, "temperature"]).in_units("keV")
-    registry.add_field((ftype, "kT"),
+    registry.add_field((ftype, "kT"), sampling_type="cell", 
                        function=_kT,
                        units="keV",
                        display_name="Temperature")
@@ -132,7 +132,7 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
         gammam1 = 2./3.
         tr = data[ftype,"kT"] / ((data[ftype, "density"]/mw)**gammam1)
         return data.apply_units(tr, field.units)
-    registry.add_field((ftype, "entropy"),
+    registry.add_field((ftype, "entropy"), sampling_type="cell", 
              units="keV*cm**2",
              function=_entropy)
 
@@ -140,13 +140,13 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
         tr = data[ftype, "metal_density"] / data[ftype, "density"]
         tr /= metallicity_sun
         return data.apply_units(tr, "Zsun")
-    registry.add_field((ftype, "metallicity"),
+    registry.add_field((ftype, "metallicity"), sampling_type="cell", 
              function=_metallicity,
              units="Zsun")
 
     def _metal_mass(field, data):
         return data[ftype, "metal_density"] * data[ftype, "cell_volume"]
-    registry.add_field((ftype, "metal_mass"),
+    registry.add_field((ftype, "metal_mass"), sampling_type="cell", 
                        function=_metal_mass,
                        units=unit_system["mass"])
 
@@ -156,13 +156,13 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
         for species in data.ds.field_info.species_names:
             field_data += data["gas", "%s_number_density" % species]
         return field_data
-    registry.add_field((ftype, "number_density"),
+    registry.add_field((ftype, "number_density"), sampling_type="cell", 
                        function = _number_density,
                        units=unit_system["number_density"])
     
     def _mean_molecular_weight(field, data):
         return (data[ftype, "density"] / (mh * data[ftype, "number_density"]))
-    registry.add_field((ftype, "mean_molecular_weight"),
+    registry.add_field((ftype, "mean_molecular_weight"), sampling_type="cell", 
               function=_mean_molecular_weight,
               units="")
 
@@ -207,7 +207,7 @@ def setup_gradient_fields(registry, grad_field, field_units, slice_info = None):
 
     for axi, ax in enumerate('xyz'):
         f = grad_func(axi, ax)
-        registry.add_field((ftype, "%s_gradient_%s" % (fname, ax)),
+        registry.add_field((ftype, "%s_gradient_%s" % (fname, ax)), sampling_type="cell", 
                            function = f,
                            validators = [ValidateSpatial(1, [grad_field])],
                            units = grad_units)
