@@ -12,6 +12,7 @@ This is a container for storing local fields defined on each load of yt.
 #
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
+import warnings
 
 from yt.utilities.logger import \
     ytLogger as mylog
@@ -23,7 +24,7 @@ from .field_info_container import \
     FieldInfoContainer
 
 class LocalFieldInfoContainer(FieldInfoContainer):
-    def add_field(self, name, function=None, **kwargs):
+    def add_field(self, name, function=None, sampling_type=None, **kwargs):
         if not isinstance(name, tuple):
             if kwargs.setdefault('particle_type', False):
                 name = ('all', name)
@@ -34,8 +35,19 @@ class LocalFieldInfoContainer(FieldInfoContainer):
         if not override and name in self:
             mylog.warning("Field %s already exists. To override use " +
                           "force_override=True.", name)
+        if kwargs.setdefault('particle_type', False):
+            if sampling_type is not None and sampling_type != "particle":
+                raise RuntimeError("Clashing definition of 'sampling_type' and "
+                               "'particle_type'. Note that 'particle_type' is "
+                               "deprecated. Please just use 'sampling_type'.")
+            else:
+                sampling_type = "particle"
+        if sampling_type is None:
+            warnings.warn("Because 'sampling_type' not specified, yt will "
+                          "assume a cell 'sampling_type'")
+            sampling_type = "cell"
         return super(LocalFieldInfoContainer,
-                     self).add_field(name, function, **kwargs)
+                     self).add_field(name, sampling_type, function, **kwargs)
 
 # Empty FieldInfoContainer
 local_fields = LocalFieldInfoContainer(None, [], None)
