@@ -16,8 +16,6 @@ These are common particle fields.
 
 import numpy as np
 
-from yt.funcs import *
-from yt.units.yt_array import YTArray
 from yt.fields.derived_field import \
     ValidateParameter, \
     ValidateSpatial
@@ -95,6 +93,7 @@ def _field_concat_slice(fname, axi):
     return _AllFields
 
 def particle_deposition_functions(ptype, coord_name, mass_name, registry):
+    unit_system = registry.ds.unit_system
     orig = set(registry.keys())
     ptype_dn = ptype.replace("_"," ").title()
     def particle_count(field, data):
@@ -104,6 +103,7 @@ def particle_deposition_functions(ptype, coord_name, mass_name, registry):
         return data.apply_units(d, field.units)
 
     registry.add_field(("deposit", "%s_count" % ptype),
+             sampling_type="cell",
              function = particle_count,
              validators = [ValidateSpatial()],
              units = '',
@@ -117,24 +117,26 @@ def particle_deposition_functions(ptype, coord_name, mass_name, registry):
         return data.apply_units(d, field.units)
 
     registry.add_field(("deposit", "%s_mass" % ptype),
+             sampling_type="cell",
              function = particle_mass,
              validators = [ValidateSpatial()],
              display_name = r"\mathrm{%s Mass}" % ptype_dn,
-             units = "g")
+             units = unit_system["mass"])
 
     def particle_density(field, data):
         pos = data[ptype, coord_name].convert_to_units("code_length")
         mass = data[ptype, mass_name].convert_to_units("code_mass")
-        d = data.deposit(pos, [data[ptype, mass_name]], method = "sum")
+        d = data.deposit(pos, [mass], method = "sum")
         d = data.ds.arr(d, "code_mass")
         d /= data["index", "cell_volume"]
         return d
 
     registry.add_field(("deposit", "%s_density" % ptype),
+             sampling_type="cell",
              function = particle_density,
              validators = [ValidateSpatial()],
              display_name = r"\mathrm{%s Density}" % ptype_dn,
-             units = "g/cm**3")
+             units = unit_system["density"])
 
     def particle_cic(field, data):
         pos = data[ptype, coord_name]
@@ -144,10 +146,11 @@ def particle_deposition_functions(ptype, coord_name, mass_name, registry):
         return d
 
     registry.add_field(("deposit", "%s_cic" % ptype),
+             sampling_type="cell",
              function = particle_cic,
              validators = [ValidateSpatial()],
              display_name = r"\mathrm{%s CIC Density}" % ptype_dn,
-             units = "g/cm**3")
+             units = unit_system["density"])
 
     def _get_density_weighted_deposit_field(fname, units, method):
         def _deposit_field(field, data):
@@ -172,19 +175,27 @@ def particle_deposition_functions(ptype, coord_name, mass_name, registry):
             function = _get_density_weighted_deposit_field(
                 "particle_velocity_%s" % ax, "cm/s", method)
             registry.add_field(
-                ("deposit", ("%s_"+name+"_velocity_%s") % (ptype, ax)),
-                function=function, units="cm/s", take_log=False,
+                ("deposit", ("%s_"+name+"_velocity_%s") % (ptype, ax)), sampling_type="cell",
+                function=function, units=unit_system["velocity"], take_log=False,
                 validators=[ValidateSpatial(0)])
 
+    for method, name in zip(("cic", "sum"), ("cic", "nn")):
+        function = _get_density_weighted_deposit_field(
+            "age", "s", method)
+        registry.add_field(
+            ("deposit", ("%s_"+name+"_age") % (ptype)), sampling_type="cell",
+            function=function, units=unit_system["time"], take_log=False,
+            validators=[ValidateSpatial(0)])
+
     # Now some translation functions.
 
     def particle_ones(field, data):
-        v = np.ones(data[ptype, mass_name].shape, dtype="float64")
+        v = np.ones(data[ptype, coord_name].shape[0], dtype="float64")
         return data.apply_units(v, field.units)
 
     registry.add_field((ptype, "particle_ones"),
+                       sampling_type="particle",
                        function = particle_ones,
-                       particle_type = True,
                        units = "",
                        display_name = r"Particle Count")
 
@@ -197,10 +208,10 @@ def particle_deposition_functions(ptype, coord_name, mass_name, registry):
         data.deposit(pos, [ids], method = "mesh_id")
         return data.apply_units(ids, "")
     registry.add_field((ptype, "mesh_id"),
+            sampling_type="particle",
             function = particle_mesh_ids,
             validators = [ValidateSpatial()],
-            units = '',
-            particle_type = True)
+            units = '')
 
     return list(set(registry.keys()).difference(orig))
 
@@ -221,14 +232,14 @@ def particle_scalar_functions(ptype, coord_name, vel_name, registry):
     for axi, ax in enumerate("xyz"):
         v, p = _get_coord_funcs(axi, ptype)
         registry.add_field((ptype, "particle_velocity_%s" % ax),
-            particle_type = True, function = v,
-            units = "code_velocity")
+            sampling_type="particle", function = v, units = "code_velocity")
         registry.add_field((ptype, "particle_position_%s" % ax),
-            particle_type = True, function = p,
-            units = "code_length")
+            sampling_type="particle", function = p, units = "code_length")
 
 def particle_vector_functions(ptype, coord_names, vel_names, registry):
 
+    unit_system = registry.ds.unit_system
+
     # This will column_stack a set of scalars to create vector fields.
 
     def _get_vec_func(_ptype, names):
@@ -239,13 +250,13 @@ def particle_vector_functions(ptype, coord_names, vel_names, registry):
             return data.apply_units(c, field.units)
         return particle_vectors
     registry.add_field((ptype, "particle_position"),
+                       sampling_type="particle",
                        function=_get_vec_func(ptype, coord_names),
-                       units = "code_length",
-                       particle_type=True)
+                       units = "code_length")
     registry.add_field((ptype, "particle_velocity"),
+                       sampling_type="particle",
                        function=_get_vec_func(ptype, vel_names),
-                       units = "cm / s",
-                       particle_type=True)
+                       units = unit_system["velocity"])
 
 def get_angular_momentum_components(ptype, data, spos, svel):
     if data.has_field_parameter("normal"):
@@ -260,6 +271,7 @@ def get_angular_momentum_components(ptype, data, spos, svel):
 def standard_particle_fields(registry, ptype,
                              spos = "particle_position_%s",
                              svel = "particle_velocity_%s"):
+    unit_system = registry.ds.unit_system
     # This function will set things up based on the scalar fields and standard
     # yt field names.
     # data.get_field_parameter("bulk_velocity") defaults to YTArray([0,0,0] cm/s)
@@ -272,10 +284,10 @@ def standard_particle_fields(registry, ptype,
                      + (data[ptype, svel % 'z'] - bulk_velocity[2])**2 )
 
     registry.add_field((ptype, "particle_velocity_magnitude"),
+                  sampling_type="particle",
                   function=_particle_velocity_magnitude,
-                  particle_type=True,
                   take_log=False,
-                  units="cm/s")
+                  units=unit_system["velocity"])
 
     def _particle_specific_angular_momentum(field, data):
         """
@@ -291,9 +303,9 @@ def standard_particle_fields(registry, ptype,
 
 
     registry.add_field((ptype, "particle_specific_angular_momentum"),
+              sampling_type="particle",
               function=_particle_specific_angular_momentum,
-              particle_type=True,
-              units="cm**2/s",
+              units=unit_system["specific_angular_momentum"],
               validators=[ValidateParameter("center")])
 
     def _get_spec_ang_mom_comp(axi, ax, _ptype):
@@ -308,11 +320,11 @@ def standard_particle_fields(registry, ptype,
         f, v = _get_spec_ang_mom_comp(axi, ax, ptype)
         registry.add_field(
             (ptype, "particle_specific_angular_momentum_%s" % ax),
-            particle_type = True, function=f, units="cm**2/s",
+            sampling_type="particle", function=f, units=unit_system["specific_angular_momentum"],
             validators=[ValidateParameter("center")]
         )
         registry.add_field((ptype, "particle_angular_momentum_%s" % ax),
-            function=v, units="g*cm**2/s", particle_type=True,
+            sampling_type="particle", function=v, units=unit_system["angular_momentum"],
             validators=[ValidateParameter('center')])
 
     def _particle_angular_momentum(field, data):
@@ -320,13 +332,14 @@ def standard_particle_fields(registry, ptype,
         return am.T
 
     registry.add_field((ptype, "particle_angular_momentum"),
+              sampling_type="particle",
               function=_particle_angular_momentum,
-              particle_type=True,
-              units="g*cm**2/s",
+              units=unit_system["angular_momentum"],
               validators=[ValidateParameter("center")])
 
     create_magnitude_field(registry, "particle_angular_momentum",
-                           "g*cm**2/s", ftype=ptype, particle_type=True)
+                           unit_system["angular_momentum"],
+                           ftype=ptype, particle_type=True)
 
     def _particle_radius(field, data):
         """The spherical radius component of the particle positions
@@ -334,13 +347,13 @@ def standard_particle_fields(registry, ptype,
         Relative to the coordinate system defined by the *normal* vector,
         and *center* field parameters.
         """
-        return get_radius(data, "particle_position_")
+        return get_radius(data, "particle_position_", field.name[0])
 
     registry.add_field(
         (ptype, "particle_radius"),
+        sampling_type="particle",
         function=_particle_radius,
-        units="cm",
-        particle_type=True,
+        units=unit_system["length"],
         validators=[ValidateParameter("center")])
 
     def _particle_position_relative(field, data):
@@ -359,9 +372,9 @@ def standard_particle_fields(registry, ptype,
 
     registry.add_field(
         (ptype, "particle_position_relative"),
+        sampling_type="particle",
         function=_particle_position_relative,
-        particle_type=True,
-        units="cm",
+        units=unit_system["length"],
         validators=[ValidateParameter("normal"), ValidateParameter("center")])
 
     def _particle_velocity_relative(field, data):
@@ -380,8 +393,9 @@ def standard_particle_fields(registry, ptype,
         return vel
 
     registry.add_field((ptype, "particle_velocity_relative"),
+              sampling_type="particle",
               function=_particle_velocity_relative,
-              particle_type=True, units="cm/s",
+              units=unit_system["velocity"],
               validators=[ValidateParameter("normal"),
                           ValidateParameter("center")])
 
@@ -395,18 +409,17 @@ def standard_particle_fields(registry, ptype,
     for axi, ax in enumerate("xyz"):
         v, p = _get_coord_funcs_relative(axi, ptype)
         registry.add_field((ptype, "particle_velocity_relative_%s" % ax),
-            particle_type = True, function = v,
-            units = "code_velocity")
+            sampling_type="particle", function = v, units = "code_velocity")
         registry.add_field((ptype, "particle_position_relative_%s" % ax),
-            particle_type = True, function = p,
-            units = "code_length")
+            sampling_type="particle", function = p, units = "code_length")
 
 
     # this is just particle radius but we add it with an alias for the sake of
     # consistent naming
     registry.add_field((ptype, "particle_position_spherical_radius"),
+              sampling_type="particle",
               function=_particle_radius,
-              particle_type=True, units="cm",
+              units=unit_system["length"],
               validators=[ValidateParameter("normal"),
                           ValidateParameter("center")])
 
@@ -415,8 +428,9 @@ def standard_particle_fields(registry, ptype,
         return data[ptype, 'particle_position_spherical_radius']
 
     registry.add_field((ptype, "particle_spherical_position_radius"),
+              sampling_type="particle",
               function=_particle_spherical_position_radius,
-              particle_type=True, units="cm",
+              units=unit_system["length"],
               validators=[ValidateParameter("normal"),
                           ValidateParameter("center")])
 
@@ -434,8 +448,8 @@ def standard_particle_fields(registry, ptype,
 
     registry.add_field(
         (ptype, "particle_position_spherical_theta"),
+        sampling_type="particle",
         function=_particle_position_spherical_theta,
-        particle_type=True,
         units="",
         validators=[ValidateParameter("center"), ValidateParameter("normal")])
 
@@ -444,8 +458,9 @@ def standard_particle_fields(registry, ptype,
         return data[ptype, 'particle_position_spherical_theta']
 
     registry.add_field((ptype, "particle_spherical_position_theta"),
+              sampling_type="particle",
               function=_particle_spherical_position_theta,
-              particle_type=True, units="",
+              units="",
               validators=[ValidateParameter("normal"),
                           ValidateParameter("center")])
 
@@ -463,8 +478,8 @@ def standard_particle_fields(registry, ptype,
 
     registry.add_field(
         (ptype, "particle_position_spherical_phi"),
+        sampling_type="particle",
         function=_particle_position_spherical_phi,
-        particle_type=True,
         units="",
         validators=[ValidateParameter("normal"), ValidateParameter("center")])
 
@@ -473,8 +488,9 @@ def standard_particle_fields(registry, ptype,
         return data[ptype, 'particle_position_spherical_phi']
 
     registry.add_field((ptype, "particle_spherical_position_phi"),
+             sampling_type="particle",
              function=_particle_spherical_position_phi,
-             particle_type=True, units="",
+             units="",
              validators=[ValidateParameter("center"),
                          ValidateParameter("normal")])
 
@@ -490,16 +506,17 @@ def standard_particle_fields(registry, ptype,
         bv = data.get_field_parameter("bulk_velocity")
         pos = data.ds.arr([data[ptype, spos % ax] for ax in "xyz"])
         vel = data.ds.arr([data[ptype, svel % ax] for ax in "xyz"])
-        theta = get_sph_theta(pos, normal)
-        phi = get_sph_phi(pos, normal)
         pos = pos - np.reshape(center, (3, 1))
         vel = vel - np.reshape(bv, (3, 1))
+        theta = get_sph_theta(pos, normal)
+        phi = get_sph_phi(pos, normal)
         sphr = get_sph_r_component(vel, theta, phi, normal)
         return sphr
 
     registry.add_field((ptype, "particle_velocity_spherical_radius"),
+              sampling_type="particle",
               function=_particle_velocity_spherical_radius,
-              particle_type=True, units="cm/s",
+              units=unit_system["velocity"],
               validators=[ValidateParameter("normal"),
                           ValidateParameter("center")])
 
@@ -508,16 +525,18 @@ def standard_particle_fields(registry, ptype,
         return data[ptype, 'particle_velocity_spherical_radius']
 
     registry.add_field((ptype, "particle_spherical_velocity_radius"),
+              sampling_type="particle",
               function=_particle_spherical_velocity_radius,
-              particle_type=True, units="cm/s",
+              units=unit_system["velocity"],
               validators=[ValidateParameter("normal"),
                           ValidateParameter("center")])
 
     # particel_velocity_spherical_radius is simply aliased to
     # "particle_radial_velocity" for convenience
     registry.add_field((ptype, "particle_radial_velocity"),
+              sampling_type="particle",
               function=_particle_spherical_velocity_radius,
-              particle_type=True, units="cm/s",
+              units=unit_system["velocity"],
               validators=[ValidateParameter("normal"),
                           ValidateParameter("center")])
 
@@ -533,18 +552,18 @@ def standard_particle_fields(registry, ptype,
         bv = data.get_field_parameter("bulk_velocity")
         pos = data.ds.arr([data[ptype, spos % ax] for ax in "xyz"])
         vel = data.ds.arr([data[ptype, svel % ax] for ax in "xyz"])
-        theta = get_sph_theta(pos, normal)
-        phi = get_sph_phi(pos, normal)
         pos = pos - np.reshape(center, (3, 1))
         vel = vel - np.reshape(bv, (3, 1))
+        theta = get_sph_theta(pos, normal)
+        phi = get_sph_phi(pos, normal)
         spht = get_sph_theta_component(vel, theta, phi, normal)
         return spht
 
     registry.add_field(
         (ptype, "particle_velocity_spherical_theta"),
+        sampling_type="particle",
         function=_particle_velocity_spherical_theta,
-        particle_type=True,
-        units="cm/s",
+        units=unit_system["velocity"],
         validators=[ValidateParameter("normal"), ValidateParameter("center")])
 
     def _particle_spherical_velocity_theta(field, data):
@@ -552,8 +571,9 @@ def standard_particle_fields(registry, ptype,
         return data[ptype, 'particle_velocity_spherical_theta']
 
     registry.add_field((ptype, "particle_spherical_velocity_theta"),
+              sampling_type="particle",
               function=_particle_spherical_velocity_theta,
-              particle_type=True, units="cm/s",
+              units=unit_system["velocity"],
               validators=[ValidateParameter("normal"),
                           ValidateParameter("center")])
 
@@ -576,9 +596,9 @@ def standard_particle_fields(registry, ptype,
 
     registry.add_field(
         (ptype, "particle_velocity_spherical_phi"),
+        sampling_type="particle",
         function=_particle_velocity_spherical_phi,
-        particle_type=True,
-        units="cm/s",
+        units=unit_system["velocity"],
         validators=[ValidateParameter("normal"), ValidateParameter("center")])
 
     def _particle_spherical_velocity_phi(field, data):
@@ -586,8 +606,9 @@ def standard_particle_fields(registry, ptype,
         return data[ptype, 'particle_spherical_velocity_theta']
 
     registry.add_field((ptype, "particle_spherical_velocity_phi"),
+              sampling_type="particle",
               function=_particle_spherical_velocity_phi,
-              particle_type=True, units="cm/s",
+              units=unit_system["velocity"],
               validators=[ValidateParameter("normal"),
                           ValidateParameter("center")])
 
@@ -606,9 +627,9 @@ def standard_particle_fields(registry, ptype,
 
     registry.add_field(
         (ptype, "particle_position_cylindrical_radius"),
+        sampling_type="particle",
         function=_particle_position_cylindrical_radius,
-        units="cm",
-        particle_type=True,
+        units=unit_system["length"],
         validators=[ValidateParameter("normal"), ValidateParameter("center")])
 
     def _particle_position_cylindrical_theta(field,data):
@@ -625,8 +646,8 @@ def standard_particle_fields(registry, ptype,
 
     registry.add_field(
         (ptype, "particle_position_cylindrical_theta"),
+        sampling_type="particle",
         function=_particle_position_cylindrical_theta,
-        particle_type=True,
         units="",
         validators=[ValidateParameter("center"), ValidateParameter("normal")])
 
@@ -645,9 +666,9 @@ def standard_particle_fields(registry, ptype,
 
     registry.add_field(
         (ptype, "particle_position_cylindrical_z"),
+        sampling_type="particle",
         function=_particle_position_cylindrical_z,
-        units="cm",
-        particle_type=True,
+        units=unit_system["length"],
         validators=[ValidateParameter("normal"), ValidateParameter("center")])
 
     def _particle_velocity_cylindrical_radius(field, data):
@@ -661,17 +682,17 @@ def standard_particle_fields(registry, ptype,
         bv = data.get_field_parameter("bulk_velocity")
         pos = data.ds.arr([data[ptype, spos % ax] for ax in "xyz"])
         vel = data.ds.arr([data[ptype, svel % ax] for ax in "xyz"])
-        theta = get_cyl_theta(pos, normal)
         pos = pos - np.reshape(center, (3, 1))
         vel = vel - np.reshape(bv, (3, 1))
+        theta = get_cyl_theta(pos, normal)
         cylr = get_cyl_r_component(vel, theta, normal)
         return cylr
 
     registry.add_field(
         (ptype, "particle_velocity_cylindrical_radius"),
+        sampling_type="particle",
         function=_particle_velocity_cylindrical_radius,
-        particle_type=True,
-        units="cm/s",
+        units=unit_system["velocity"],
         validators=[ValidateParameter("normal"), ValidateParameter("center")])
 
     def _particle_velocity_cylindrical_theta(field, data):
@@ -685,17 +706,17 @@ def standard_particle_fields(registry, ptype,
         bv = data.get_field_parameter("bulk_velocity")
         pos = data.ds.arr([data[ptype, spos % ax] for ax in "xyz"])
         vel = data.ds.arr([data[ptype, svel % ax] for ax in "xyz"])
-        theta = get_cyl_theta(pos, normal)
         pos = pos - np.reshape(center, (3, 1))
         vel = vel - np.reshape(bv, (3, 1))
+        theta = get_cyl_theta(pos, normal)
         cylt = get_cyl_theta_component(vel, theta, normal)
         return cylt
 
     registry.add_field(
         (ptype, "particle_velocity_cylindrical_theta"),
+        sampling_type="particle",
         function=_particle_velocity_cylindrical_theta,
-        particle_type=True,
-        units="cm/s",
+        units=unit_system["velocity"],
         validators=[ValidateParameter("normal"), ValidateParameter("center")])
 
     def _particle_cylindrical_velocity_theta(field, data):
@@ -703,8 +724,9 @@ def standard_particle_fields(registry, ptype,
         return data[ptype, 'particle_velocity_cylindrical_theta']
 
     registry.add_field((ptype, "particle_cylindrical_velocity_theta"),
+              sampling_type="particle",
               function=_particle_cylindrical_velocity_theta,
-              particle_type=True, units="cm/s",
+              units="cm/s",
               validators=[ValidateParameter("normal"),
                           ValidateParameter("center")])
 
@@ -726,9 +748,9 @@ def standard_particle_fields(registry, ptype,
 
     registry.add_field(
         (ptype, "particle_velocity_cylindrical_z"),
+        sampling_type="particle",
         function=_particle_velocity_cylindrical_z,
-        particle_type=True,
-        units="cm/s",
+        units=unit_system["velocity"],
         validators=[ValidateParameter("normal"), ValidateParameter("center")])
 
     def _particle_cylindrical_velocity_z(field, data):
@@ -736,8 +758,9 @@ def standard_particle_fields(registry, ptype,
         return data[ptype, "particle_velocity_cylindrical_z"]
 
     registry.add_field((ptype, "particle_cylindrical_velocity_z"),
+              sampling_type="particle",
               function=_particle_cylindrical_velocity_z,
-              particle_type=True, units="cm/s",
+              units=unit_system["velocity"],
               validators=[ValidateParameter("normal"),
                           ValidateParameter("center")])
 
@@ -758,24 +781,29 @@ def add_particle_average(registry, ptype, field_name,
         v[np.isnan(v)] = 0.0
         return v
     fn = ("deposit", "%s_avg_%s" % (ptype, field_name))
-    registry.add_field(fn, function=_pfunc_avg,
+    registry.add_field(fn, sampling_type="cell", function=_pfunc_avg,
                        validators = [ValidateSpatial(0)],
-                       particle_type = False,
                        units = field_units)
     return fn
 
 def add_volume_weighted_smoothed_field(ptype, coord_name, mass_name,
         smoothing_length_name, density_name, smoothed_field, registry,
-        nneighbors = None):
-    field_name = ("deposit", "%s_smoothed_%s" % (ptype, smoothed_field))
+        nneighbors = 64, kernel_name = 'cubic'):
+    unit_system = registry.ds.unit_system
+    if kernel_name == 'cubic':
+        field_name = ("deposit", "%s_smoothed_%s" % (ptype, smoothed_field))
+    else:
+        field_name = ("deposit", "%s_%s_smoothed_%s" % (ptype, kernel_name,
+                      smoothed_field))
     field_units = registry[ptype, smoothed_field].units
     def _vol_weight(field, data):
         pos = data[ptype, coord_name]
         pos = pos.convert_to_units("code_length")
-        mass = data[ptype, mass_name].in_cgs()
-        dens = data[ptype, density_name].in_cgs()
+        mass = data[ptype, mass_name].in_base(unit_system.name)
+        dens = data[ptype, density_name].in_base(unit_system.name)
         quan = data[ptype, smoothed_field]
-        quan = quan.convert_to_units(field_units)
+        if hasattr(quan, "units"):
+            quan = quan.convert_to_units(field_units)
 
         if smoothing_length_name is None:
             hsml = np.zeros(quan.shape, dtype='float64') - 1
@@ -783,18 +811,23 @@ def add_volume_weighted_smoothed_field(ptype, coord_name, mass_name,
         else:
             hsml = data[ptype, smoothing_length_name]
             hsml.convert_to_units("code_length")
-        kwargs = {}
-        if nneighbors:
-            kwargs['nneighbors'] = nneighbors
+        # This is for applying cutoffs, similar to in the SPLASH paper.
+        smooth_cutoff = data["index","cell_volume"]**(1./3)
+        smooth_cutoff.convert_to_units("code_length")
         # volume_weighted smooth operations return lists of length 1.
         rv = data.smooth(pos, [mass, hsml, dens, quan],
+                         index_fields=[smooth_cutoff],
                          method="volume_weighted",
-                         create_octree=True)[0]
+                         create_octree=True,
+                         nneighbors=nneighbors,
+                         kernel_name=kernel_name)[0]
         rv[np.isnan(rv)] = 0.0
         # Now some quick unit conversions.
+        # This should be used when seeking a non-normalized value:
+        rv /= hsml.uq**3 / hsml.uq.in_base(unit_system.name).uq**3
         rv = data.apply_units(rv, field_units)
         return rv
-    registry.add_field(field_name, function = _vol_weight,
+    registry.add_field(field_name, sampling_type="cell", function = _vol_weight,
                        validators = [ValidateSpatial(0)],
                        units = field_units)
     registry.find_dependencies((field_name,))
@@ -811,35 +844,11 @@ def add_nearest_neighbor_field(ptype, coord_name, registry, nneighbors = 64):
                          nneighbors = nneighbors)
         # Now some quick unit conversions.
         return distances
-    registry.add_field(field_name, function = _nth_neighbor,
+    registry.add_field(field_name, sampling_type="particle", function = _nth_neighbor,
                        validators = [ValidateSpatial(0)],
-                       particle_type = True,
                        units = "code_length")
     return [field_name]
 
-def add_density_kernel(ptype, coord_name, mass_name, registry, nneighbors = 64):
-    field_name = (ptype, "smoothed_density")
-    field_units = registry[ptype, mass_name].units
-    def _nth_neighbor(field, data):
-        pos = data[ptype, coord_name]
-        pos.convert_to_units("code_length")
-        mass = data[ptype, mass_name]
-        mass.convert_to_units("g")
-        densities = mass * 0.0
-        data.particle_operation(pos, [mass, densities],
-                         method="density",
-                         nneighbors = nneighbors)
-        ones = pos.prod(axis=1) # Get us in code_length**3
-        ones[:] = 1.0
-        densities /= ones
-        # Now some quick unit conversions.
-        return densities
-    registry.add_field(field_name, function = _nth_neighbor,
-                       validators = [ValidateSpatial(0)],
-                       particle_type = True,
-                       units = "g/cm**3")
-    return [field_name]
-
 def add_union_field(registry, ptype, field_name, units):
     """
     Create a field that is the concatenation of multiple particle types.
@@ -851,6 +860,6 @@ def add_union_field(registry, ptype, field_name, units):
                              for dep_type in data.ds.particle_types_raw])
 
     registry.add_field((ptype, field_name),
+                       sampling_type="particle",
                        function=_cat_field,
-                       particle_type=True,
                        units=units)
