@@ -11,6 +11,7 @@ from yt.testing import \
     assert_array_almost_equal_nulp, \
     assert_array_equal, \
     assert_raises, \
+    assert_allclose_units, \
     requires_file
 from yt.utilities.cosmology import \
     Cosmology
@@ -24,27 +25,6 @@ from yt.utilities.exceptions import \
     YTFieldUnitParseError, \
     YTDimensionalityError
 
-base_ds = None
-
-
-def setup():
-    global base_ds
-    # Make this super teeny tiny
-    fields, units = [], []
-
-    for fname, (code_units, aliases, dn) in StreamFieldInfo.known_other_fields:
-        fields.append(("gas", fname))
-        units.append(code_units)
-
-    base_ds = fake_random_ds(4, fields=fields, units=units, particles=20)
-
-    base_ds.index
-    base_ds.cosmological_simulation = 1
-    base_ds.cosmology = Cosmology()
-    from yt.config import ytcfg
-    ytcfg["yt","__withintesting"] = "True"
-    np.seterr(all = 'ignore')
-
 def get_params(ds):
     return dict(
         axis = 0,
@@ -52,6 +32,8 @@ def get_params(ds):
             registry = ds.unit_registry),
         bulk_velocity = YTArray((0.0, 0.0, 0.0),
             "cm/s", registry = ds.unit_registry),
+        bulk_magnetic_field = YTArray((0.0, 0.0, 0.0),
+            "G", registry = ds.unit_registry),        
         normal = YTArray((0.0, 0.0, 1.0),
             "", registry = ds.unit_registry),
         cp_x_vec = YTArray((1.0, 0.0, 0.0),
@@ -71,18 +53,79 @@ _base_fields = (("gas", "density"),
                 ("gas", "velocity_y"),
                 ("gas", "velocity_z"))
 
-def realistic_ds(fields, particle_fields, nprocs):
-    np.random.seed(int(0x4d3d3d3))
-    global base_ds
-    units = [base_ds._get_field_info(*f).units for f in fields]
-    punits = [base_ds._get_field_info('io', f).units for f in particle_fields]
-    fields = [_strip_ftype(f) for f in fields]
+def _strip_ftype(field):
+    if not isinstance(field, tuple):
+        return field
+    elif field[0] in ("all", "io"):
+        return field
+    return field[1]
 
-    ds = fake_random_ds(16, fields=fields, units=units, nprocs=nprocs,
-                        particle_fields=particle_fields,
-                        particle_field_units=punits,
-                        particles=base_ds.stream_handler.particle_count[0][0])
 
+class TestFieldAccess(object):
+    description = None
+
+    def __init__(self, field_name, ds, nprocs):
+        # Note this should be a field name
+        self.field_name = field_name
+        self.description = "Accessing_%s_%s" % (field_name, nprocs)
+        self.nprocs = nprocs
+        self.ds = ds
+
+    def __call__(self):
+        field = self.ds._get_field_info(*self.field_name)
+        skip_grids = False
+        needs_spatial = False
+        for v in field.validators:
+            if getattr(v, "ghost_zones", 0) > 0:
+                skip_grids = True
+            if hasattr(v, "ghost_zones"):
+                needs_spatial = True
+
+        ds = self.ds
+
+        # This gives unequal sized grids as well as subgrids
+        dd1 = ds.all_data()
+        dd2 = ds.all_data()
+        sp = get_params(ds)
+        dd1.field_parameters.update(sp)
+        dd2.field_parameters.update(sp)
+        with np.errstate(all='ignore'):
+            v1 = dd1[self.field_name]
+            # No more conversion checking
+            assert_equal(v1, dd1[self.field_name])
+            if not needs_spatial:
+                with field.unit_registry(dd2):
+                    res = field._function(field, dd2)
+                    res = dd2.apply_units(res, field.units)
+                assert_array_almost_equal_nulp(v1, res, 4)
+            if not skip_grids:
+                for g in ds.index.grids:
+                    g.field_parameters.update(sp)
+                    v1 = g[self.field_name]
+                    g.clear_data()
+                    g.field_parameters.update(sp)
+                    r1 = field._function(field, g)
+                    if field.particle_type:
+                        assert_equal(v1.shape[0], g.NumberOfParticles)
+                    else:
+                        assert_array_equal(r1.shape, v1.shape)
+                        for ax in 'xyz':
+                            assert_array_equal(g[ax].shape, v1.shape)
+                    with field.unit_registry(g):
+                        res = field._function(field, g)
+                        assert_array_equal(v1.shape, res.shape)
+                        res = g.apply_units(res, field.units)
+                    assert_array_almost_equal_nulp(v1, res, 4)
+
+def get_base_ds(nprocs):
+    fields, units = [], []
+
+    for fname, (code_units, aliases, dn) in StreamFieldInfo.known_other_fields:
+        fields.append(("gas", fname))
+        units.append(code_units)
+
+    ds = fake_random_ds(
+        4, fields=fields, units=units, particles=20, nprocs=nprocs)
     ds.parameters["HydroMethod"] = "streaming"
     ds.parameters["EOSType"] = 1.0
     ds.parameters["EOSSoundSpeed"] = 1.0
@@ -98,108 +141,37 @@ def realistic_ds(fields, particle_fields, nprocs):
                              omega_matter=ds.omega_matter,
                              omega_lambda=ds.omega_lambda,
                              unit_registry=ds.unit_registry)
+    # ensures field errors are raised during testing
+    # see FieldInfoContainer.check_derived_fields
+    ds._field_test_dataset = True
+    ds.index
     return ds
-
-def _strip_ftype(field):
-    if not isinstance(field, tuple):
-        return field
-    elif field[0] in ("all", "io"):
-        return field
-    return field[1]
-
-
-class TestFieldAccess(object):
-    description = None
-
-    def __init__(self, field_name, nproc):
-        # Note this should be a field name
-        self.field_name = field_name
-        self.description = "Accessing_%s_%s" % (field_name, nproc)
-        self.nproc = nproc
-
-    def __call__(self):
-        global base_ds
-        field = base_ds._get_field_info(*self.field_name)
-        deps = field.get_dependencies(ds = base_ds)
-        requested = deps.requested
-        particle_fields = \
-            ['particle_position_x', 'particle_position_y', 'particle_position_z',
-             'particle_velocity_x', 'particle_velocity_y', 'particle_velocity_z',
-             'particle_mass']
-        fields = list(_base_fields)
-
-        for rf in requested:
-            if rf[0] == 'io' or rf[0] == 'all':
-                if rf not in particle_fields or rf[1] not in particle_fields:
-                    particle_fields.append(rf[1])
-            else:
-                fields.append(rf)
-
-        skip_grids = False
-        needs_spatial = False
-        for v in field.validators:
-            f = getattr(v, "fields", None)
-            if f: fields += f
-            if getattr(v, "ghost_zones", 0) > 0:
-                skip_grids = True
-            if hasattr(v, "ghost_zones"):
-                needs_spatial = True
-
-        ds = realistic_ds(fields, particle_fields, self.nproc)
-
-        # This gives unequal sized grids as well as subgrids
-        dd1 = ds.all_data()
-        dd2 = ds.all_data()
-        sp = get_params(ds)
-        dd1.field_parameters.update(sp)
-        dd2.field_parameters.update(sp)
-        v1 = dd1[self.field_name]
-        # No more conversion checking
-        assert_equal(v1, dd1[self.field_name])
-        if not needs_spatial:
-            with field.unit_registry(dd2):
-                res = field._function(field, dd2)
-                res = dd2.apply_units(res, field.units)
-            assert_array_almost_equal_nulp(v1, res, 4)
-        if not skip_grids:
-            for g in ds.index.grids:
-                g.field_parameters.update(sp)
-                v1 = g[self.field_name]
-                g.clear_data()
-                g.field_parameters.update(sp)
-                r1 = field._function(field, g)
-                if field.particle_type:
-                    assert_equal(v1.shape[0], g.NumberOfParticles)
-                else:
-                    assert_array_equal(r1.shape, v1.shape)
-                    for ax in 'xyz':
-                        assert_array_equal(g[ax].shape, v1.shape)
-                with field.unit_registry(g):
-                    res = field._function(field, g)
-                    assert_array_equal(v1.shape, res.shape)
-                    res = g.apply_units(res, field.units)
-                assert_array_almost_equal_nulp(v1, res, 4)
-
+    
 def test_all_fields():
-    global base_ds
-    for field in sorted(base_ds.field_info):
+    datasets = {}
+        
+    for nprocs in [1, 4, 8]:
+        ds = get_base_ds(nprocs)
+        datasets[nprocs] = ds
+
+    for field in sorted(ds.field_info):
         if field[1].find("beta_p") > -1:
             continue
         if field[1].find("vertex") > -1:
             # don't test the vertex fields for now
             continue
-        if field in base_ds.field_list:
+        if field in ds.field_list:
             # Don't know how to test this.  We need some way of having fields
             # that are fallbacks be tested, but we don't have that now.
             continue
 
-        for nproc in [1, 4, 8]:
-            test_all_fields.__name__ = "%s_%s" % (field, nproc)
-            yield TestFieldAccess(field, nproc)
+        for nprocs in [1, 4, 8]:
+            test_all_fields.__name__ = "%s_%s" % (field, nprocs)
+            yield TestFieldAccess(field, datasets[nprocs], nprocs)
 
 def test_add_deposited_particle_field():
     # NOT tested: "std", "mesh_id", "nearest" and "simple_smooth"
-    global base_ds
+    base_ds = get_base_ds(1)
     ad = base_ds.all_data()
 
     # Test "count", "sum" and "cic" method
@@ -231,9 +203,9 @@ def test_add_smoothed_particle_field():
     assert_almost_equal(ret.sum(), 638.5652315154682)
 
 def test_add_gradient_fields():
-    global base_ds
-    gfields = base_ds.add_gradient_fields(("gas","density"))
-    gfields += base_ds.add_gradient_fields(("index", "ones"))
+    ds = get_base_ds(1)
+    gfields = ds.add_gradient_fields(("gas","density"))
+    gfields += ds.add_gradient_fields(("index", "ones"))
     field_list = [('gas', 'density_gradient_x'),
                   ('gas', 'density_gradient_y'),
                   ('gas', 'density_gradient_z'),
@@ -243,7 +215,7 @@ def test_add_gradient_fields():
                   ('index', 'ones_gradient_z'),
                   ('index', 'ones_gradient_magnitude')]
     assert_equal(gfields, field_list)
-    ad = base_ds.all_data()
+    ad = ds.all_data()
     for field in field_list:
         ret = ad[field]
         if field[0] == 'gas':
@@ -267,15 +239,23 @@ def test_add_field_unit_semantics():
     def unitless_data(field, data):
             return np.ones(data['density'].shape)
 
-    ds.add_field(('gas','density_alias_no_units'), function=density_alias)
-    ds.add_field(('gas','density_alias_auto'), function=density_alias,
-                 units='auto', dimensions='density')
-    ds.add_field(('gas','density_alias_wrong_units'), function=density_alias,
+    ds.add_field(('gas','density_alias_no_units'), sampling_type='cell',
+                 function=density_alias)
+    ds.add_field(('gas','density_alias_auto'), sampling_type='cell',
+                 function=density_alias, units='auto', dimensions='density')
+    ds.add_field(('gas','density_alias_wrong_units'),
+                 function=density_alias,
+                 sampling_type='cell',
                  units='m/s')
-    ds.add_field(('gas','density_alias_unparseable_units'), function=density_alias,
+    ds.add_field(('gas','density_alias_unparseable_units'),
+                 sampling_type='cell',
+                 function=density_alias,
                  units='dragons')
-    ds.add_field(('gas','density_alias_auto_wrong_dims'), function=density_alias,
-                 units='auto', dimensions="temperature")
+    ds.add_field(('gas','density_alias_auto_wrong_dims'),
+                 function=density_alias,
+                 sampling_type='cell',
+                 units='auto',
+                 dimensions="temperature")
     assert_raises(YTFieldUnitError, get_data, ds, 'density_alias_no_units')
     assert_raises(YTFieldUnitError, get_data, ds, 'density_alias_wrong_units')
     assert_raises(YTFieldUnitParseError, get_data, ds,
@@ -285,11 +265,22 @@ def test_add_field_unit_semantics():
     dens = ad['density_alias_auto']
     assert_equal(str(dens.units), 'g/cm**3')
 
-    ds.add_field(('gas','dimensionless'), function=unitless_data)
-    ds.add_field(('gas','dimensionless_auto'), function=unitless_data,
-                 units='auto', dimensions='dimensionless')
-    ds.add_field(('gas','dimensionless_explicit'), function=unitless_data, units='')
-    ds.add_field(('gas','dimensionful'), function=unitless_data, units='g/cm**3')
+    ds.add_field(('gas','dimensionless'),
+                 sampling_type='cell',
+                 function=unitless_data)
+    ds.add_field(('gas','dimensionless_auto'),
+                 function=unitless_data,
+                 sampling_type='cell',
+                 units='auto',
+                 dimensions='dimensionless')
+    ds.add_field(('gas','dimensionless_explicit'),
+                 function=unitless_data,
+                 sampling_type='cell',
+                 units='')
+    ds.add_field(('gas','dimensionful'),
+                 sampling_type='cell',
+                 function=unitless_data,
+                 units='g/cm**3')
 
     assert_equal(str(ad['dimensionless'].units), 'dimensionless')
     assert_equal(str(ad['dimensionless_auto'].units), 'dimensionless')
@@ -310,7 +301,8 @@ def test_add_field_string():
     def density_alias(field, data):
         return data['density']
 
-    ds.add_field('density_alias', function=density_alias, units='g/cm**3')
+    ds.add_field('density_alias', sampling_type='cell',
+                 function=density_alias, units='g/cm**3')
 
     ad['density_alias']
     assert ds.derived_field_list[0] == 'density_alias'
@@ -321,7 +313,8 @@ def test_add_field_string_aliasing():
     def density_alias(field, data):
         return data['density']
 
-    ds.add_field('density_alias', function=density_alias, units='g/cm**3')
+    ds.add_field('density_alias', sampling_type='cell',
+                 function=density_alias, units='g/cm**3')
 
     ds.field_info['density_alias']
     ds.field_info['gas', 'density_alias']
@@ -331,8 +324,9 @@ def test_add_field_string_aliasing():
     def pmass_alias(field, data):
         return data['particle_mass']
         
-    ds.add_field('particle_mass_alias', function=pmass_alias, 
-                 units='g', particle_type=True)
+    ds.add_field('particle_mass_alias', function=pmass_alias,
+                 sampling_type='particle',
+                 units='g')
 
     ds.field_info['particle_mass_alias']
     ds.field_info['all', 'particle_mass_alias']
@@ -347,11 +341,20 @@ def test_morton_index():
     a2 = np.argsort(mi2)
     assert_array_equal(a1, a2)
 
-if __name__ == "__main__":
-    setup()
-    for t in test_all_fields():
-        t()
-    test_add_deposited_particle_field()
-    test_add_field_unit_semantics()
-    test_array_like_field()
-    test_add_field_string()
+
+def test_field_inference():
+    ds = fake_random_ds(16)
+    ds.index
+    # If this is not true this means the result of field inference depends
+    # on the order we did field detection, which is random in Python3
+    assert_equal(ds._last_freq, (None, None))
+
+ISOGAL = 'IsolatedGalaxy/galaxy0030/galaxy0030'
+
+@requires_file(ISOGAL)
+def test_deposit_amr():
+    ds = load(ISOGAL)
+    for i, g in enumerate(ds.index.grids):
+        gpm = g['particle_mass'].sum()
+        dpm = g['deposit', 'all_mass'].sum()
+        assert_allclose_units(gpm, dpm)
