@@ -44,7 +44,7 @@ def create_magnitude_field(registry, basename, field_units,
             mag += data[fn] * data[fn]
         return np.sqrt(mag)
 
-    registry.add_field((ftype, "%s_magnitude" % basename),
+    registry.add_field((ftype, "%s_magnitude" % basename), sampling_type="cell", 
                        function=_magnitude, units=field_units,
                        validators=validators, particle_type=particle_type)
 
@@ -62,12 +62,13 @@ def create_squared_field(registry, basename, field_units,
             squared += data[fn] * data[fn]
         return squared
 
-    registry.add_field((ftype, "%s_squared" % basename),
+    registry.add_field((ftype, "%s_squared" % basename), sampling_type="cell", 
                        function=_squared, units=field_units,
                        validators=validators, particle_type=particle_type)
 
 def create_vector_fields(registry, basename, field_units,
                          ftype="gas", slice_info=None):
+    from yt.units.unit_object import Unit
     # slice_info would be the left, the right, and the factor.
     # For example, with the old Enzo-ZEUS fields, this would be:
     # slice(None, -2, None)
@@ -109,7 +110,7 @@ def create_vector_fields(registry, basename, field_units,
         rv[np.isnan(theta)] = 0.0
         return rv
 
-    registry.add_field((ftype, "%s_spherical_radius" % basename),
+    registry.add_field((ftype, "%s_spherical_radius" % basename), sampling_type="cell", 
                        function=_spherical_radius_component,
                        units=field_units,
                        validators=[ValidateParameter("normal"),
@@ -126,17 +127,17 @@ def create_vector_fields(registry, basename, field_units,
         return np.sqrt(data[ftype, "%s_magnitude" % basename]**2.0 -
                        data[ftype, "%s_spherical_radius" % basename]**2.0)
 
-    registry.add_field((ftype, "radial_%s" % basename),
+    registry.add_field((ftype, "radial_%s" % basename), sampling_type="cell", 
                        function=_radial,
                        units=field_units,
                        validators=[ValidateParameter("normal"),
                                    ValidateParameter("center")])
 
-    registry.add_field((ftype, "radial_%s_absolute" % basename),
+    registry.add_field((ftype, "radial_%s_absolute" % basename), sampling_type="cell", 
                        function=_radial_absolute,
                        units=field_units)
 
-    registry.add_field((ftype, "tangential_%s" % basename),
+    registry.add_field((ftype, "tangential_%s" % basename), sampling_type="cell", 
                        function=_tangential,
                        units=field_units)
 
@@ -153,7 +154,7 @@ def create_vector_fields(registry, basename, field_units,
         phi = resize_vector(data["index", "spherical_phi"], vectors)
         return get_sph_theta_component(vectors, theta, phi, normal)
 
-    registry.add_field((ftype, "%s_spherical_theta" % basename),
+    registry.add_field((ftype, "%s_spherical_theta" % basename), sampling_type="cell", 
                        function=_spherical_theta_component,
                        units=field_units,
                        validators=[ValidateParameter("normal"),
@@ -172,7 +173,7 @@ def create_vector_fields(registry, basename, field_units,
         phi = resize_vector(data["index", "spherical_phi"], vectors)
         return get_sph_phi_component(vectors, phi, normal)
 
-    registry.add_field((ftype, "%s_spherical_phi" % basename),
+    registry.add_field((ftype, "%s_spherical_phi" % basename), sampling_type="cell", 
                        function=_spherical_phi_component,
                        units=field_units,
                        validators=[ValidateParameter("normal"),
@@ -182,20 +183,22 @@ def create_vector_fields(registry, basename, field_units,
     def _cp_vectors(ax):
         def _cp_val(field, data):
             vec = data.get_field_parameter("cp_%s_vec" % (ax))
-            bv = data.get_field_parameter("bulk_%s" % basename)
-            if bv is None: bv = np.zeros(3)
-            tr  = (data[xn] - bv[0]) * vec[0]
-            tr += (data[yn] - bv[1]) * vec[1]
-            tr += (data[zn] - bv[2]) * vec[2]
+            bv = data.get_field_parameter("bulk_%s" % basename, None)
+            if bv is None:
+                bv = data.ds.arr(np.zeros(3), data[xn].units)
+            tr  = (data[xn] - bv[0]) * vec.d[0]
+            tr += (data[yn] - bv[1]) * vec.d[1]
+            tr += (data[zn] - bv[2]) * vec.d[2]
             return tr
         return _cp_val
-    registry.add_field((ftype, "cutting_plane_%s_x" % basename),
+
+    registry.add_field((ftype, "cutting_plane_%s_x" % basename), sampling_type="cell", 
                        function=_cp_vectors('x'),
                        units=field_units)
-    registry.add_field((ftype, "cutting_plane_%s_y" % basename),
+    registry.add_field((ftype, "cutting_plane_%s_y" % basename), sampling_type="cell", 
                        function=_cp_vectors('y'),
                        units=field_units)
-    registry.add_field((ftype, "cutting_plane_%s_z" % basename),
+    registry.add_field((ftype, "cutting_plane_%s_z" % basename), sampling_type="cell", 
                        function=_cp_vectors('z'),
                        units=field_units)
 
@@ -217,20 +220,23 @@ def create_vector_fields(registry, basename, field_units,
     def _divergence_abs(field, data):
         return np.abs(data[ftype, "%s_divergence" % basename])
 
-    registry.add_field((ftype, "%s_divergence" % basename),
+    field_units = Unit(field_units, registry=registry.ds.unit_registry)
+    div_units = field_units / registry.ds.unit_system["length"]
+
+    registry.add_field((ftype, "%s_divergence" % basename), sampling_type="cell", 
                        function=_divergence,
-                       units="1/s",
+                       units=div_units,
                        validators=[ValidateSpatial(1)])
     
-    registry.add_field((ftype, "%s_divergence_absolute" % basename),
+    registry.add_field((ftype, "%s_divergence_absolute" % basename), sampling_type="cell", 
                        function=_divergence_abs,
-                       units="1/s")
+                       units=div_units)
 
     def _tangential_over_magnitude(field, data):
         tr = data[ftype, "tangential_%s" % basename] / \
              data[ftype, "%s_magnitude" % basename]
         return np.abs(tr)
-    registry.add_field((ftype, "tangential_over_%s_magnitude" % basename),
+    registry.add_field((ftype, "tangential_over_%s_magnitude" % basename), sampling_type="cell", 
              function=_tangential_over_magnitude,
              take_log=False)
 
@@ -246,7 +252,7 @@ def create_vector_fields(registry, basename, field_units,
         theta = resize_vector(data["index", 'cylindrical_theta'], vectors)
         return get_cyl_r_component(vectors, theta, normal)
 
-    registry.add_field((ftype, "%s_cylindrical_radius" % basename),
+    registry.add_field((ftype, "%s_cylindrical_radius" % basename), sampling_type="cell", 
                        function=_cylindrical_radius_component,
                        units=field_units,
                        validators=[ValidateParameter("normal")])
@@ -255,7 +261,7 @@ def create_vector_fields(registry, basename, field_units,
         """This field is deprecated and will be removed in a future version"""
         return data[ftype, '%s_cylindrical_radius' % basename]
 
-    registry.add_field((ftype, "cylindrical_radial_%s" % basename),
+    registry.add_field((ftype, "cylindrical_radial_%s" % basename), sampling_type="cell", 
                        function=_cylindrical_radial,
                        units=field_units)
 
@@ -263,7 +269,7 @@ def create_vector_fields(registry, basename, field_units,
         """This field is deprecated and will be removed in a future version"""
         return np.abs(data[ftype, '%s_cylindrical_radius' % basename])
 
-    registry.add_field((ftype, "cylindrical_radial_%s_absolute" % basename),
+    registry.add_field((ftype, "cylindrical_radial_%s_absolute" % basename), sampling_type="cell", 
                        function=_cylindrical_radial_absolute,
                        units=field_units,
                        validators=[ValidateParameter("normal")])
@@ -281,7 +287,7 @@ def create_vector_fields(registry, basename, field_units,
         theta = np.tile(theta, (3,) + (1,)*len(theta.shape))
         return get_cyl_theta_component(vectors, theta, normal)
 
-    registry.add_field((ftype, "%s_cylindrical_theta" % basename),
+    registry.add_field((ftype, "%s_cylindrical_theta" % basename), sampling_type="cell", 
                        function=_cylindrical_theta_component,
                        units=field_units,
                        validators=[ValidateParameter("normal"),
@@ -296,11 +302,11 @@ def create_vector_fields(registry, basename, field_units,
         """This field is deprecated and will be removed in a future release"""
         return np.abs(data['cylindrical_tangential_%s' % basename])
 
-    registry.add_field((ftype, "cylindrical_tangential_%s" % basename),
+    registry.add_field((ftype, "cylindrical_tangential_%s" % basename), sampling_type="cell", 
                        function=_cylindrical_tangential,
                        units=field_units)
 
-    registry.add_field((ftype, "cylindrical_tangential_%s_absolute" % basename),
+    registry.add_field((ftype, "cylindrical_tangential_%s_absolute" % basename), sampling_type="cell", 
                        function=_cylindrical_tangential_absolute,
                        units=field_units)
 
@@ -314,7 +320,7 @@ def create_vector_fields(registry, basename, field_units,
         vectors = obtain_rv_vec(data, (xn, yn, zn), "bulk_%s" % basename)
         return get_cyl_z_component(vectors, normal)
 
-    registry.add_field((ftype, "%s_cylindrical_z" % basename),
+    registry.add_field((ftype, "%s_cylindrical_z" % basename), sampling_type="cell", 
                        function=_cylindrical_z_component,
                        units=field_units,
                        validators=[ValidateParameter("normal"),
@@ -350,7 +356,7 @@ def create_averaged_field(registry, basename, field_units, ftype="gas",
         new_field2[1:-1, 1:-1, 1:-1] = new_field / weight_field
         return new_field2
 
-    registry.add_field((ftype, "averaged_%s" % basename),
+    registry.add_field((ftype, "averaged_%s" % basename), sampling_type="cell", 
                        function=_averaged_field,
                        units=field_units,
                        validators=validators)
