@@ -13,13 +13,8 @@ ART-specific fields
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-import numpy as np
-
 from yt.fields.field_info_container import \
     FieldInfoContainer
-from yt.units.yt_array import \
-    YTArray
-from yt.frontends.art.definitions import *
 
 b_units = "code_magnetic"
 ra_units = "code_length / code_time**2"
@@ -62,16 +57,17 @@ class ARTFieldInfo(FieldInfoContainer):
     )
 
     def setup_fluid_fields(self):
+        unit_system = self.ds.unit_system
         def _temperature(field, data):
             r0 = data.ds.parameters['boxh'] / data.ds.parameters['ng']
             tr = data.ds.quan(3.03e5 * r0**2, 'K/code_velocity**2')
             tr *= data.ds.parameters['wmu'] * data.ds.parameters['Om0']
             tr *= (data.ds.parameters['gamma'] - 1.)
             tr /= data.ds.parameters['aexpn']**2
-            return  tr * data['art', 'GasEnergy'] / data['art', 'Density']
-        self.add_field(('gas', 'temperature'),
+            return tr * data['art', 'GasEnergy'] / data['art', 'Density']
+        self.add_field(('gas', 'temperature'), sampling_type="cell", 
                        function=_temperature, 
-                       units='K')
+                       units=unit_system["temperature"])
 
         def _get_vel(axis):
             def velocity(field, data):
@@ -79,9 +75,9 @@ class ARTFieldInfo(FieldInfoContainer):
                         data[('gas','density')])
             return velocity
         for ax in 'xyz':
-            self.add_field(('gas','velocity_%s' % ax),
+            self.add_field(('gas','velocity_%s' % ax), sampling_type="cell", 
                            function = _get_vel(ax),
-                           units='cm/s')
+                           units=unit_system["velocity"])
 
         def _momentum_magnitude(field, data):
             tr = (data['gas','momentum_x']**2 +
@@ -89,31 +85,31 @@ class ARTFieldInfo(FieldInfoContainer):
                   data['gas','momentum_z']**2)**0.5
             tr *= data['index','cell_volume'].in_units('cm**3')
             return tr
-        self.add_field(('gas', 'momentum_magnitude'),
+        self.add_field(('gas', 'momentum_magnitude'), sampling_type="cell", 
                        function=_momentum_magnitude,
-                       units='g*cm/s')
+                       units=unit_system["momentum"])
 
         def _velocity_magnitude(field, data):
             tr = data['gas','momentum_magnitude']
             tr /= data['gas','cell_mass']
             return tr
-        self.add_field(('gas', 'velocity_magnitude'),
+        self.add_field(('gas', 'velocity_magnitude'), sampling_type="cell", 
                        function=_velocity_magnitude,
-                       units='cm/s')
+                       units=unit_system["velocity"])
 
         def _metal_density(field, data):
             tr = data['gas','metal_ia_density']
             tr += data['gas','metal_ii_density']
             return tr
-        self.add_field(('gas','metal_density'),
+        self.add_field(('gas','metal_density'), sampling_type="cell", 
                        function=_metal_density,
-                       units='g/cm**3')
+                       units=unit_system["density"])
 
         def _metal_mass_fraction(field, data):
             tr = data['gas','metal_density']
             tr /= data['gas','density']
             return tr
-        self.add_field(('gas', 'metal_mass_fraction'),
+        self.add_field(('gas', 'metal_mass_fraction'), sampling_type="cell", 
                        function=_metal_mass_fraction,
                        units='')
 
@@ -121,7 +117,7 @@ class ARTFieldInfo(FieldInfoContainer):
             tr = (1. - data.ds.parameters['Y_p'] - 
                   data['gas', 'metal_mass_fraction'])
             return tr
-        self.add_field(('gas', 'H_mass_fraction'),
+        self.add_field(('gas', 'H_mass_fraction'), sampling_type="cell", 
                        function=_H_mass_fraction,
                        units='')
 
@@ -129,6 +125,6 @@ class ARTFieldInfo(FieldInfoContainer):
             tr = data['gas','metal_mass_fraction']
             tr /= data['gas','H_mass_fraction']
             return tr
-        self.add_field(('gas', 'metallicity'),
+        self.add_field(('gas', 'metallicity'), sampling_type="cell", 
                        function=_metallicity,
                        units='')
