@@ -41,7 +41,7 @@ d9p = "D9p_500/10MpcBox_HartGal_csf_a0.500.d"
 def test_d9p():
     ds = data_dir_load(d9p)
     ds.index
-    yield assert_equal, str(ds), "10MpcBox_HartGal_csf_a0.500.d"
+    assert_equal(str(ds), "10MpcBox_HartGal_csf_a0.500.d")
     dso = [None, ("sphere", ("max", (0.1, 'unitary')))]
     for field in _fields:
         for axis in [0, 1, 2]:
@@ -56,18 +56,18 @@ def test_d9p():
     ad = ds.all_data()
     # 'Ana' variable values output from the ART Fortran 'ANA' analysis code
     AnaNStars = 6255
-    yield assert_equal, ad[('stars', 'particle_type')].size, AnaNStars
-    yield assert_equal, ad[('specie4', 'particle_type')].size, AnaNStars
+    assert_equal(ad[('stars', 'particle_type')].size, AnaNStars)
+    assert_equal(ad[('specie4', 'particle_type')].size, AnaNStars)
 
     # The *real* asnwer is 2833405, but yt misses one particle since it lives
     # on a domain boundary. See issue 814. When that is fixed, this test
     # will need to be updated
     AnaNDM = 2833404
-    yield assert_equal, ad[('darkmatter', 'particle_type')].size, AnaNDM
-    yield assert_equal, (ad[('specie0', 'particle_type')].size +
-                         ad[('specie1', 'particle_type')].size +
-                         ad[('specie2', 'particle_type')].size +
-                         ad[('specie3', 'particle_type')].size), AnaNDM
+    assert_equal(ad[('darkmatter', 'particle_type')].size, AnaNDM)
+    assert_equal((ad[('specie0', 'particle_type')].size +
+                  ad[('specie1', 'particle_type')].size +
+                  ad[('specie2', 'particle_type')].size +
+                  ad[('specie3', 'particle_type')].size), AnaNDM)
 
     for spnum in range(5):
         npart_read = ad['specie%s' % spnum, 'particle_type'].size
@@ -81,34 +81,33 @@ def test_d9p():
     AnaVolume = YTQuantity(364.640074656, 'Mpc**3')
     Volume = 1
     for i in ds.domain_width.in_units('Mpc'):
-        yield assert_almost_equal, i, AnaBoxSize
+        assert_almost_equal(i, AnaBoxSize)
         Volume *= i
-    yield assert_almost_equal, Volume, AnaVolume
+    assert_almost_equal(Volume, AnaVolume)
 
     AnaNCells = 4087490
-    yield assert_equal, len(ad[('index', 'cell_volume')]), AnaNCells
+    assert_equal(len(ad[('index', 'cell_volume')]), AnaNCells)
 
     AnaTotDMMass = YTQuantity(1.01191786808255e+14, 'Msun')
-    yield (assert_almost_equal,
-           ad[('darkmatter', 'particle_mass')].sum().in_units('Msun'),
-           AnaTotDMMass)
+    assert_almost_equal(
+        ad[('darkmatter', 'particle_mass')].sum().in_units('Msun'),
+        AnaTotDMMass)
 
     AnaTotStarMass = YTQuantity(1776701.3990607238, 'Msun')
-    yield (assert_almost_equal,
-           ad[('stars', 'particle_mass')].sum().in_units('Msun'),
-           AnaTotStarMass)
+    assert_almost_equal(ad[('stars', 'particle_mass')].sum().in_units('Msun'),
+                        AnaTotStarMass)
 
     AnaTotStarMassInitial = YTQuantity(2423468.2801332865, 'Msun')
-    yield (assert_almost_equal,
-           ad[('stars', 'particle_mass_initial')].sum().in_units('Msun'),
-           AnaTotStarMassInitial)
+    assert_almost_equal(
+        ad[('stars', 'particle_mass_initial')].sum().in_units('Msun'),
+        AnaTotStarMassInitial)
 
     AnaTotGasMass = YTQuantity(1.7826982029216785e+13, 'Msun')
-    yield (assert_almost_equal, ad[('gas', 'cell_mass')].sum().in_units('Msun'),
-           AnaTotGasMass)
+    assert_almost_equal(ad[('gas', 'cell_mass')].sum().in_units('Msun'),
+                        AnaTotGasMass)
 
     AnaTotTemp = YTQuantity(150219844793.39072, 'K')  # just leaves
-    yield assert_equal, ad[('gas', 'temperature')].sum(), AnaTotTemp
+    assert_equal(ad[('gas', 'temperature')].sum(), AnaTotTemp)
 
 
 @requires_file(d9p)
@@ -117,5 +116,4 @@ def test_ARTDataset():
 
 @requires_file(d9p)
 def test_units_override():
-    for test in units_override_check(d9p):
-        yield test
+    units_override_check(d9p)
