@@ -30,13 +30,11 @@
 #include <mpi.h>
 #endif
 
-#include <stdint.h>
-#ifndef int64_t
 #ifdef _WIN32
 typedef __int64 int64_t;
-#endif
+typedef __int32 int32_t;
 #else
-#error "Undefined int64_t!"
+#include <stdint.h>
 #endif
 
 #define ARTIO_OPEN_HEADER					0
