@@ -23,7 +23,12 @@
 #ifndef __ARTIO_EDIAN_H__
 #define __ARTIO_EDIAN_H__
 
+#ifdef _WIN32
+typedef __int64 int64_t;
+typedef __int32 int32_t;
+#else
 #include <stdint.h>
+#endif
 
 void artio_int_swap(int32_t *src, int count);
 void artio_float_swap(float *src, int count);
