@@ -26,7 +26,12 @@
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef _WIN32
+typedef __int64 int64_t;
+typedef __int32 int32_t;
+#else
 #include <stdint.h>
+#endif
 
 int artio_particle_find_file(artio_particle_file *phandle, int start, int end, int64_t sfc);
 artio_particle_file *artio_particle_file_allocate(void);
