@@ -30,7 +30,6 @@ from yt.funcs import \
     ensure_tuple, \
     get_pbar, \
     setdefaultattr
-from yt.config import ytcfg
 from yt.data_objects.grid_patch import \
     AMRGridPatch
 from yt.geometry.grid_geometry_handler import \
@@ -66,28 +65,6 @@ class EnzoGrid(AMRGridPatch):
         self._parent_id = -1
         self.Level = -1
 
-    def _guess_properties_from_parent(self):
-        """
-        We know that our grid boundary occurs on the cell boundary of our
-        parent.  This can be a very expensive process, but it is necessary
-        in some indexs, where yt is unable to generate a completely
-        space-filling tiling of grids, possibly due to the finite accuracy in a
-        standard Enzo index file.
-        """
-        rf = self.ds.refine_by
-        my_ind = self.id - self._id_offset
-        self.dds = self.Parent.dds/rf
-        ParentLeftIndex = np.rint((self.LeftEdge-self.Parent.LeftEdge)/self.Parent.dds)
-        self.start_index = rf*(ParentLeftIndex + self.Parent.get_global_startindex()).astype('int64')
-        self.LeftEdge = self.Parent.LeftEdge + self.Parent.dds * ParentLeftIndex
-        self.RightEdge = self.LeftEdge + self.ActiveDimensions*self.dds
-        self.index.grid_left_edge[my_ind,:] = self.LeftEdge
-        self.index.grid_right_edge[my_ind,:] = self.RightEdge
-        self._child_mask = None
-        self._child_index_mask = None
-        self._child_indices = None
-        self._setup_dx()
-
     def set_filename(self, filename):
         """
         Intelligently set the filename.
@@ -276,7 +253,7 @@ class EnzoHierarchy(GridIndex):
             version = float(params["Internal"]["Provenance"]["VersionNumber"])
         if version >= 3.0:
             active_particles = True
-            nap = dict((ap_type, []) for ap_type in 
+            nap = dict((ap_type, []) for ap_type in
                 params["Physics"]["ActiveParticles"]["ActiveParticlesEnabled"])
         else:
             if "AppendActiveParticleType" in self.parameters:
@@ -380,13 +357,10 @@ class EnzoHierarchy(GridIndex):
         mylog.info("Finished rebuilding")
 
     def _populate_grid_objects(self):
-        reconstruct = ytcfg.getboolean("yt","reconstruct_index")
         for g,f in izip(self.grids, self.filenames):
             g._prepare_grid()
             g._setup_dx()
             g.set_filename(f[0])
-            if reconstruct:
-                if g.Parent is not None: g._guess_properties_from_parent()
         del self.filenames # No longer needed.
         self.max_level = self.grid_levels.max()
 
@@ -425,7 +399,7 @@ class EnzoHierarchy(GridIndex):
             for apt in aps:
                 dd = field._copy_def()
                 dd.pop("name")
-                self.ds.field_info.add_field((apt, fname), **dd)
+                self.ds.field_info.add_field((apt, fname), sampling_type="cell", **dd)
 
     def _detect_output_fields(self):
         self.field_list = []
@@ -761,7 +735,7 @@ class EnzoDataset(Dataset):
         """
         # Let's read the file
         with open(self.parameter_filename, "r") as f:
-            line = f.readline().strip() 
+            line = f.readline().strip()
             f.seek(0)
             if line == "Internal:":
                 self._parse_enzo3_parameter_file(f)
@@ -852,7 +826,7 @@ class EnzoDataset(Dataset):
             self.unique_identifier = self.parameters["CurrentTimeIdentifier"]
         else:
             self.unique_identifier = \
-                int(os.stat(self.parameter_filename)[stat.ST_CTIME])
+                str(int(os.stat(self.parameter_filename)[stat.ST_CTIME]))
         if self.dimensionality > 1:
             self.domain_dimensions = self.parameters["TopGridDimensions"]
             if len(self.domain_dimensions) < 3:
@@ -873,11 +847,6 @@ class EnzoDataset(Dataset):
             self.periodicity += (False, False)
 
         self.gamma = self.parameters["Gamma"]
-        # To be enabled when we can break old pickles:
-        #if "MetaDataSimulationUUID" in self.parameters:
-        #    self.unique_identifier = self.parameters["MetaDataSimulationUUID"]
-        self.unique_identifier = self.parameters.get("MetaDataDatasetUUID",
-                self.parameters.get("CurrentTimeIdentifier", None))
         if self.parameters["ComovingCoordinates"]:
             self.cosmological_simulation = 1
             self.current_redshift = self.parameters["CosmologyCurrentRedshift"]
@@ -1099,4 +1068,3 @@ def rlines(f, keepends=False):
             for line in lines:
                 yield line
     yield buf  # First line.
-
