@@ -13,28 +13,26 @@ FLASH-specific data structures
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-import h5py
+import os
 import stat
 import numpy as np
 import weakref
 
-from yt.config import ytcfg
-from yt.funcs import *
 from yt.data_objects.grid_patch import \
     AMRGridPatch
+from yt.data_objects.static_output import \
+    Dataset, ParticleFile
+from yt.funcs import \
+    mylog, \
+    setdefaultattr
 from yt.geometry.grid_geometry_handler import \
     GridIndex
-from yt.geometry.geometry_handler import \
-    YTDataChunk
-from yt.data_objects.static_output import \
-    Dataset
-from yt.utilities.definitions import \
-    mpc_conversion, sec_conversion
+from yt.geometry.particle_geometry_handler import \
+    ParticleIndex
 from yt.utilities.file_handler import \
-    HDF5FileHandler
-from yt.utilities.io_handler import \
-    io_registry
-from yt.utilities.physical_constants import cm_per_mpc
+    HDF5FileHandler, \
+    warn_h5py
+from yt.utilities.physical_ratios import cm_per_mpc
 from .fields import FLASHFieldInfo
 
 class FLASHGrid(AMRGridPatch):
@@ -71,7 +69,6 @@ class FLASHHierarchy(GridIndex):
         pass
 
     def _detect_output_fields(self):
-        ncomp = self._handle["/unknown names"].shape[0]
         self.field_list = [("flash", s.decode("ascii","ignore"))
                            for s in self._handle["/unknown names"][:].flat]
         if ("/particle names" in self._particle_handle):
@@ -84,7 +81,11 @@ class FLASHHierarchy(GridIndex):
             self.num_grids = self.dataset._find_parameter(
                 "integer", "globalnumblocks", True)
         except KeyError:
-            self.num_grids = self._handle["/simulation parameters"][0][0]
+            try:
+                self.num_grids = \
+                    self._handle['simulation parameters']['total blocks'][0]
+            except KeyError:
+                self.num_grids = self._handle["/simulation parameters"][0][0]
         
     def _parse_index(self):
         f = self._handle # shortcut
@@ -152,16 +153,7 @@ class FLASHHierarchy(GridIndex):
             gre[:,2] = 2.0 * np.pi
             return
 
-        # Now, for cartesian data.
-        for i in range(self.num_grids):
-            dx = dxs[self.grid_levels[i],:]
-            gle[i][:ND] = np.rint(gle[i][:ND]/dx[0][:ND])*dx[0][:ND]
-            gre[i][:ND] = np.rint(gre[i][:ND]/dx[0][:ND])*dx[0][:ND]
-
     def _populate_grid_objects(self):
-        # We only handle 3D data, so offset is 7 (nfaces+1)
-        
-        offset = 7
         ii = np.argsort(self.grid_levels.flat)
         gid = self._handle["/gid"][:]
         first_ind = -(self.dataset.refine_by**self.dataset.dimensionality)
@@ -193,7 +185,8 @@ class FLASHDataset(Dataset):
     def __init__(self, filename, dataset_type='flash_hdf5',
                  storage_filename = None,
                  particle_filename = None, 
-                 units_override = None):
+                 units_override = None,
+                 unit_system = "cgs"):
 
         self.fluid_types += ("flash",)
         if self._handle is not None: return
@@ -202,18 +195,35 @@ class FLASHDataset(Dataset):
         self.particle_filename = particle_filename
 
         if self.particle_filename is None:
-            self._particle_handle = self._handle
+            # try to guess the particle filename
+            try:
+                self._particle_handle = HDF5FileHandler(filename.replace('plt_cnt', 'part'))
+                self.particle_filename = filename.replace('plt_cnt', 'part')
+                mylog.info('Particle file found: %s' % self.particle_filename.split('/')[-1])
+            except IOError:
+                self._particle_handle = self._handle
         else:
+            # particle_filename is specified by user
             try:
                 self._particle_handle = HDF5FileHandler(self.particle_filename)
             except:
                 raise IOError(self.particle_filename)
+        # Check if the particle file has the same time
+        if self._particle_handle != self._handle:
+            part_time = self._particle_handle.handle.get('real scalars')[0][1]
+            plot_time = self._handle.handle.get('real scalars')[0][1]
+            if not np.isclose(part_time, plot_time):
+                self._particle_handle = self._handle
+                mylog.warning('%s and %s are not at the same time. ' % (self.particle_filename, filename) +
+                              'This particle file will not be used.')
+
         # These should be explicitly obtained from the file, but for now that
         # will wait until a reorganization of the source tree and better
         # generalization.
         self.refine_by = 2
 
-        Dataset.__init__(self, filename, dataset_type, units_override=units_override)
+        Dataset.__init__(self, filename, dataset_type, units_override=units_override,
+                         unit_system=unit_system)
         self.storage_filename = storage_filename
 
         self.parameters["HydroMethod"] = 'flash' # always PPM DE
@@ -239,21 +249,17 @@ class FLASHDataset(Dataset):
         else:
             length_factor = 1.0
             temperature_factor = 1.0
-        self.magnetic_unit = self.quan(b_factor, "gauss")
 
-        self.length_unit = self.quan(length_factor, "cm")
-        self.mass_unit = self.quan(1.0, "g")
-        self.time_unit = self.quan(1.0, "s")
-        self.velocity_unit = self.quan(1.0, "cm/s")
-        self.temperature_unit = self.quan(temperature_factor, "K")
-        # Still need to deal with:
-        #self.conversion_factors['temp'] = (1.0 + self.current_redshift)**-2.0
-        self.unit_registry.modify("code_magnetic", self.magnetic_unit)
-        
+        setdefaultattr(self, 'magnetic_unit', self.quan(b_factor, "gauss"))
+        setdefaultattr(self, 'length_unit', self.quan(length_factor, "cm"))
+        setdefaultattr(self, 'mass_unit', self.quan(1.0, "g"))
+        setdefaultattr(self, 'time_unit', self.quan(1.0, "s"))
+        setdefaultattr(self, 'velocity_unit', self.quan(1.0, "cm/s"))
+        setdefaultattr(
+            self, 'temperature_unit', self.quan(temperature_factor, "K"))
+
     def set_code_units(self):
         super(FLASHDataset, self).set_code_units()
-        self.unit_registry.modify("code_temperature",
-                                  self.temperature_unit.value)
 
     def _find_parameter(self, ptype, pname, scalar = False):
         nn = "/%s %s" % (ptype,
@@ -310,11 +316,14 @@ class FLASHDataset(Dataset):
                 if hn not in self._handle:
                     continue
                 if hn is 'simulation parameters':
-                    zipover = zip(self._handle[hn].dtype.names,self._handle[hn][0])
+                    zipover = ((name, self._handle[hn][name][0])
+                               for name in self._handle[hn].dtype.names)
                 else:
                     zipover = zip(self._handle[hn][:,'name'],self._handle[hn][:,'value'])
                 for varname, val in zipover:
                     vn = varname.strip()
+                    if hasattr(vn, 'decode'):
+                        vn = vn.decode("ascii", "ignore")
                     if hn.startswith("string"):
                         pval = val.strip()
                     else:
@@ -324,7 +333,7 @@ class FLASHDataset(Dataset):
                                    "scalar of the same name".format(hn[:-1],vn))
                     if hasattr(pval, 'decode'):
                         pval = pval.decode("ascii", "ignore")
-                    self.parameters[vn.decode("ascii", "ignore")] = pval
+                    self.parameters[vn] = pval
         
         # Determine block size
         try:
@@ -363,25 +372,32 @@ class FLASHDataset(Dataset):
         if self.dimensionality == 1: nblocky = 1
 
         # Determine domain boundaries
-        self.domain_left_edge = np.array(
+        dle = np.array(
             [self.parameters["%smin" % ax] for ax in 'xyz']).astype("float64")
-        self.domain_right_edge = np.array(
+        dre = np.array(
             [self.parameters["%smax" % ax] for ax in 'xyz']).astype("float64")
         if self.dimensionality < 3:
             for d in [dimensionality]+list(range(3-dimensionality)):
-                if self.domain_left_edge[d] == self.domain_right_edge[d]:
-                    mylog.warning('Identical domain left edge and right edges '
-                                  'along dummy dimension (%i), attempting to read anyway' % d)
-                    self.domain_right_edge[d] = self.domain_left_edge[d]+1.0
+                if dle[d] == dre[d]:
+                    mylog.warning(
+                        'Identical domain left edge and right edges '
+                        'along dummy dimension (%i), attempting to read anyway'
+                        % d)
+                    dre[d] = dle[d]+1.0
         if self.dimensionality < 3 and self.geometry == "cylindrical":
             mylog.warning("Extending theta dimension to 2PI + left edge.")
-            self.domain_right_edge[2] = self.domain_left_edge[2] + 2*np.pi
+            dre[2] = dle[2] + 2*np.pi
         elif self.dimensionality < 3 and self.geometry == "polar":
             mylog.warning("Extending theta dimension to 2PI + left edge.")
-            self.domain_right_edge[1] = self.domain_left_edge[1] + 2*np.pi
+            dre[1] = dle[1] + 2*np.pi
         elif self.dimensionality < 3 and self.geometry == "spherical":
             mylog.warning("Extending phi dimension to 2PI + left edge.")
-            self.domain_right_edge[2] = self.domain_left_edge[2] + 2*np.pi
+            dre[2] = dle[2] + 2*np.pi
+        if self.dimensionality == 1 and self.geometry == "spherical":
+            mylog.warning("Extending theta dimension to PI + left edge.")
+            dre[1] = dle[1] + np.pi
+        self.domain_left_edge = dle
+        self.domain_right_edge = dre
         self.domain_dimensions = \
             np.array([nblockx*nxb,nblocky*nyb,nblockz*nzb])
 
@@ -418,8 +434,68 @@ class FLASHDataset(Dataset):
             fileh = HDF5FileHandler(args[0])
             if "bounding box" in fileh["/"].keys():
                 return True
-        except:
+        except (IOError, OSError, ImportError):
             pass
         return False
 
+    @classmethod
+    def _guess_candidates(cls, base, directories, files):
+        candidates = [_ for _ in files if
+                      ("_hdf5_plt_cnt_" in _) or
+                      ("_hdf5_chk_" in _)]
+        # Typically, Flash won't have nested outputs.
+        return candidates, (len(candidates) == 0)
 
+    def close(self):
+        self._handle.close()
+
+class FLASHParticleFile(ParticleFile):
+    pass
+
+class FLASHParticleDataset(FLASHDataset):
+    _index_class = ParticleIndex
+    over_refine_factor = 1
+    filter_bbox = False
+    _file_class = FLASHParticleFile
+
+    def __init__(self, filename, dataset_type='flash_particle_hdf5',
+                 storage_filename = None,
+                 units_override = None,
+                 n_ref = 64, unit_system = "cgs"):
+
+        if self._handle is not None: return
+        self._handle = HDF5FileHandler(filename)
+        self.n_ref = n_ref
+        self.refine_by = 2
+        Dataset.__init__(self, filename, dataset_type, units_override=units_override,
+                         unit_system=unit_system)
+        self.storage_filename = storage_filename
+
+    def _parse_parameter_file(self):
+        # Let the superclass do all the work but then
+        # fix the domain dimensions
+        super(FLASHParticleDataset, self)._parse_parameter_file()
+        nz = 1 << self.over_refine_factor
+        domain_dimensions = np.zeros(3, "int32")
+        domain_dimensions[:self.dimensionality] = nz
+        self.domain_dimensions = domain_dimensions
+        self.filename_template = self.parameter_filename
+        self.file_count = 1
+
+    @classmethod
+    def _is_valid(self, *args, **kwargs):
+        warn_h5py(args[0])
+        try:
+            fileh = HDF5FileHandler(args[0])
+            if "bounding box" not in fileh["/"].keys() \
+                and "localnp" in fileh["/"].keys():
+                return True
+        except (IOError, OSError, ImportError):
+            pass
+        return False
+
+    @classmethod
+    def _guess_candidates(cls, base, directories, files):
+        candidates = [_ for _ in files if "_hdf5_part_" in _]
+        # Typically, Flash won't have nested outputs.
+        return candidates, (len(candidates) == 0)
