@@ -15,43 +15,67 @@ from __future__ import print_function
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-import h5py
+from yt.extern.six import string_types
+from yt.funcs import only_on_root
+from yt.utilities.on_demand_imports import _h5py as h5py
 import numpy as np
 import stat
 import struct
 import os
-import types
 
 from yt.data_objects.static_output import \
     ParticleFile
 from yt.frontends.sph.data_structures import \
-    ParticleDataset
+    SPHDataset
 from yt.geometry.particle_geometry_handler import \
     ParticleIndex
 from yt.utilities.cosmology import \
     Cosmology
-from yt.utilities.definitions import \
-    sec_conversion
 from yt.utilities.fortran_utils import read_record
 from yt.utilities.logger import ytLogger as mylog
 
 from .definitions import \
     gadget_header_specs, \
     gadget_field_specs, \
-    gadget_ptype_specs
+    gadget_ptype_specs, \
+    SNAP_FORMAT_2_OFFSET
 
 from .fields import \
     GadgetFieldInfo
 
+
 def _fix_unit_ordering(unit):
-    if isinstance(unit[0], str):
+    if isinstance(unit[0], string_types):
         unit = unit[1], unit[0]
     return unit
 
+
+def _get_gadget_format(filename):
+    # check and return gadget binary format with file endianness
+    ff = open(filename, 'rb')
+    (rhead,) = struct.unpack('<I', ff.read(4))
+    ff.close()
+    if (rhead == 134217728):
+        return 2, '>'
+    elif (rhead == 8):
+        return 2, '<'
+    elif (rhead == 65536):
+        return 1, '>'
+    elif (rhead == 256):
+        return 1, '<'
+    else:
+        raise RuntimeError("Incorrect Gadget format %s!" % str(rhead))
+
+
 class GadgetBinaryFile(ParticleFile):
     def __init__(self, ds, io, filename, file_id):
+        gformat = _get_gadget_format(filename)
         with open(filename, "rb") as f:
-            self.header = read_record(f, ds._header_spec)
+            if gformat[0] == 2:
+                f.seek(f.tell() + SNAP_FORMAT_2_OFFSET)
+            self.header = read_record(f, ds._header_spec, endian=gformat[1])
+            if gformat[0] == 2:
+                f.seek(f.tell() + SNAP_FORMAT_2_OFFSET)
             self._position_offset = f.tell()
             f.seek(0, os.SEEK_END)
             self._file_size = f.tell()
@@ -63,7 +87,8 @@ class GadgetBinaryFile(ParticleFile):
             field_list, self.total_particles,
             self._position_offset, self._file_size)
 
-class GadgetDataset(ParticleDataset):
+
+class GadgetDataset(SPHDataset):
     _index_class = ParticleIndex
     _file_class = GadgetBinaryFile
     _field_info_class = GadgetFieldInfo
@@ -76,20 +101,26 @@ class GadgetDataset(ParticleDataset):
                  additional_fields=(),
                  unit_base=None, n_ref=64,
                  over_refine_factor=1,
-                 bounding_box = None,
-                 header_spec = "default",
-                 field_spec = "default",
-                 ptype_spec = "default",
-                 units_override=None):
-        if self._instantiated: return
+                 kernel_name=None,
+                 index_ptype="all",
+                 bounding_box=None,
+                 header_spec="default",
+                 field_spec="default",
+                 ptype_spec="default",
+                 units_override=None,
+                 unit_system="cgs",
+                 use_dark_factor = False,
+                 w_0 = -1.0,
+                 w_a = 0.0):
+        if self._instantiated:
+            return
         self._header_spec = self._setup_binary_spec(
             header_spec, gadget_header_specs)
         self._field_spec = self._setup_binary_spec(
             field_spec, gadget_field_specs)
         self._ptype_spec = self._setup_binary_spec(
             ptype_spec, gadget_ptype_specs)
-        self.n_ref = n_ref
-        self.over_refine_factor = over_refine_factor
+        self.index_ptype = index_ptype
         self.storage_filename = None
         if unit_base is not None and "UnitLength_in_cm" in unit_base:
             # We assume this is comoving, because in the absence of comoving
@@ -100,14 +131,23 @@ class GadgetDataset(ParticleDataset):
             bbox = np.array(bounding_box, dtype="float64")
             if bbox.shape == (2, 3):
                 bbox = bbox.transpose()
-            self.domain_left_edge = bbox[:,0]
-            self.domain_right_edge = bbox[:,1]
+            self.domain_left_edge = bbox[:, 0]
+            self.domain_right_edge = bbox[:, 1]
         else:
             self.domain_left_edge = self.domain_right_edge = None
         if units_override is not None:
-            raise RuntimeError("units_override is not supported for GadgetDataset. "+
+            raise RuntimeError("units_override is not supported for GadgetDataset. " +
                                "Use unit_base instead.")
-        super(GadgetDataset, self).__init__(filename, dataset_type)
+
+        # Set dark energy parameters before cosmology object is created
+        self.use_dark_factor = use_dark_factor
+        self.w_0 = w_0
+        self.w_a = w_a
+
+        super(GadgetDataset, self).__init__(
+            filename, dataset_type=dataset_type, unit_system=unit_system,
+            n_ref=n_ref, over_refine_factor=over_refine_factor,
+            kernel_name=kernel_name)
         if self.cosmological_simulation:
             self.time_unit.convert_to_units('s/h')
             self.length_unit.convert_to_units('kpccm/h')
@@ -131,9 +171,11 @@ class GadgetDataset(ParticleDataset):
     def _get_hvals(self):
         # The entries in this header are capitalized and named to match Table 4
         # in the GADGET-2 user guide.
-
+        gformat = _get_gadget_format(self.parameter_filename)
         f = open(self.parameter_filename, 'rb')
-        hvals = read_record(f, self._header_spec)
+        if gformat[0] == 2:
+            f.seek(f.tell() + SNAP_FORMAT_2_OFFSET)
+        hvals = read_record(f, self._header_spec, endian=gformat[1])
         for i in hvals:
             if len(hvals[i]) == 1:
                 hvals[i] = hvals[i][0]
@@ -171,7 +213,8 @@ class GadgetDataset(ParticleDataset):
         # It may be possible to deduce whether ComovingIntegration is on
         # somehow, but opinions on this vary.
         if self.omega_lambda == 0.0:
-            mylog.info("Omega Lambda is 0.0, so we are turning off Cosmology.")
+            only_on_root(
+                mylog.info, "Omega Lambda is 0.0, so we are turning off Cosmology.")
             self.hubble_constant = 1.0  # So that scaling comes out correct
             self.cosmological_simulation = 0
             self.current_redshift = 0.0
@@ -185,8 +228,8 @@ class GadgetDataset(ParticleDataset):
             cosmo = Cosmology(self.hubble_constant,
                               self.omega_matter, self.omega_lambda)
             self.current_time = cosmo.hubble_time(self.current_redshift)
-            mylog.info("Calculating time from %0.3e to be %0.3e seconds",
-                       hvals["Time"], self.current_time)
+            only_on_root(mylog.info, "Calculating time from %0.3e to be %0.3e seconds",
+                         hvals["Time"], self.current_time)
         self.parameters = hvals
 
         prefix = os.path.abspath(
@@ -201,14 +244,17 @@ class GadgetDataset(ParticleDataset):
         self.file_count = hvals["NumFiles"]
 
     def _set_code_unit_attributes(self):
-        # If no units passed in by user, set a sane default (Gadget-2 users guide).
+        # If no units passed in by user, set a sane default (Gadget-2 users
+        # guide).
         if self._unit_base is None:
             if self.cosmological_simulation == 1:
-                mylog.info("Assuming length units are in kpc/h (comoving)")
-                self._unit_base = dict(length = (1.0, "kpccm/h"))
+                only_on_root(
+                    mylog.info, "Assuming length units are in kpc/h (comoving)")
+                self._unit_base = dict(length=(1.0, "kpccm/h"))
             else:
-                mylog.info("Assuming length units are in kpc (physical)")
-                self._unit_base = dict(length = (1.0, "kpc"))
+                only_on_root(
+                    mylog.info, "Assuming length units are in kpc (physical)")
+                self._unit_base = dict(length=(1.0, "kpc"))
 
         # If units passed in by user, decide what to do about
         # co-moving and factors of h
@@ -279,26 +325,26 @@ class GadgetDataset(ParticleDataset):
     @staticmethod
     def _validate_header(filename):
         '''
-        This method automatically detects whether the Gadget file is big/little endian 
-        and is not corrupt/invalid using the first 4 bytes in the file.  It returns a 
-        tuple of (Valid, endianswap) where Valid is a boolean that is true if the file 
-        is a Gadget binary file, and endianswap is the endianness character '>' or '<'. 
+        This method automatically detects whether the Gadget file is big/little endian
+        and is not corrupt/invalid using the first 4 bytes in the file.  It returns a
+        tuple of (Valid, endianswap) where Valid is a boolean that is true if the file
+        is a Gadget binary file, and endianswap is the endianness character '>' or '<'.
         '''
         try:
-            f = open(filename,'rb')
+            f = open(filename, 'rb')
         except IOError:
             try:
-                f = open(filename+".0")
+                f = open(filename + ".0")
             except IOError:
                 return False, 1
-        
+
         # First int32 is 256 for a Gadget2 binary file with SnapFormat=1,
-        # 8 for a Gadget2 binary file with SnapFormat=2 file, 
+        # 8 for a Gadget2 binary file with SnapFormat=2 file,
         # or the byte swapped equivalents (65536 and 134217728).
         # The int32 following the header (first 4+256 bytes) must equal this
         # number.
         try:
-            (rhead,) = struct.unpack('<I',f.read(4))
+            (rhead,) = struct.unpack('<I', f.read(4))
         except struct.error:
             f.close()
             return False, 1
@@ -307,25 +353,27 @@ class GadgetDataset(ParticleDataset):
             endianswap = '<'
         elif rhead == 65536:
             endianswap = '>'
-        # Disabled for now (does any one still use SnapFormat=2?)
-        # If so, alternate read would be needed based on header.
-        # elif rhead == 8:
-        #     return True, '<'
-        # elif rhead == 134217728:
-        #     return True, '>'
+        elif rhead in (8, 134217728):
+            # This is only true for snapshot format 2
+            # we do not currently support double precision
+            # snap format 2 data
+            f.close()
+            return True, 'f4'
         else:
             f.close()
             return False, 1
         # Read in particle number from header
-        np0 = sum(struct.unpack(endianswap+'IIIIII',f.read(6*4)))
-        # Read in size of position block. It should be 4 bytes per float, 
+        np0 = sum(struct.unpack(endianswap + 'IIIIII', f.read(6 * 4)))
+        # Read in size of position block. It should be 4 bytes per float,
         # with 3 coordinates (x,y,z) per particle. (12 bytes per particle)
-        f.seek(4+256+4,0)
-        np1 = struct.unpack(endianswap+'I',f.read(4))[0]/(4*3)
+        f.seek(4 + 256 + 4, 0)
+        np1 = struct.unpack(endianswap + 'I', f.read(4))[0] / (4 * 3)
         f.close()
         # Compare
         if np0 == np1:
-            return True, endianswap
+            return True, 'f4'
+        elif np1 == 2*np0:
+            return True, 'f8'
         else:
             return False, 1
 
@@ -334,6 +382,7 @@ class GadgetDataset(ParticleDataset):
         # First 4 bytes used to check load
         return GadgetDataset._validate_header(args[0])[0]
 
+
 class GadgetHDF5Dataset(GadgetDataset):
     _file_class = ParticleFile
     _field_info_class = GadgetFieldInfo
@@ -341,19 +390,23 @@ class GadgetHDF5Dataset(GadgetDataset):
     _suffix = ".hdf5"
 
     def __init__(self, filename, dataset_type="gadget_hdf5",
-                 unit_base = None, n_ref=64,
+                 unit_base=None, n_ref=64,
                  over_refine_factor=1,
-                 bounding_box = None,
-                 units_override=None):
+                 kernel_name=None,
+                 index_ptype="all",
+                 bounding_box=None,
+                 units_override=None,
+                 unit_system="cgs"):
         self.storage_filename = None
         filename = os.path.abspath(filename)
         if units_override is not None:
-            raise RuntimeError("units_override is not supported for GadgetHDF5Dataset. "+
+            raise RuntimeError("units_override is not supported for GadgetHDF5Dataset. " +
                                "Use unit_base instead.")
         super(GadgetHDF5Dataset, self).__init__(
             filename, dataset_type, unit_base=unit_base, n_ref=n_ref,
-            over_refine_factor=over_refine_factor,
-            bounding_box = bounding_box)
+            over_refine_factor=over_refine_factor, index_ptype=index_ptype,
+            kernel_name=kernel_name, bounding_box=bounding_box,
+            unit_system=unit_system)
 
     def _get_hvals(self):
         handle = h5py.File(self.parameter_filename, mode="r")
@@ -372,8 +425,6 @@ class GadgetHDF5Dataset(GadgetDataset):
         handle.close()
         return uvals
 
-
-
     def _set_owls_eagle(self):
 
         self.dimensionality = 3
@@ -392,7 +443,8 @@ class GadgetHDF5Dataset(GadgetDataset):
 
         if self.domain_left_edge is None:
             self.domain_left_edge = np.zeros(3, "float64")
-            self.domain_right_edge = np.ones(3, "float64") * self.parameters["BoxSize"]
+            self.domain_right_edge = np.ones(
+                3, "float64") * self.parameters["BoxSize"]
 
         nz = 1 << self.over_refine_factor
         self.domain_dimensions = np.ones(3, "int32") * nz
@@ -416,9 +468,11 @@ class GadgetHDF5Dataset(GadgetDataset):
 
         # note the contents of the HDF5 Units group are in _unit_base
         # note the velocity stored on disk is sqrt(a) dx/dt
-        self.length_unit = self.quan(self._unit_base["UnitLength_in_cm"], 'cmcm/h')
+        self.length_unit = self.quan(
+            self._unit_base["UnitLength_in_cm"], 'cmcm/h')
         self.mass_unit = self.quan(self._unit_base["UnitMass_in_g"], 'g/h')
-        self.velocity_unit = self.quan(self._unit_base["UnitVelocity_in_cm_per_s"], 'cm/s')
+        self.velocity_unit = self.quan(
+            self._unit_base["UnitVelocity_in_cm_per_s"], 'cm/s')
         self.time_unit = self.quan(self._unit_base["UnitTime_in_s"], 's/h')
 
     @classmethod
@@ -429,7 +483,7 @@ class GadgetHDF5Dataset(GadgetDataset):
         try:
             fh = h5py.File(args[0], mode='r')
             valid = all(ng in fh["/"] for ng in need_groups) and \
-              not any(vg in fh["/"] for vg in veto_groups)
+                not any(vg in fh["/"] for vg in veto_groups)
             fh.close()
         except:
             valid = False
