@@ -35,8 +35,8 @@ class GadgetFieldInfo(SPHFieldInfo):
 
     def _setup_four_metal_fractions(self, ptype):
         """
-        This function breaks the FourMetalFractions field (if present) 
-        into its four component metal fraction fields and adds 
+        This function breaks the FourMetalFractions field (if present)
+        into its four component metal fraction fields and adds
         corresponding metal density fields which will later get smoothed
 
         This gets used with the Gadget group0000 format
@@ -52,8 +52,8 @@ class GadgetFieldInfo(SPHFieldInfo):
                 return _Fraction
 
             self.add_field( (ptype, metal_name+"_fraction"),
-                            function=_Fraction_wrap(i), 
-                            particle_type=True,
+                            sampling_type="particle",
+                            function=_Fraction_wrap(i),
                             units="")
 
             # add the metal density fields
@@ -64,8 +64,8 @@ class GadgetFieldInfo(SPHFieldInfo):
                 return _Metal_density
 
             self.add_field( (ptype, metal_name+"_density"),
-                            function=_Density_wrap(i), 
-                            particle_type=True,
+                            sampling_type="particle",
+                            function=_Density_wrap(i),
                             units=self.ds.unit_system["density"])
 
     def setup_gas_particle_fields(self, ptype):
@@ -83,15 +83,18 @@ class GadgetFieldInfo(SPHFieldInfo):
                 # Assume cosmic abundances
                 x_H = 0.76
                 gamma = 5.0/3.0
-                # Assume zero ionization
-                mu = 4.0 / (3.0 * x_H + 1.0)
+                if data.has_field_parameter("mean_molecular_weight"):
+                    mu = data.get_field_parameter("mean_molecular_weight")
+                else:
+                    # Assume zero ionization
+                    mu = 4.0 / (3.0 * x_H + 1.0)
                 ret = data[ptype, "InternalEnergy"]*(gamma-1)*mu*mp/kb
                 return ret.in_units(self.ds.unit_system["temperature"])
 
         self.add_field(
             (ptype, "Temperature"),
+            sampling_type="particle",
             function=_temperature,
-            particle_type=True,
             units=self.ds.unit_system["temperature"])
         self.alias((ptype, 'temperature'), (ptype, 'Temperature'))
 
