@@ -18,7 +18,9 @@ from yt.utilities.on_demand_imports import _h5py as h5py
 import numpy as np
 
 from yt.utilities.exceptions import YTDomainOverflow
-from yt.funcs import mylog
+from yt.funcs import \
+    mylog, \
+    parse_h5_attr
 
 from yt.utilities.io_handler import \
     BaseIOHandler
@@ -42,11 +44,13 @@ class IOHandlerHaloCatalogHDF5(BaseIOHandler):
         for chunk in chunks:
             for obj in chunk.objs:
                 data_files.update(obj.data_files)
+        pn = "particle_position_%s"
         for data_file in sorted(data_files):
             with h5py.File(data_file.filename, "r") as f:
-                x = f['particle_position_x'].value.astype("float64")
-                y = f['particle_position_y'].value.astype("float64")
-                z = f['particle_position_z'].value.astype("float64")
+                units = parse_h5_attr(f[pn % "x"], "units")
+                x, y, z = \
+                  (self.ds.arr(f[pn % ax].value.astype("float64"), units)
+                   for ax in "xyz")
                 yield "halos", (x, y, z)
 
     def _read_particle_fields(self, chunks, ptf, selector):
@@ -59,12 +63,14 @@ class IOHandlerHaloCatalogHDF5(BaseIOHandler):
         for chunk in chunks:
             for obj in chunk.objs:
                 data_files.update(obj.data_files)
+        pn = "particle_position_%s"
         for data_file in sorted(data_files):
             with h5py.File(data_file.filename, "r") as f:
                 for ptype, field_list in sorted(ptf.items()):
-                    x = f['particle_position_x'].value.astype("float64")
-                    y = f['particle_position_y'].value.astype("float64")
-                    z = f['particle_position_z'].value.astype("float64")
+                    units = parse_h5_attr(f[pn % "x"], "units")
+                    x, y, z = \
+                      (self.ds.arr(f[pn % ax].value.astype("float64"), units)
+                       for ax in "xyz")
                     mask = selector.select_points(x, y, z, 0.0)
                     del x, y, z
                     if mask is None: continue
@@ -78,31 +84,31 @@ class IOHandlerHaloCatalogHDF5(BaseIOHandler):
         mylog.debug("Initializing index % 5i (% 7i particles)",
                     data_file.file_id, pcount)
         ind = 0
+        if pcount == 0: return None
         with h5py.File(data_file.filename, "r") as f:
             if not f.keys(): return None
             pos = np.empty((pcount, 3), dtype="float64")
-            pos = data_file.ds.arr(pos, "code_length")
+            units = parse_h5_attr(f["particle_position_x"], "units")
             dx = np.finfo(f['particle_position_x'].dtype).eps
-            dx = 2.0*self.ds.quan(dx, "code_length")
+            dx = 2.0 * self.ds.quan(dx, units).to("code_length")
             pos[:,0] = f["particle_position_x"].value
             pos[:,1] = f["particle_position_y"].value
             pos[:,2] = f["particle_position_z"].value
+            pos = data_file.ds.arr(pos, units). to("code_length")
+            dle = self.ds.domain_left_edge.to("code_length")
+            dre = self.ds.domain_right_edge.to("code_length")
             # These are 32 bit numbers, so we give a little lee-way.
             # Otherwise, for big sets of particles, we often will bump into the
             # domain edges.  This helps alleviate that.
-            np.clip(pos, self.ds.domain_left_edge + dx,
-                         self.ds.domain_right_edge - dx, pos)
-            if np.any(pos.min(axis=0) < self.ds.domain_left_edge) or \
-               np.any(pos.max(axis=0) > self.ds.domain_right_edge):
+            np.clip(pos, dle + dx, dre - dx, pos)
+            if np.any(pos.min(axis=0) < dle) or \
+               np.any(pos.max(axis=0) > dre):
                 raise YTDomainOverflow(pos.min(axis=0),
                                        pos.max(axis=0),
-                                       self.ds.domain_left_edge,
-                                       self.ds.domain_right_edge)
+                                       dle, dre)
             regions.add_data_file(pos, data_file.file_id)
             morton[ind:ind+pos.shape[0]] = compute_morton(
-                pos[:,0], pos[:,1], pos[:,2],
-                data_file.ds.domain_left_edge,
-                data_file.ds.domain_right_edge)
+                pos[:,0], pos[:,1], pos[:,2], dle, dre)
         return morton
 
     def _count_particles(self, data_file):
@@ -111,6 +117,7 @@ class IOHandlerHaloCatalogHDF5(BaseIOHandler):
     def _identify_fields(self, data_file):
         with h5py.File(data_file.filename, "r") as f:
             fields = [("halos", field) for field in f]
-            units = dict([(("halos", field), 
-                           f[field].attrs["units"]) for field in f])
+            units = dict([(("halos", field),
+                           parse_h5_attr(f[field], "units"))
+                          for field in f])
         return fields, units
