@@ -19,7 +19,10 @@ import numpy as np
 from yt.utilities.io_handler import \
     BaseIOHandler
 from yt.utilities.logger import ytLogger as mylog
+from yt.utilities.physical_ratios import cm_per_km, cm_per_mpc
 import yt.utilities.fortran_utils as fpu
+from yt.utilities.lib.cosmology_time import \
+    get_ramses_ages
 from yt.extern.six import PY3
 
 if PY3:
@@ -35,7 +38,6 @@ class IOHandlerRAMSES(BaseIOHandler):
         # Each domain subset will contain a hydro_offset array, which gives
         # pointers to level-by-level hydro information
         tr = defaultdict(list)
-        cp = 0
         for chunk in chunks:
             for subset in chunk.objs:
                 # Now we read the entire thing
@@ -102,4 +104,15 @@ class IOHandlerRAMSES(BaseIOHandler):
             tr[field] = fpu.read_vector(f, dt)
             if field[1].startswith("particle_position"):
                 np.divide(tr[field], subset.domain.ds["boxlen"], tr[field])
+            cosmo = subset.domain.ds.cosmological_simulation
+            if cosmo == 1 and field[1] == "particle_age":
+                tf = subset.domain.ds.t_frw
+                dtau = subset.domain.ds.dtau
+                tauf = subset.domain.ds.tau_frw
+                tsim = subset.domain.ds.time_simu
+                h100 = subset.domain.ds.hubble_constant
+                nOver2 = subset.domain.ds.n_frw/2
+                t_scale = 1./(h100 * 100 * cm_per_km / cm_per_mpc)/subset.domain.ds['unit_t']
+                ages = tr[field]
+                tr[field] = get_ramses_ages(tf,tauf,dtau,tsim,t_scale,ages,nOver2,len(ages))            
         return tr
