@@ -13,17 +13,13 @@ Enzo-specific IO functions
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-from collections import defaultdict
-
-import os
 import numpy as np
 
 from yt.utilities.io_handler import \
-    BaseIOHandler, _axis_ids
+    BaseIOHandler
 from yt.utilities.logger import ytLogger as mylog
-from yt.units.yt_array import YTArray
 from yt.utilities.lib.geometry_utils import compute_morton
-from yt.utilities.exceptions import *
+from yt.utilities.exceptions import YTDomainOverflow
 
 class IOHandlerStream(BaseIOHandler):
 
@@ -48,11 +44,12 @@ class IOHandlerStream(BaseIOHandler):
 
     def _read_fluid_selection(self, chunks, selector, fields, size):
         chunks = list(chunks)
-        if any((ftype not in ("gas",) for ftype, fname in fields)):
+        if any((ftype not in self.ds.fluid_types for ftype, fname in fields)):
             raise NotImplementedError
         rv = {}
         for field in fields:
             rv[field] = self.ds.arr(np.empty(size, dtype="float64"))
+
         ng = sum(len(c.objs) for c in chunks)
         mylog.debug("Reading %s cells of %s fields in %s blocks",
                     size, [f2 for f1, f2 in fields], ng)
@@ -221,7 +218,7 @@ class IOHandlerStreamHexahedral(BaseIOHandler):
             rv[field] = np.empty(size, dtype="float64")
         ngrids = sum(len(chunk.objs) for chunk in chunks)
         mylog.debug("Reading %s cells of %s fields in %s blocks",
-                    size, [fname for ftype, fname in fields], ngrids)
+                    size, [fn for ft, fn in fields], ngrids)
         for field in fields:
             ind = 0
             ftype, fname = field
@@ -255,3 +252,47 @@ class IOHandlerStreamOctree(BaseIOHandler):
                         subset.domain_id - subset._domain_offset][field]
                 subset.fill(field_vals, rv, selector, ind)
         return rv
+
+
+class IOHandlerStreamUnstructured(BaseIOHandler):
+    _dataset_type = "stream_unstructured"
+
+    def __init__(self, ds):
+        self.fields = ds.stream_handler.fields
+        super(IOHandlerStreamUnstructured, self).__init__(ds)
+
+    def _read_fluid_selection(self, chunks, selector, fields, size):
+        chunks = list(chunks)
+        rv = {}
+        for field in fields:
+            ftype, fname = field
+            if ftype == "all":
+                ci = np.concatenate([mesh.connectivity_indices
+                                     for mesh in self.ds.index.mesh_union])
+            else:
+                mesh_id = int(ftype[-1]) - 1
+                m = self.ds.index.meshes[mesh_id]
+                ci = m.connectivity_indices
+            num_elem = ci.shape[0]
+            if fname in self.ds._node_fields:
+                nodes_per_element = ci.shape[1]
+                rv[field] = np.empty((num_elem, nodes_per_element), dtype="float64")
+            else:
+                rv[field] = np.empty(num_elem, dtype="float64")
+        for field in fields:
+            ind = 0
+            ftype, fname = field
+            if ftype == "all":
+                objs = [mesh for mesh in self.ds.index.mesh_union]
+            else:
+                mesh_ids = [int(ftype[-1])]
+                chunk = chunks[mesh_ids[0] - 1]
+                objs = chunk.objs
+            for g in objs:
+                ds = self.fields[g.mesh_id].get(field, None)
+                if ds is None:
+                    f = ('connect%d' % (g.mesh_id + 1), fname)
+                    ds = self.fields[g.mesh_id][f]
+                ind += g.select(selector, ds, rv[field], ind) # caches
+        return rv
+
