@@ -22,7 +22,7 @@ import glob
 import os
 
 from yt.frontends.sph.data_structures import \
-    ParticleDataset
+    SPHDataset
 from yt.funcs import deprecate
 from yt.geometry.particle_geometry_handler import \
     ParticleIndex
@@ -31,7 +31,8 @@ from yt.data_objects.static_output import \
 from yt.utilities.cosmology import \
     Cosmology
 from yt.utilities.physical_constants import \
-    G, \
+    G
+from yt.utilities.physical_ratios import \
     cm_per_kpc
 
 from .fields import \
@@ -53,7 +54,7 @@ class TipsyFile(ParticleFile):
     def _calculate_offsets(self, field_list):
         self.field_offsets = self.io._calculate_particle_offsets(self)
 
-class TipsyDataset(ParticleDataset):
+class TipsyDataset(SPHDataset):
     _index_class = ParticleIndex
     _file_class = TipsyFile
     _field_info_class = TipsyFieldInfo
@@ -73,15 +74,15 @@ class TipsyDataset(ParticleDataset):
                  parameter_file=None,
                  cosmology_parameters=None,
                  n_ref=64, over_refine_factor=1,
+                 kernel_name=None,
                  bounding_box=None,
-                 units_override=None):
+                 units_override=None,
+                 unit_system="cgs"):
         # Because Tipsy outputs don't have a fixed domain boundary, one can
         # specify a bounding box which effectively gives a domain_left_edge
         # and domain_right_edge
         self.bounding_box = bounding_box
         self.filter_bbox = (bounding_box is not None)
-        self.n_ref = n_ref
-        self.over_refine_factor = over_refine_factor
         if field_dtypes is None:
             field_dtypes = {}
         success, self.endian = self._validate_header(filename)
@@ -89,7 +90,6 @@ class TipsyDataset(ParticleDataset):
             print("SOMETHING HAS GONE WRONG.  NBODIES != SUM PARTICLES.")
             print("%s != (%s == %s + %s + %s)" % (
                 self.parameters['nbodies'],
-                tot,
                 self.parameters['nsph'],
                 self.parameters['ndark'],
                 self.parameters['nstar']))
@@ -112,7 +112,10 @@ class TipsyDataset(ParticleDataset):
         if units_override is not None:
             raise RuntimeError("units_override is not supported for TipsyDataset. "+
                                "Use unit_base instead.")
-        super(TipsyDataset, self).__init__(filename, dataset_type)
+        super(TipsyDataset, self).__init__(
+            filename, dataset_type=dataset_type, unit_system=unit_system,
+            n_ref=n_ref, over_refine_factor=over_refine_factor,
+            kernel_name=kernel_name)
 
     def __repr__(self):
         return os.path.basename(self.parameter_filename)
@@ -223,8 +226,8 @@ class TipsyDataset(ParticleDataset):
         if self.bounding_box is None and (
                 self.domain_left_edge is None or
                 self.domain_right_edge is None):
-            self.domain_left_edge = np.nan
-            self.domain_right_edge = np.nan
+            self.domain_left_edge = np.array([np.nan, np.nan, np.nan])
+            self.domain_right_edge = np.array([np.nan, np.nan, np.nan])
             self.index
         super(TipsyDataset, self)._set_derived_attrs()
 
