@@ -14,15 +14,18 @@ Tipsy tests using the AGORA dataset
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-from yt.testing import *
+from collections import OrderedDict
+
+from yt.testing import \
+    assert_equal, \
+    requires_file
 from yt.utilities.answer_testing.framework import \
     requires_ds, \
-    small_patch_amr, \
-    big_patch_amr, \
     data_dir_load, \
-    PixelizedProjectionValuesTest, \
+    sph_answer, \
+    create_obj, \
     FieldValuesTest, \
-    create_obj
+    PixelizedProjectionValuesTest
 from yt.frontends.tipsy.api import TipsyDataset
 
 _fields = (("deposit", "all_density"),
@@ -42,13 +45,13 @@ def test_pkdgrav():
                   unit_base = {'length': (60.0, "Mpccm/h")},
                   n_ref = 64)
     ds = data_dir_load(pkdgrav, TipsyDataset, (), kwargs)
-    yield assert_equal, str(ds), "halo1e11_run1.00400"
+    assert_equal(str(ds), "halo1e11_run1.00400")
     dso = [ None, ("sphere", ("c", (0.3, 'unitary')))]
     dd = ds.all_data()
-    yield assert_equal, dd["Coordinates"].shape, (26847360, 3)
+    assert_equal(dd["Coordinates"].shape, (26847360, 3))
     tot = sum(dd[ptype,"Coordinates"].shape[0]
               for ptype in ds.particle_types if ptype != "all")
-    yield assert_equal, tot, 26847360
+    assert_equal(tot, 26847360)
     for dobj_name in dso:
         for field in _fields:
             for axis in [0, 1, 2]:
@@ -60,11 +63,11 @@ def test_pkdgrav():
         dobj = create_obj(ds, dobj_name)
         s1 = dobj["ones"].sum()
         s2 = sum(mask.sum() for block, mask in dobj.blocks)
-        yield assert_equal, s1, s2
+        assert_equal(s1, s2)
 
-gasoline = "agora_1e11.00400/agora_1e11.00400"
-@requires_ds(gasoline, big_data = True, file_check = True)
-def test_gasoline():
+gasoline_dmonly = "agora_1e11.00400/agora_1e11.00400"
+@requires_ds(gasoline_dmonly, big_data = True, file_check = True)
+def test_gasoline_dmonly():
     cosmology_parameters = dict(current_redshift = 0.0,
                                 omega_lambda = 0.728,
                                 omega_matter = 0.272,
@@ -72,14 +75,14 @@ def test_gasoline():
     kwargs = dict(cosmology_parameters = cosmology_parameters,
                   unit_base = {'length': (60.0, "Mpccm/h")},
                   n_ref = 64)
-    ds = data_dir_load(gasoline, TipsyDataset, (), kwargs)
-    yield assert_equal, str(ds), "agora_1e11.00400"
+    ds = data_dir_load(gasoline_dmonly, TipsyDataset, (), kwargs)
+    assert_equal(str(ds), "agora_1e11.00400")
     dso = [ None, ("sphere", ("c", (0.3, 'unitary')))]
     dd = ds.all_data()
-    yield assert_equal, dd["Coordinates"].shape, (10550576, 3)
+    assert_equal(dd["Coordinates"].shape, (10550576, 3))
     tot = sum(dd[ptype,"Coordinates"].shape[0]
               for ptype in ds.particle_types if ptype != "all")
-    yield assert_equal, tot, 10550576
+    assert_equal(tot, 10550576)
     for dobj_name in dso:
         for field in _fields:
             for axis in [0, 1, 2]:
@@ -91,9 +94,29 @@ def test_gasoline():
         dobj = create_obj(ds, dobj_name)
         s1 = dobj["ones"].sum()
         s2 = sum(mask.sum() for block, mask in dobj.blocks)
-        yield assert_equal, s1, s2
+        assert_equal(s1, s2)
 
+tg_fields = OrderedDict(
+    [
+        (('gas', 'density'), None),
+        (('gas', 'temperature'), None),
+        (('gas', 'temperature'), ('gas', 'density')),
+        (('gas', 'velocity_magnitude'), None),
+        (('gas', 'Fe_fraction'), None),
+        (('Stars', 'Metals'), None),
+    ]
+)
 
+tipsy_gal = 'TipsyGalaxy/galaxy.00300'
+@requires_ds(tipsy_gal)
+def test_tipsy_galaxy():
+    ds = data_dir_load(tipsy_gal)
+    for test in sph_answer(ds, 'galaxy.00300', 315372, tg_fields):
+        test_tipsy_galaxy.__name__ = test.description
+        yield test
+
+@requires_file(gasoline_dmonly)
 @requires_file(pkdgrav)
 def test_TipsyDataset():
     assert isinstance(data_dir_load(pkdgrav), TipsyDataset)
+    assert isinstance(data_dir_load(gasoline_dmonly), TipsyDataset)
