@@ -19,7 +19,10 @@ import numpy as np
 from yt.extern.six import \
     u
 from yt.funcs import \
-    mylog
+    mylog, \
+    parse_h5_attr
+from yt.geometry.selection_routines import \
+    GridSelector
 from yt.utilities.exceptions import \
     YTDomainOverflow
 from yt.utilities.io_handler import \
@@ -36,20 +39,19 @@ class IOHandlerYTNonspatialhdf5(BaseIOHandler):
 
     def _read_fluid_selection(self, g, selector, fields):
         rv = {}
-        if selector.__class__.__name__ == "GridSelector":
+        if isinstance(selector, GridSelector):
             if g.id in self._cached_fields:
                 gf = self._cached_fields[g.id]
                 rv.update(gf)
             if len(rv) == len(fields): return rv
             f = h5py.File(u(g.filename), "r")
-            gds = f["data"]
             for field in fields:
                 if field in rv:
                     self._hits += 1
                     continue
                 self._misses += 1
                 ftype, fname = field
-                rv[(ftype, fname)] = gds[fname].value
+                rv[(ftype, fname)] = f[ftype][fname].value
             if self._cache_on:
                 for gid in rv:
                     self._cached_fields.setdefault(gid, {})
@@ -69,7 +71,7 @@ class IOHandlerYTGridHDF5(BaseIOHandler):
         rv = {}
         # Now we have to do something unpleasant
         chunks = list(chunks)
-        if selector.__class__.__name__ == "GridSelector":
+        if isinstance(selector, GridSelector):
             if not (len(chunks) == len(chunks[0].objs) == 1):
                 raise RuntimeError
             g = chunks[0].objs[0]
@@ -131,6 +133,7 @@ class IOHandlerYTGridHDF5(BaseIOHandler):
         return rv
 
     def _read_particle_coords(self, chunks, ptf):
+        pn = "particle_position_%s"
         chunks = list(chunks)
         for chunk in chunks:
             f = None
@@ -141,9 +144,10 @@ class IOHandlerYTGridHDF5(BaseIOHandler):
                 if g.NumberOfParticles == 0:
                     continue
                 for ptype, field_list in sorted(ptf.items()):
-                    pn = "particle_position_%s"
-                    x, y, z = (np.asarray(f[ptype][pn % ax].value, dtype="=f8")
-                               for ax in 'xyz')
+                    units = parse_h5_attr(f[ptype][pn % "x"], "units")
+                    x, y, z = \
+                      (self.ds.arr(f[ptype][pn % ax].value.astype("float64"), units)
+                       for ax in "xyz")
                     for field in field_list:
                         if np.asarray(f[ptype][field]).ndim > 1:
                             self._array_fields[field] = f[ptype][field].shape
@@ -151,6 +155,7 @@ class IOHandlerYTGridHDF5(BaseIOHandler):
             if f: f.close()
 
     def _read_particle_fields(self, chunks, ptf, selector):
+        pn = "particle_position_%s"
         chunks = list(chunks)
         for chunk in chunks: # These should be organized by grid filename
             f = None
@@ -161,9 +166,10 @@ class IOHandlerYTGridHDF5(BaseIOHandler):
                 if g.NumberOfParticles == 0:
                     continue
                 for ptype, field_list in sorted(ptf.items()):
-                    pn = "particle_position_%s"
-                    x, y, z = (np.asarray(f[ptype][pn % ax].value, dtype="=f8")
-                               for ax in 'xyz')
+                    units = parse_h5_attr(f[ptype][pn % "x"], "units")
+                    x, y, z = \
+                      (self.ds.arr(f[ptype][pn % ax].value.astype("float64"), units)
+                       for ax in "xyz")
                     mask = selector.select_points(x, y, z, 0.0)
                     if mask is None: continue
                     for field in field_list:
@@ -189,9 +195,10 @@ class IOHandlerYTDataContainerHDF5(BaseIOHandler):
                 for ptype, field_list in sorted(ptf.items()):
                     pcount = data_file.total_particles[ptype]
                     if pcount == 0: continue
-                    x = _get_position_array(ptype, f, "x")
-                    y = _get_position_array(ptype, f, "y")
-                    z = _get_position_array(ptype, f, "z")
+                    units = _get_position_array_units(ptype, f, "x")
+                    x, y, z = \
+                      (self.ds.arr(_get_position_array(ptype, f, ax), units)
+                       for ax in "xyz")
                     yield ptype, (x, y, z)
 
     def _read_particle_fields(self, chunks, ptf, selector):
@@ -204,9 +211,10 @@ class IOHandlerYTDataContainerHDF5(BaseIOHandler):
         for data_file in sorted(data_files):
             with h5py.File(data_file.filename, "r") as f:
                 for ptype, field_list in sorted(ptf.items()):
-                    x = _get_position_array(ptype, f, "x")
-                    y = _get_position_array(ptype, f, "y")
-                    z = _get_position_array(ptype, f, "z")
+                    units = _get_position_array_units(ptype, f, "x")
+                    x, y, z = \
+                      (self.ds.arr(_get_position_array(ptype, f, ax), units)
+                       for ax in "xyz")
                     mask = selector.select_points(x, y, z, 0.0)
                     del x, y, z
                     if mask is None: continue
@@ -225,31 +233,33 @@ class IOHandlerYTDataContainerHDF5(BaseIOHandler):
             for ptype in all_count:
                 if ptype not in f or all_count[ptype] == 0: continue
                 pos = np.empty((all_count[ptype], 3), dtype="float64")
-                pos = data_file.ds.arr(pos, "code_length")
+                units = _get_position_array_units(ptype, f, "x")
                 if ptype == "grid":
                     dx = f["grid"]["dx"].value.min()
+                    dx = self.ds.quan(
+                        dx, parse_h5_attr(f["grid"]["dx"], "units")).to("code_length")
                 else:
                     dx = 2. * np.finfo(f[ptype]["particle_position_x"].dtype).eps
-                dx = self.ds.quan(dx, "code_length")
+                    dx = self.ds.quan(dx, units).to("code_length")
                 pos[:,0] = _get_position_array(ptype, f, "x")
                 pos[:,1] = _get_position_array(ptype, f, "y")
                 pos[:,2] = _get_position_array(ptype, f, "z")
+                pos = self.ds.arr(pos, units).to("code_length")
+                dle = self.ds.domain_left_edge.to("code_length")
+                dre = self.ds.domain_right_edge.to("code_length")
+
                 # These are 32 bit numbers, so we give a little lee-way.
                 # Otherwise, for big sets of particles, we often will bump into the
                 # domain edges.  This helps alleviate that.
-                np.clip(pos, self.ds.domain_left_edge + dx,
-                             self.ds.domain_right_edge - dx, pos)
-                if np.any(pos.min(axis=0) < self.ds.domain_left_edge) or \
-                   np.any(pos.max(axis=0) > self.ds.domain_right_edge):
+                np.clip(pos, dle + dx, dre - dx, pos)
+                if np.any(pos.min(axis=0) < dle) or \
+                   np.any(pos.max(axis=0) > dre):
                     raise YTDomainOverflow(pos.min(axis=0),
                                            pos.max(axis=0),
-                                           self.ds.domain_left_edge,
-                                           self.ds.domain_right_edge)
+                                           dle, dre)
                 regions.add_data_file(pos, data_file.file_id)
                 morton[ind:ind+pos.shape[0]] = compute_morton(
-                    pos[:,0], pos[:,1], pos[:,2],
-                    data_file.ds.domain_left_edge,
-                    data_file.ds.domain_right_edge)
+                    pos[:,0], pos[:,1], pos[:,2], dle, dre)
                 ind += pos.shape[0]
         return morton
 
@@ -263,7 +273,7 @@ class IOHandlerYTDataContainerHDF5(BaseIOHandler):
             for ptype in f:
                 fields.extend([(ptype, str(field)) for field in f[ptype]])
                 units.update(dict([((ptype, str(field)), 
-                                    f[ptype][field].attrs["units"])
+                                    parse_h5_attr(f[ptype][field], "units"))
                                    for field in f[ptype]]))
         return fields, units
 
@@ -285,7 +295,7 @@ class IOHandlerYTSpatialPlotHDF5(IOHandlerYTDataContainerHDF5):
                     x = _get_position_array(ptype, f, "px")
                     y = _get_position_array(ptype, f, "py")
                     z = np.zeros(x.size, dtype="float64") + \
-                      self.ds.domain_left_edge[2].in_cgs().d
+                      self.ds.domain_left_edge[2].to("code_length").d
                     yield ptype, (x, y, z)
 
     def _read_particle_fields(self, chunks, ptf, selector):
@@ -302,7 +312,7 @@ class IOHandlerYTSpatialPlotHDF5(IOHandlerYTDataContainerHDF5):
                     x = _get_position_array(ptype, f, "px")
                     y = _get_position_array(ptype, f, "py")
                     z = np.zeros(all_count[ptype], dtype="float64") + \
-                      self.ds.domain_left_edge[2].in_cgs().d
+                      self.ds.domain_left_edge[2].to("code_length").d
                     mask = selector.select_points(x, y, z, 0.0)
                     del x, y, z
                     if mask is None: continue
@@ -321,32 +331,32 @@ class IOHandlerYTSpatialPlotHDF5(IOHandlerYTDataContainerHDF5):
             for ptype in all_count:
                 if ptype not in f or all_count[ptype] == 0: continue
                 pos = np.empty((all_count[ptype], 3), dtype="float64")
-                pos = data_file.ds.arr(pos, "code_length")
+                pos = self.ds.arr(pos, "code_length")
                 if ptype == "grid":
                     dx = f["grid"]["pdx"].value.min()
+                    dx = self.ds.quan(
+                        dx, parse_h5_attr(f["grid"]["pdx"], "units")).to("code_length")
                 else:
                     raise NotImplementedError
-                dx = self.ds.quan(dx, "code_length")
                 pos[:,0] = _get_position_array(ptype, f, "px")
                 pos[:,1] = _get_position_array(ptype, f, "py")
                 pos[:,2] = np.zeros(all_count[ptype], dtype="float64") + \
-                  self.ds.domain_left_edge[2].in_cgs().d
+                  self.ds.domain_left_edge[2].to("code_length").d
+                dle = self.ds.domain_left_edge.to("code_length")
+                dre = self.ds.domain_right_edge.to("code_length")
+
                 # These are 32 bit numbers, so we give a little lee-way.
                 # Otherwise, for big sets of particles, we often will bump into the
                 # domain edges.  This helps alleviate that.
-                np.clip(pos, self.ds.domain_left_edge + dx,
-                             self.ds.domain_right_edge - dx, pos)
-                if np.any(pos.min(axis=0) < self.ds.domain_left_edge) or \
-                   np.any(pos.max(axis=0) > self.ds.domain_right_edge):
+                np.clip(pos, dle + dx, dre - dx, pos)
+                if np.any(pos.min(axis=0) < dle) or \
+                   np.any(pos.max(axis=0) > dre):
                     raise YTDomainOverflow(pos.min(axis=0),
                                            pos.max(axis=0),
-                                           self.ds.domain_left_edge,
-                                           self.ds.domain_right_edge)
+                                           dre, dle)
                 regions.add_data_file(pos, data_file.file_id)
                 morton[ind:ind+pos.shape[0]] = compute_morton(
-                    pos[:,0], pos[:,1], pos[:,2],
-                    data_file.ds.domain_left_edge,
-                    data_file.ds.domain_right_edge)
+                    pos[:,0], pos[:,1], pos[:,2], dle, dre)
                 ind += pos.shape[0]
         return morton
 
@@ -356,3 +366,10 @@ def _get_position_array(ptype, f, ax):
     else:
         pos_name = "particle_position_"
     return f[ptype][pos_name + ax].value.astype("float64")
+
+def _get_position_array_units(ptype, f, ax):
+    if ptype == "grid":
+        pos_name = ""
+    else:
+        pos_name = "particle_position_"
+    return parse_h5_attr(f[ptype][pos_name + ax], "units")
