@@ -15,23 +15,19 @@ Coordinate handler base class.
 #-----------------------------------------------------------------------------
 
 import numpy as np
-import abc
 import weakref
 from numbers import Number
 
-from yt.funcs import *
-from yt.fields.field_info_container import \
-    NullFunc, FieldInfoContainer
-from yt.utilities.io_handler import io_registry
-from yt.utilities.logger import ytLogger as mylog
-from yt.utilities.parallel_tools.parallel_analysis_interface import \
-    ParallelAnalysisInterface
-from yt.utilities.lib.pixelization_routines import \
-    pixelize_cylinder
-import yt.visualization._MPL as _MPL
+from yt.extern.six import string_types
+from yt.funcs import \
+    validate_width_tuple, \
+    fix_unitary, \
+    iterable
 from yt.units.yt_array import \
     YTArray, YTQuantity
-from yt.extern.six import string_types
+from yt.utilities.exceptions import \
+    YTCoordinateNotImplemented, \
+    YTInvalidWidthError
 
 def _unknown_coord(field, data):
     raise YTCoordinateNotImplemented
@@ -45,6 +41,12 @@ def _get_coord_fields(axi, units = "code_length"):
         return data._reshape_vals(rv)
     return _dds, _coords
 
+def _get_vert_fields(axi, units = "code_length"):
+    def _vert(field, data):
+        rv = data.ds.arr(data.fcoords_vertex[...,axi].copy(), units)
+        return rv
+    return _vert
+
 def validate_iterable_width(width, ds, unit=None):
     if isinstance(width[0], tuple) and isinstance(width[1], tuple):
         validate_width_tuple(width[0])
@@ -69,7 +71,8 @@ def validate_iterable_width(width, ds, unit=None):
                     ds.quan(width[0], fix_unitary(width[1])))
 
 class CoordinateHandler(object):
-    
+    name = None
+
     def __init__(self, ds, ordering):
         self.ds = weakref.proxy(ds)
         self.axis_order = ordering
@@ -83,6 +86,9 @@ class CoordinateHandler(object):
         # pixelizer
         raise NotImplementedError
 
+    def pixelize_line(self, field, start_point, end_point, npoints):
+        raise NotImplementedError
+
     def distance(self, start, end):
         p1 = self.convert_to_cartesian(start)
         p2 = self.convert_to_cartesian(end)
@@ -130,10 +136,13 @@ class CoordinateHandler(object):
         self._axis_id = ai
         return ai
 
+    _image_axis_name = None
     @property
     def image_axis_name(self):
         # Default
-        rv = {}
+        if self._image_axis_name is not None:
+            return self._image_axis_name
+        self._image_axis_name = rv = {}
         for i in range(3):
             rv[i] = (self.axis_name[self.x_axis[i]],
                      self.axis_name[self.y_axis[i]])
@@ -259,4 +268,3 @@ def cylindrical_to_cartesian(coord, center = (0,0,0)):
     c2[...,1] = np.sin(coord[...,0]) * coord[...,1] + center[1]
     c2[...,2] = coord[...,2]
     return c2
-
