@@ -1,5 +1,5 @@
 """
-Cylindrical fields
+Definitions for cylindrical coordinate systems
 
 
 
@@ -21,14 +21,14 @@ from .coordinate_handler import \
     _get_coord_fields, \
     cylindrical_to_cartesian, \
     cartesian_to_cylindrical
-import yt.visualization._MPL as _MPL
 from yt.utilities.lib.pixelization_routines import \
-    pixelize_cylinder
+    pixelize_cartesian, pixelize_cylinder
 #
 # Cylindrical fields
 #
 
 class CylindricalCoordinateHandler(CoordinateHandler):
+    name = "cylindrical"
 
     def __init__(self, ds, ordering = ('r', 'z', 'theta')):
         super(CylindricalCoordinateHandler, self).__init__(ds, ordering)
@@ -39,57 +39,59 @@ class CylindricalCoordinateHandler(CoordinateHandler):
 
     def setup_fields(self, registry):
         # return the fields for r, z, theta
-        registry.add_field(("index", "dx"), function=_unknown_coord)
-        registry.add_field(("index", "dy"), function=_unknown_coord)
-        registry.add_field(("index", "x"), function=_unknown_coord)
-        registry.add_field(("index", "y"), function=_unknown_coord)
+        registry.add_field(("index", "dx"), sampling_type="cell",  function=_unknown_coord)
+        registry.add_field(("index", "dy"), sampling_type="cell",  function=_unknown_coord)
+        registry.add_field(("index", "x"), sampling_type="cell",  function=_unknown_coord)
+        registry.add_field(("index", "y"), sampling_type="cell",  function=_unknown_coord)
         f1, f2 = _get_coord_fields(self.axis_id['r'])
-        registry.add_field(("index", "dr"), function = f1,
+        registry.add_field(("index", "dr"), sampling_type="cell",  function = f1,
                            display_field = False,
                            units = "code_length")
-        registry.add_field(("index", "r"), function = f2,
+        registry.add_field(("index", "r"), sampling_type="cell",  function = f2,
                            display_field = False,
                            units = "code_length")
 
         f1, f2 = _get_coord_fields(self.axis_id['z'])
-        registry.add_field(("index", "dz"), function = f1,
+        registry.add_field(("index", "dz"), sampling_type="cell",  function = f1,
                            display_field = False,
                            units = "code_length")
-        registry.add_field(("index", "z"), function = f2,
+        registry.add_field(("index", "z"), sampling_type="cell",  function = f2,
                            display_field = False,
                            units = "code_length")
 
         f1, f2 = _get_coord_fields(self.axis_id['theta'], "")
-        registry.add_field(("index", "dtheta"), function = f1,
+        registry.add_field(("index", "dtheta"), sampling_type="cell",  function = f1,
                            display_field = False,
                            units = "")
-        registry.add_field(("index", "theta"), function = f2,
+        registry.add_field(("index", "theta"), sampling_type="cell",  function = f2,
                            display_field = False,
                            units = "")
 
         def _CylindricalVolume(field, data):
-            return data["index", "dtheta"] \
-                 * data["index", "r"] \
-                 * data["index", "dr"] \
-                 * data["index", "dz"]
-        registry.add_field(("index", "cell_volume"),
+            r = data["index", "r"]
+            dr = data["index", "dr"]
+            vol = 0.5*((r+0.5*dr)**2-(r-0.5*dr)**2)
+            vol *= data["index", "dtheta"]
+            vol *= data["index", "dz"]
+            return vol
+        registry.add_field(("index", "cell_volume"), sampling_type="cell",
                  function=_CylindricalVolume,
                  units = "code_length**3")
 
         def _path_r(field, data):
             return data["index", "dr"]
-        registry.add_field(("index", "path_element_r"),
+        registry.add_field(("index", "path_element_r"), sampling_type="cell", 
                  function = _path_r,
                  units = "code_length")
         def _path_theta(field, data):
             # Note: this already assumes cell-centered
             return data["index", "r"] * data["index", "dtheta"]
-        registry.add_field(("index", "path_element_theta"),
+        registry.add_field(("index", "path_element_theta"), sampling_type="cell", 
                  function = _path_theta,
                  units = "code_length")
         def _path_z(field, data):
             return data["index", "dz"]
-        registry.add_field(("index", "path_element_z"),
+        registry.add_field(("index", "path_element_z"), sampling_type="cell", 
                  function = _path_z,
                  units = "code_length")
 
@@ -113,19 +115,22 @@ class CylindricalCoordinateHandler(CoordinateHandler):
         period[1] = self.period[self.y_axis[dim]]
         if hasattr(period, 'in_units'):
             period = period.in_units("code_length").d
-        buff = _MPL.Pixelize(data_source['px'], data_source['py'],
-                             data_source['pdx'], data_source['pdy'],
-                             data_source[field], size[0], size[1],
-                             bounds, int(antialias),
-                             period, int(periodic)).transpose()
+        buff = np.zeros(size, dtype="f8")
+        pixelize_cartesian(buff, data_source['px'], data_source['py'],
+                                  data_source['pdx'], data_source['pdy'],
+                                  data_source[field],
+                                  bounds, int(antialias),
+                                  period, int(periodic))
         return buff
 
     def _cyl_pixelize(self, data_source, field, bounds, size, antialias):
-        buff = pixelize_cylinder(data_source['px'],
-                                 data_source['pdx'],
-                                 data_source['py'],
-                                 data_source['pdy'],
-                                 size, data_source[field], bounds)
+        buff = np.zeros((size[1], size[0]), dtype="f8")
+        pixelize_cylinder(buff,
+                          data_source['px'],
+                          data_source['pdx'],
+                          data_source['py'],
+                          data_source['pdy'],
+                          data_source[field], bounds)
         return buff
 
     _x_pairs = (('r', 'theta'), ('z', 'r'), ('theta', 'r'))
