@@ -37,59 +37,61 @@ class GeographicCoordinateHandler(CoordinateHandler):
 
     def setup_fields(self, registry):
         # return the fields for r, z, theta
-        registry.add_field(("index", "dx"), function=_unknown_coord)
-        registry.add_field(("index", "dy"), function=_unknown_coord)
-        registry.add_field(("index", "dz"), function=_unknown_coord)
-        registry.add_field(("index", "x"), function=_unknown_coord)
-        registry.add_field(("index", "y"), function=_unknown_coord)
-        registry.add_field(("index", "z"), function=_unknown_coord)
+        registry.add_field(("index", "dx"), sampling_type="cell",  function=_unknown_coord)
+        registry.add_field(("index", "dy"), sampling_type="cell",  function=_unknown_coord)
+        registry.add_field(("index", "dz"), sampling_type="cell",  function=_unknown_coord)
+        registry.add_field(("index", "x"), sampling_type="cell",  function=_unknown_coord)
+        registry.add_field(("index", "y"), sampling_type="cell",  function=_unknown_coord)
+        registry.add_field(("index", "z"), sampling_type="cell",  function=_unknown_coord)
         f1, f2 = _get_coord_fields(self.axis_id['latitude'], "")
-        registry.add_field(("index", "dlatitude"), function = f1,
+        registry.add_field(("index", "dlatitude"), sampling_type="cell",  function = f1,
                            display_field = False,
                            units = "")
-        registry.add_field(("index", "latitude"), function = f2,
+        registry.add_field(("index", "latitude"), sampling_type="cell",  function = f2,
                            display_field = False,
                            units = "")
 
         f1, f2 = _get_coord_fields(self.axis_id['longitude'], "")
-        registry.add_field(("index", "dlongitude"), function = f1,
+        registry.add_field(("index", "dlongitude"), sampling_type="cell",  function = f1,
                            display_field = False,
                            units = "")
-        registry.add_field(("index", "longitude"), function = f2,
+        registry.add_field(("index", "longitude"), sampling_type="cell",  function = f2,
                            display_field = False,
                            units = "")
 
         f1, f2 = _get_coord_fields(self.axis_id[self.radial_axis])
-        registry.add_field(("index", "d%s" % (self.radial_axis,)), function = f1,
+        registry.add_field(("index", "d%s" % (self.radial_axis,)), sampling_type="cell",  function = f1,
                            display_field = False,
                            units = "code_length")
-        registry.add_field(("index", self.radial_axis), function = f2,
+        registry.add_field(("index", self.radial_axis), sampling_type="cell",  function = f2,
                            display_field = False,
                            units = "code_length")
 
         def _SphericalVolume(field, data):
-            # r**2 sin theta dr dtheta dphi
             # We can use the transformed coordinates here.
-            vol = data["index", "r"]**2.0
-            vol *= data["index", "dr"]
-            vol *= np.sin(data["index", "theta"])
-            vol *= data["index", "dtheta"]
+            # Here we compute the spherical volume element exactly
+            r = data["index", "r"]
+            dr = data["index", "dr"]
+            theta = data["index", "theta"]
+            dtheta = data["index", "dtheta"]
+            vol = ((r+0.5*dr)**3-(r-0.5*dr)**3) / 3.0
+            vol *= np.cos(theta-0.5*dtheta)-np.cos(theta+0.5*dtheta)
             vol *= data["index", "dphi"]
             return vol
-        registry.add_field(("index", "cell_volume"),
+        registry.add_field(("index", "cell_volume"), sampling_type="cell",
                  function=_SphericalVolume,
                  units = "code_length**3")
 
         def _path_radial_axis(field, data):
             return data["index", "d%s" % self.radial_axis]
-        registry.add_field(("index", "path_element_%s" % self.radial_axis),
+        registry.add_field(("index", "path_element_%s" % self.radial_axis), sampling_type="cell", 
                  function = _path_radial_axis,
                  units = "code_length")
         def _path_latitude(field, data):
             # We use r here explicitly
             return data["index", "r"] * \
                 data["index", "dlatitude"] * np.pi/180.0
-        registry.add_field(("index", "path_element_latitude"),
+        registry.add_field(("index", "path_element_latitude"), sampling_type="cell", 
                  function = _path_latitude,
                  units = "code_length")
         def _path_longitude(field, data):
@@ -97,31 +99,31 @@ class GeographicCoordinateHandler(CoordinateHandler):
             return data["index", "r"] \
                     * data["index", "dlongitude"] * np.pi/180.0 \
                     * np.sin((data["index", "latitude"] + 90.0) * np.pi/180.0)
-        registry.add_field(("index", "path_element_longitude"),
+        registry.add_field(("index", "path_element_longitude"), sampling_type="cell", 
                  function = _path_longitude,
                  units = "code_length")
 
         def _latitude_to_theta(field, data):
             # latitude runs from -90 to 90
             return (data["latitude"] + 90) * np.pi/180.0
-        registry.add_field(("index", "theta"),
+        registry.add_field(("index", "theta"), sampling_type="cell", 
                  function = _latitude_to_theta,
                  units = "")
         def _dlatitude_to_dtheta(field, data):
             return data["dlatitude"] * np.pi/180.0
-        registry.add_field(("index", "dtheta"),
+        registry.add_field(("index", "dtheta"), sampling_type="cell", 
                  function = _dlatitude_to_dtheta,
                  units = "")
 
         def _longitude_to_phi(field, data):
             # longitude runs from -180 to 180
             return (data["longitude"] + 180) * np.pi/180.0
-        registry.add_field(("index", "phi"),
+        registry.add_field(("index", "phi"), sampling_type="cell", 
                  function = _longitude_to_phi,
                  units = "")
         def _dlongitude_to_dphi(field, data):
             return data["dlongitude"] * np.pi/180.0
-        registry.add_field(("index", "dphi"),
+        registry.add_field(("index", "dphi"), sampling_type="cell", 
                  function = _dlongitude_to_dphi,
                  units = "")
 
@@ -138,7 +140,7 @@ class GeographicCoordinateHandler(CoordinateHandler):
                 else:
                     surface_height = data.ds.quan(0.0, "code_length")
             return data["altitude"] + surface_height
-        registry.add_field(("index", "r"),
+        registry.add_field(("index", "r"), sampling_type="cell", 
                  function=_altitude_to_radius,
                  units = "code_length")
         registry.alias(("index", "dr"), ("index", "daltitude"))
@@ -203,9 +205,9 @@ class GeographicCoordinateHandler(CoordinateHandler):
         else:
             # We should never get here!
             raise NotImplementedError
-        buff = pixelize_cylinder(r, data_source['pdy'],
-                                 px, pdx,
-                                 size, data_source[field], bounds)
+        buff = np.zeros((size[1], size[0]), dtype="f8")
+        pixelize_cylinder(buff, r, data_source['pdy'],
+                          px, pdx, data_source[field], bounds)
         if do_transpose:
             buff = buff.transpose()
         return buff
@@ -341,7 +343,7 @@ class InternalGeographicCoordinateHandler(GeographicCoordinateHandler):
                     rax = self.axis_id[self.radial_axis]
                     outer_radius = data.ds.domain_right_edge[rax]
             return -1.0 * data["depth"] + outer_radius
-        registry.add_field(("index", "r"),
+        registry.add_field(("index", "r"), sampling_type="cell", 
                  function=_depth_to_radius,
                  units = "code_length")
         registry.alias(("index", "dr"), ("index", "ddepth"))
