@@ -14,20 +14,39 @@ from __future__ import print_function
 #-----------------------------------------------------------------------------
 
 import hashlib
+from yt.extern.six import string_types
 from yt.extern.six.moves import cPickle
 import itertools as it
 import numpy as np
 import importlib
 import os
-from yt.funcs import *
+import unittest
+from yt.funcs import iterable
 from yt.config import ytcfg
-from numpy.testing import assert_array_equal, assert_almost_equal, \
-    assert_approx_equal, assert_array_almost_equal, assert_equal, \
-    assert_array_less, assert_string_equal, assert_array_almost_equal_nulp,\
-    assert_allclose, assert_raises
-from yt.units.yt_array import uconcatenate
-import yt.fields.api as field_api
+# we import this in a weird way from numpy.testing to avoid triggering
+# flake8 errors from the unused imports. These test functions are imported
+# elsewhere in yt from here so we want them to be imported here.
+from numpy.testing import assert_array_equal, assert_almost_equal  # NOQA
+from numpy.testing import assert_approx_equal, assert_array_almost_equal  # NOQA
+from numpy.testing import assert_equal, assert_array_less  # NOQA
+from numpy.testing import assert_string_equal  # NOQA
+from numpy.testing import assert_array_almost_equal_nulp  # NOQA
+from numpy.testing import assert_allclose, assert_raises  # NOQA
+from numpy.random import RandomState
 from yt.convenience import load
+from yt.units.yt_array import YTArray, YTQuantity
+from yt.utilities.exceptions import YTUnitOperationError
+
+# Expose assert_true and assert_less_equal from unittest.TestCase
+# this is adopted from nose. Doing this here allows us to avoid importing
+# nose at the top level.
+class _Dummy(unittest.TestCase):
+    def nop():
+        pass
+_t = _Dummy('nop')
+
+assert_true = getattr(_t, 'assertTrue')
+assert_less_equal = getattr(_t, 'assertLessEqual')
 
 
 def assert_rel_equal(a1, a2, decimals, err_msg='', verbose=True):
@@ -159,8 +178,10 @@ def fake_random_ds(
         fields = ("density", "velocity_x", "velocity_y", "velocity_z"),
         units = ('g/cm**3', 'cm/s', 'cm/s', 'cm/s'),
         particle_fields=None, particle_field_units=None,
-        negative = False, nprocs = 1, particles = 0, length_unit=1.0):
+        negative = False, nprocs = 1, particles = 0, length_unit=1.0,
+        unit_system="cgs", bbox=None):
     from yt.frontends.stream.api import load_uniform_grid
+    prng = RandomState(0x4d3d3d3)
     if not iterable(ndims):
         ndims = [ndims, ndims, ndims]
     else:
@@ -176,7 +197,7 @@ def fake_random_ds(
             offsets.append(0.0)
     data = {}
     for field, offset, u in zip(fields, offsets, units):
-        v = (np.random.random(ndims) - offset) * peak_value
+        v = (prng.random_sample(ndims) - offset) * peak_value
         if field[0] == "all":
             data['number_of_particles'] = v.size
             v = v.ravel()
@@ -185,17 +206,20 @@ def fake_random_ds(
         if particle_fields is not None:
             for field, unit in zip(particle_fields, particle_field_units):
                 if field in ('particle_position', 'particle_velocity'):
-                    data['io', field] = (np.random.random((particles, 3)), unit)
+                    data['io', field] = (
+                        prng.random_sample((int(particles), 3)), unit)
                 else:
-                    data['io', field] = (np.random.random(size=particles), unit)
+                    data['io', field] = (
+                        prng.random_sample(size=int(particles)), unit)
         else:
             for f in ('particle_position_%s' % ax for ax in 'xyz'):
-                data['io', f] = (np.random.random(size=particles), 'code_length')
+                data['io', f] = (prng.random_sample(size=particles), 'code_length')
             for f in ('particle_velocity_%s' % ax for ax in 'xyz'):
-                data['io', f] = (np.random.random(size=particles) - 0.5, 'cm/s')
-            data['io', 'particle_mass'] = (np.random.random(particles), 'g')
+                data['io', f] = (prng.random_sample(size=particles) - 0.5, 'cm/s')
+            data['io', 'particle_mass'] = (prng.random_sample(particles), 'g')
         data['number_of_particles'] = particles
-    ug = load_uniform_grid(data, ndims, length_unit=length_unit, nprocs=nprocs)
+    ug = load_uniform_grid(data, ndims, length_unit=length_unit, nprocs=nprocs,
+                           unit_system=unit_system, bbox=bbox)
     return ug
 
 _geom_transforms = {
@@ -205,10 +229,13 @@ _geom_transforms = {
     'cylindrical': ( (0.0, 0.0, 0.0), (1.0, 1.0, 2.0*np.pi) ), # rzt
     'polar'      : ( (0.0, 0.0, 0.0), (1.0, 2.0*np.pi, 1.0) ), # rtz
     'geographic' : ( (-90.0, -180.0, 0.0), (90.0, 180.0, 1000.0) ), # latlonalt
+    'internal_geographic' :
+                   ( (-90.0, -180.0, 0.0), (90.0, 180.0, 1000.0) ), # latlondep
 }
 
-def fake_amr_ds(fields = ("Density",), geometry = "cartesian"):
+def fake_amr_ds(fields = ("Density",), geometry = "cartesian", particles=0):
     from yt.frontends.stream.api import load_amr_grids
+    prng = RandomState(0x4d3d3d3)
     LE, RE = _geom_transforms[geometry]
     LE = np.array(LE)
     RE = np.array(RE)
@@ -222,10 +249,20 @@ def fake_amr_ds(fields = ("Density",), geometry = "cartesian"):
                      right_edge = right_edge,
                      dimensions = dims)
         for f in fields:
-            gdata[f] = np.random.random(dims)
+            gdata[f] = prng.random_sample(dims)
+        if particles:
+            for i, f in enumerate('particle_position_%s' % ax for ax in 'xyz'):
+                pdata = prng.random_sample(particles)
+                pdata /= (right_edge[i] - left_edge[i])
+                pdata += left_edge[i]
+                gdata['io', f] = (pdata, 'code_length')
+            for f in ('particle_velocity_%s' % ax for ax in 'xyz'):
+                gdata['io', f] = (prng.random_sample(particles) - 0.5, 'cm/s')
+            gdata['io', 'particle_mass'] = (prng.random_sample(particles), 'g')
+            gdata['number_of_particles'] = particles
         data.append(gdata)
     bbox = np.array([LE, RE]).T
-    return load_amr_grids(data, [32, 32, 32], 1.0, geometry=geometry, bbox=bbox)
+    return load_amr_grids(data, [32, 32, 32], geometry=geometry, bbox=bbox)
 
 def fake_particle_ds(
         fields = ("particle_position_x",
@@ -239,6 +276,8 @@ def fake_particle_ds(
         negative = (False, False, False, False, True, True, True),
         npart = 16**3, length_unit=1.0):
     from yt.frontends.stream.api import load_particles
+
+    prng = RandomState(0x4d3d3d3)
     if not iterable(negative):
         negative = [negative for f in fields]
     assert(len(fields) == len(negative))
@@ -251,14 +290,162 @@ def fake_particle_ds(
     data = {}
     for field, offset, u in zip(fields, offsets, units):
         if "position" in field:
-            v = np.random.normal(loc=0.5, scale=0.25, size=npart)
+            v = prng.normal(loc=0.5, scale=0.25, size=npart)
             np.clip(v, 0.0, 1.0, v)
-        v = (np.random.random(npart) - offset)
+        v = (prng.random_sample(npart) - offset)
         data[field] = (v, u)
     bbox = np.array([[0.0, 1.0], [0.0, 1.0], [0.0, 1.0]])
     ds = load_particles(data, 1.0, bbox=bbox)
     return ds
 
+
+def fake_tetrahedral_ds():
+    from yt.frontends.stream.api import load_unstructured_mesh
+    from yt.frontends.stream.sample_data.tetrahedral_mesh import \
+        _connectivity, _coordinates
+
+    prng = RandomState(0x4d3d3d3)
+
+    # the distance from the origin
+    node_data = {}
+    dist = np.sum(_coordinates**2, 1)
+    node_data[('connect1', 'test')] = dist[_connectivity]
+
+    # each element gets a random number
+    elem_data = {}
+    elem_data[('connect1', 'elem')] = prng.rand(_connectivity.shape[0])
+
+    ds = load_unstructured_mesh(_connectivity,
+                                _coordinates,
+                                node_data=node_data,
+                                elem_data=elem_data)
+    return ds
+
+
+def fake_hexahedral_ds():
+    from yt.frontends.stream.api import load_unstructured_mesh
+    from yt.frontends.stream.sample_data.hexahedral_mesh import \
+        _connectivity, _coordinates
+
+    prng = RandomState(0x4d3d3d3)
+    # the distance from the origin
+    node_data = {}
+    dist = np.sum(_coordinates**2, 1)
+    node_data[('connect1', 'test')] = dist[_connectivity-1]
+
+    # each element gets a random number
+    elem_data = {}
+    elem_data[('connect1', 'elem')] = prng.rand(_connectivity.shape[0])
+
+    ds = load_unstructured_mesh(_connectivity-1,
+                                _coordinates,
+                                node_data=node_data,
+                                elem_data=elem_data)
+    return ds
+
+def small_fake_hexahedral_ds():
+    from yt.frontends.stream.api import load_unstructured_mesh
+
+    _coordinates = np.array([[-1., -1., -1.],
+                              [ 0., -1., -1.],
+                              [ -0.,  0., -1.],
+                              [-1.,  -0., -1.],
+                              [-1., -1.,  0.],
+                              [ -0., -1.,  0.],
+                              [ -0.,  0.,  -0.],
+                              [-1.,  0.,  -0.]])
+    _connectivity = np.array([[1, 2, 3, 4, 5, 6, 7, 8]])
+
+    # the distance from the origin
+    node_data = {}
+    dist = np.sum(_coordinates**2, 1)
+    node_data[('connect1', 'test')] = dist[_connectivity-1]
+
+    ds = load_unstructured_mesh(_connectivity-1,
+                                _coordinates,
+                                node_data=node_data)
+    return ds
+
+
+
+def fake_vr_orientation_test_ds(N = 96, scale=1):
+    """
+    create a toy dataset that puts a sphere at (0,0,0), a single cube
+    on +x, two cubes on +y, and three cubes on +z in a domain from
+    [-1*scale,1*scale]**3.  The lower planes
+    (x = -1*scale, y = -1*scale, z = -1*scale) are also given non-zero
+    values.
+
+    This dataset allows you to easily explore orientations and
+    handiness in VR and other renderings
+
+    Parameters
+    ----------
+
+    N : integer
+       The number of cells along each direction
+
+    scale : float
+       A spatial scale, the domain boundaries will be multiplied by scale to
+       test datasets that have spatial different scales (e.g. data in CGS units)
+
+    """
+    from yt.frontends.stream.api import load_uniform_grid
+
+    xmin = ymin = zmin = -1.0 * scale
+    xmax = ymax = zmax = 1.0 * scale
+
+    dcoord = (xmax - xmin)/N
+
+    arr = np.zeros((N,N,N), dtype=np.float64)
+    arr[:,:,:] = 1.e-4
+
+    bbox = np.array([ [xmin, xmax], [ymin, ymax], [zmin, zmax] ])
+
+    # coordinates -- in the notation data[i, j, k]
+    x = (np.arange(N) + 0.5)*dcoord + xmin
+    y = (np.arange(N) + 0.5)*dcoord + ymin
+    z = (np.arange(N) + 0.5)*dcoord + zmin
+
+    x3d, y3d, z3d = np.meshgrid(x, y, z, indexing="ij")
+
+    # sphere at the origin
+    c = np.array( [0.5*(xmin + xmax), 0.5*(ymin + ymax), 0.5*(zmin + zmax) ] )
+    r = np.sqrt((x3d - c[0])**2 + (y3d - c[1])**2 + (z3d - c[2])**2)
+    arr[r < 0.05] = 1.0
+
+    arr[abs(x3d - xmin) < 2*dcoord] = 0.3
+    arr[abs(y3d - ymin) < 2*dcoord] = 0.3
+    arr[abs(z3d - zmin) < 2*dcoord] = 0.3
+
+    # single cube on +x
+    xc = 0.75 * scale
+    dx = 0.05 * scale
+    idx = np.logical_and(np.logical_and(x3d > xc-dx, x3d < xc+dx),
+                         np.logical_and(np.logical_and(y3d > -dx, y3d < dx),
+                                        np.logical_and(z3d > -dx, z3d < dx)) )
+    arr[idx] = 1.0
+
+    # two cubes on +y
+    dy = 0.05 * scale
+    for yc in [0.65 * scale, 0.85 * scale]:
+        idx = np.logical_and(np.logical_and(y3d > yc-dy, y3d < yc+dy),
+                             np.logical_and(np.logical_and(x3d > -dy, x3d < dy),
+                                            np.logical_and(z3d > -dy, z3d < dy)) )
+        arr[idx] = 0.8
+
+    # three cubes on +z
+    dz = 0.05 * scale
+    for zc in [0.5 * scale, 0.7 * scale, 0.9 * scale]:
+        idx = np.logical_and(np.logical_and(z3d > zc-dz, z3d < zc+dz),
+                             np.logical_and(np.logical_and(x3d > -dz, x3d < dz),
+                                            np.logical_and(y3d > -dz, y3d < dz)) )
+        arr[idx] = 0.6
+
+    data = dict(density = (arr, "g/cm**3"))
+    ds = load_uniform_grid(data, arr.shape, bbox=bbox)
+    return ds
+
 def expand_keywords(keywords, full=False):
     """
     expand_keywords is a means for testing all possible keyword
@@ -275,42 +462,47 @@ def expand_keywords(keywords, full=False):
     list.  Be careful, by using full=True, you may be in for an exponentially
     larger number of tests!
 
+    Parameters
+    ----------
+
     keywords : dict
         a dictionary where the keys are the keywords for the function,
         and the values of each key are the possible values that this key
         can take in the function
 
-   full : bool
+    full : bool
         if set to True, every possible combination of given keywords is
         returned
 
     Returns
     -------
+
     array of dicts
         An array of dictionaries to be individually passed to the appropriate
         function matching these kwargs.
 
     Examples
     --------
+
     >>> keywords = {}
     >>> keywords['dpi'] = (50, 100, 200)
-    >>> keywords['cmap'] = ('algae', 'jet')
+    >>> keywords['cmap'] = ('arbre', 'kelp')
     >>> list_of_kwargs = expand_keywords(keywords)
     >>> print list_of_kwargs
 
-    array([{'cmap': 'algae', 'dpi': 50},
-           {'cmap': 'jet', 'dpi': 100},
-           {'cmap': 'algae', 'dpi': 200}], dtype=object)
+    array([{'cmap': 'arbre', 'dpi': 50},
+           {'cmap': 'kelp', 'dpi': 100},
+           {'cmap': 'arbre', 'dpi': 200}], dtype=object)
 
     >>> list_of_kwargs = expand_keywords(keywords, full=True)
     >>> print list_of_kwargs
 
-    array([{'cmap': 'algae', 'dpi': 50},
-           {'cmap': 'algae', 'dpi': 100},
-           {'cmap': 'algae', 'dpi': 200},
-           {'cmap': 'jet', 'dpi': 50},
-           {'cmap': 'jet', 'dpi': 100},
-           {'cmap': 'jet', 'dpi': 200}], dtype=object)
+    array([{'cmap': 'arbre', 'dpi': 50},
+           {'cmap': 'arbre', 'dpi': 100},
+           {'cmap': 'arbre', 'dpi': 200},
+           {'cmap': 'kelp', 'dpi': 50},
+           {'cmap': 'kelp', 'dpi': 100},
+           {'cmap': 'kelp', 'dpi': 200}], dtype=object)
 
     >>> for kwargs in list_of_kwargs:
     ...     write_projection(*args, **kwargs)
@@ -328,7 +520,7 @@ def expand_keywords(keywords, full=False):
         # Determine the maximum number of values any of the keywords has
         num_lists = 0
         for val in keywords.values():
-            if isinstance(val, str):
+            if isinstance(val, string_types):
                 num_lists = max(1.0, num_lists)
             else:
                 num_lists = max(len(val), num_lists)
@@ -345,7 +537,7 @@ def expand_keywords(keywords, full=False):
             list_of_kwarg_dicts[i] = {}
             for key in keywords.keys():
                 # if it's a string, use it (there's only one)
-                if isinstance(keywords[key], str):
+                if isinstance(keywords[key], string_types):
                     list_of_kwarg_dicts[i][key] = keywords[key]
                 # if there are more options, use the i'th val
                 elif i < len(keywords[key]):
@@ -410,7 +602,7 @@ def units_override_check(fn):
         unit_attr = getattr(ds2, "%s_unit" % u, None)
         if unit_attr is not None:
             attrs2.append(unit_attr)
-    yield assert_equal, attrs1, attrs2
+    assert_equal(attrs1, attrs2)
 
 # This is an export of the 40 grids in IsolatedGalaxy that are of level 4 or
 # lower.  It's just designed to give a sample AMR index to deal with.
@@ -645,25 +837,25 @@ def check_results(func):
     Examples
     --------
 
-    @check_results
-    def my_func(ds):
-        return ds.domain_width
+    >>> @check_results
+    ... def my_func(ds):
+    ...     return ds.domain_width
 
-    my_func(ds)
+    >>> my_func(ds)
 
-    @check_results
-    def field_checker(dd, field_name):
-        return dd[field_name]
+    >>> @check_results
+    ... def field_checker(dd, field_name):
+    ...     return dd[field_name]
 
-    field_cheker(ds.all_data(), 'density', result_basename='density')
+    >>> field_cheker(ds.all_data(), 'density', result_basename='density')
 
     """
     def compute_results(func):
         def _func(*args, **kwargs):
             name = kwargs.pop("result_basename", func.__name__)
             rv = func(*args, **kwargs)
-            if hasattr(rv, "convert_to_cgs"):
-                rv.convert_to_cgs()
+            if hasattr(rv, "convert_to_base"):
+                rv.convert_to_base()
                 _rv = rv.ndarray_view()
             else:
                 _rv = rv
@@ -686,8 +878,8 @@ def check_results(func):
         def _func(*args, **kwargs):
             name = kwargs.pop("result_basename", func.__name__)
             rv = func(*args, **kwargs)
-            if hasattr(rv, "convert_to_cgs"):
-                rv.convert_to_cgs()
+            if hasattr(rv, "convert_to_base"):
+                rv.convert_to_base()
                 _rv = rv.ndarray_view()
             else:
                 _rv = rv
@@ -727,9 +919,10 @@ def periodicity_cases(ds):
                 yield center
 
 def run_nose(verbose=False, run_answer_tests=False, answer_big_data=False,
-             call_pdb = False):
-    import nose, os, sys, yt
-    from yt.funcs import mylog
+             call_pdb=False, module=None):
+    from yt.utilities.on_demand_imports import _nose
+    import sys
+    from yt.utilities.logger import ytLogger as mylog
     orig_level = mylog.getEffectiveLevel()
     mylog.setLevel(50)
     nose_argv = sys.argv
@@ -742,8 +935,10 @@ def run_nose(verbose=False, run_answer_tests=False, answer_big_data=False,
         nose_argv.append('--with-answer-testing')
     if answer_big_data:
         nose_argv.append('--answer-big-data')
+    if module:
+        nose_argv.append(module)
     initial_dir = os.getcwd()
-    yt_file = os.path.abspath(yt.__file__)
+    yt_file = os.path.abspath(__file__)
     yt_dir = os.path.dirname(yt_file)
     if os.path.samefile(os.path.dirname(yt_dir), initial_dir):
         # Provide a nice error message to work around nose bug
@@ -761,7 +956,7 @@ def run_nose(verbose=False, run_answer_tests=False, answer_big_data=False,
             )
     os.chdir(yt_dir)
     try:
-        nose.run(argv=nose_argv)
+        _nose.run(argv=nose_argv)
     finally:
         os.chdir(initial_dir)
         mylog.setLevel(orig_level)
@@ -785,9 +980,12 @@ def assert_allclose_units(actual, desired, rtol=1e-7, atol=0, **kwargs):
         with the units of ``actual`` and ``desired``. If no units are attached,
         assumes the same units as ``desired``. Defaults to zero.
 
+    Notes
+    -----
     Also accepts additional keyword arguments accepted by
     :func:`numpy.testing.assert_allclose`, see the documentation of that
     function for details.
+
     """
     # Create a copy to ensure this function does not alter input arrays
     act = YTArray(actual)
@@ -821,3 +1019,30 @@ def assert_allclose_units(actual, desired, rtol=1e-7, atol=0, **kwargs):
     at = at.value
 
     return assert_allclose(act, des, rt, at, **kwargs)
+
+def assert_fname(fname):
+    """Function that checks file type using libmagic"""
+    if fname is None:
+        return
+
+    with open(fname, 'rb') as fimg:
+        data = fimg.read()
+    image_type = ''
+
+    # see http://www.w3.org/TR/PNG/#5PNG-file-signature
+    if data.startswith(b'\211PNG\r\n\032\n'):
+        image_type = '.png'
+    # see http://www.mathguide.de/info/tools/media-types/image/jpeg
+    elif data.startswith(b'\377\330'):
+        image_type = '.jpeg'
+    elif data.startswith(b'%!PS-Adobe'):
+        data_str = data.decode("utf-8", "ignore")
+        if 'EPSF' in data_str[:data_str.index('\n')]:
+            image_type = '.eps'
+        else:
+            image_type = '.ps'
+    elif data.startswith(b'%PDF'):
+        image_type = '.pdf'
+
+    return image_type == os.path.splitext(fname)[1]
+
