@@ -26,6 +26,7 @@ from yt.testing import \
     fake_random_ds, assert_allclose_units, \
     assert_almost_equal
 from yt.units.unit_registry import UnitRegistry
+from yt.units import electrostatic_unit, elementary_charge
 
 # dimensions
 from yt.units.dimensions import \
@@ -65,7 +66,6 @@ def test_no_conflicting_symbols():
                 raise RuntimeError
 
             full_set.add(new_symbol)
-    yield assert_true, True
 
 def test_dimensionless():
     """
@@ -74,20 +74,20 @@ def test_dimensionless():
     """
     u1 = Unit()
 
-    yield assert_true, u1.is_dimensionless
-    yield assert_true, u1.expr == 1
-    yield assert_true, u1.base_value == 1
-    yield assert_true, u1.dimensions == 1
+    assert_true(u1.is_dimensionless)
+    assert_true(u1.expr == 1)
+    assert_true(u1.base_value == 1)
+    assert_true(u1.dimensions == 1)
 
     u2 = Unit("")
 
-    yield assert_true, u2.is_dimensionless
-    yield assert_true, u2.expr == 1
-    yield assert_true, u2.base_value == 1
-    yield assert_true, u2.dimensions == 1
+    assert_true(u2.is_dimensionless)
+    assert_true(u2.expr == 1)
+    assert_true(u2.base_value == 1)
+    assert_true(u2.dimensions == 1)
 
-    yield assert_equal, u1.latex_repr, ''
-    yield assert_equal, u2.latex_repr, ''
+    assert_equal(u1.latex_repr, '')
+    assert_equal(u2.latex_repr, '')
 
 #
 # Start init tests
@@ -100,39 +100,38 @@ def test_create_from_string():
     """
 
     u1 = Unit("g * cm**2 * s**-2")
-    yield assert_true, u1.dimensions == energy
-    yield assert_true, u1.base_value == 1.0
+    assert_true(u1.dimensions == energy)
+    assert_true(u1.base_value == 1.0)
 
     # make sure order doesn't matter
     u2 = Unit("cm**2 * s**-2 * g")
-    yield assert_true, u2.dimensions == energy
-    yield assert_true, u2.base_value == 1.0
+    assert_true(u2.dimensions == energy)
+    assert_true(u2.base_value == 1.0)
 
     # Test rationals
     u3 = Unit("g**0.5 * cm**-0.5 * s**-1")
-    yield assert_true, u3.dimensions == magnetic_field
-    yield assert_true, u3.base_value == 1.0
+    assert_true(u3.dimensions == magnetic_field)
+    assert_true(u3.base_value == 1.0)
 
     # sqrt functions
     u4 = Unit("sqrt(g)/sqrt(cm)/s")
-    yield assert_true, u4.dimensions == magnetic_field
-    yield assert_true, u4.base_value == 1.0
+    assert_true(u4.dimensions == magnetic_field)
+    assert_true(u4.base_value == 1.0)
 
     # commutative sqrt function
     u5 = Unit("sqrt(g/cm)/s")
-    yield assert_true, u5.dimensions == magnetic_field
-    yield assert_true, u5.base_value == 1.0
+    assert_true(u5.dimensions == magnetic_field)
+    assert_true(u5.base_value == 1.0)
 
     # nonzero CGS conversion factor
     u6 = Unit("Msun/pc**3")
-    yield assert_true, u6.dimensions == mass/length**3
-    yield assert_array_almost_equal_nulp, np.array([u6.base_value]), \
-        np.array([mass_sun_grams/cm_per_pc**3])
+    assert_true(u6.dimensions == mass/length**3)
+    assert_array_almost_equal_nulp(np.array([u6.base_value]), np.array([mass_sun_grams/cm_per_pc**3]))
 
-    yield assert_raises, UnitParseError, Unit, 'm**m'
-    yield assert_raises, UnitParseError, Unit, 'm**g'
-    yield assert_raises, UnitParseError, Unit, 'm+g'
-    yield assert_raises, UnitParseError, Unit, 'm-g'
+    assert_raises(UnitParseError, Unit, 'm**m')
+    assert_raises(UnitParseError, Unit, 'm**g')
+    assert_raises(UnitParseError, Unit, 'm+g')
+    assert_raises(UnitParseError, Unit, 'm-g')
 
 
 def test_create_from_expr():
@@ -156,20 +155,20 @@ def test_create_from_expr():
     u3 = Unit(s3)
     u4 = Unit(s4)
 
-    yield assert_true, u1.expr == s1
-    yield assert_true, u2.expr == s2
-    yield assert_true, u3.expr == s3
-    yield assert_true, u4.expr == s4
+    assert_true(u1.expr == s1)
+    assert_true(u2.expr == s2)
+    assert_true(u3.expr == s3)
+    assert_true(u4.expr == s4)
 
-    yield assert_allclose_units, u1.base_value, pc_cgs, 1e-12
-    yield assert_allclose_units, u2.base_value, yr_cgs, 1e-12
-    yield assert_allclose_units, u3.base_value, pc_cgs * yr_cgs, 1e-12
-    yield assert_allclose_units, u4.base_value, pc_cgs**2 / yr_cgs, 1e-12
+    assert_allclose_units(u1.base_value, pc_cgs, 1e-12)
+    assert_allclose_units(u2.base_value, yr_cgs, 1e-12)
+    assert_allclose_units(u3.base_value, pc_cgs * yr_cgs, 1e-12)
+    assert_allclose_units(u4.base_value, pc_cgs**2 / yr_cgs, 1e-12)
 
-    yield assert_true, u1.dimensions == length
-    yield assert_true, u2.dimensions == time
-    yield assert_true, u3.dimensions == length * time
-    yield assert_true, u4.dimensions == length**2 / time
+    assert_true(u1.dimensions == length)
+    assert_true(u2.dimensions == time)
+    assert_true(u3.dimensions == length * time)
+    assert_true(u4.dimensions == length**2 / time)
 
 
 def test_create_with_duplicate_dimensions():
@@ -183,11 +182,11 @@ def test_create_with_duplicate_dimensions():
     km_cgs = cm_per_km
     Mpc_cgs = cm_per_mpc
 
-    yield assert_true, u1.base_value == 1
-    yield assert_true, u1.dimensions == power
+    assert_true(u1.base_value == 1)
+    assert_true(u1.dimensions == power)
 
-    yield assert_allclose_units, u2.base_value, km_cgs / Mpc_cgs, 1e-12
-    yield assert_true, u2.dimensions == rate
+    assert_allclose_units(u2.base_value, km_cgs / Mpc_cgs, 1e-12)
+    assert_true(u2.dimensions == rate)
 
 def test_create_new_symbol():
     """
@@ -196,21 +195,21 @@ def test_create_new_symbol():
     """
     u1 = Unit("abc", base_value=42, dimensions=(mass/time))
 
-    yield assert_true, u1.expr == Symbol("abc", positive=True)
-    yield assert_true, u1.base_value == 42
-    yield assert_true, u1.dimensions == mass / time
+    assert_true(u1.expr == Symbol("abc", positive=True))
+    assert_true(u1.base_value == 42)
+    assert_true(u1.dimensions == mass / time)
 
     u1 = Unit("abc", base_value=42, dimensions=length**3)
 
-    yield assert_true, u1.expr == Symbol("abc", positive=True)
-    yield assert_true, u1.base_value == 42
-    yield assert_true, u1.dimensions == length**3
+    assert_true(u1.expr == Symbol("abc", positive=True))
+    assert_true(u1.base_value == 42)
+    assert_true(u1.dimensions == length**3)
 
     u1 = Unit("abc", base_value=42, dimensions=length*(mass*length))
 
-    yield assert_true, u1.expr == Symbol("abc", positive=True)
-    yield assert_true, u1.base_value == 42
-    yield assert_true,  u1.dimensions == length**2*mass
+    assert_true(u1.expr == Symbol("abc", positive=True))
+    assert_true(u1.base_value == 42)
+    assert_true( u1.dimensions == length**2*mass)
 
     assert_raises(UnitParseError, Unit, 'abc', base_value=42,
                   dimensions=length**length)
@@ -229,9 +228,9 @@ def test_create_fail_on_unknown_symbol():
     try:
         Unit(Symbol("jigawatts"))
     except UnitParseError:
-        yield assert_true, True
+        assert_true(True)
     else:
-        yield assert_true, False
+        assert_true(False)
 
 def test_create_fail_on_bad_symbol_type():
     """
@@ -241,9 +240,9 @@ def test_create_fail_on_bad_symbol_type():
     try:
         Unit([1])  # something other than Expr and str
     except UnitParseError:
-        yield assert_true, True
+        assert_true(True)
     else:
-        yield assert_true, False
+        assert_true(False)
 
 def test_create_fail_on_bad_dimensions_type():
     """
@@ -253,9 +252,9 @@ def test_create_fail_on_bad_dimensions_type():
     try:
         Unit("a", base_value=1, dimensions="(mass)")
     except UnitParseError:
-        yield assert_true, True
+        assert_true(True)
     else:
-        yield assert_true, False
+        assert_true(False)
 
 
 def test_create_fail_on_dimensions_content():
@@ -270,7 +269,7 @@ def test_create_fail_on_dimensions_content():
     except UnitParseError:
         pass
     else:
-        yield assert_true, False
+        assert_true(False)
 
 
 def test_create_fail_on_base_value_type():
@@ -281,9 +280,9 @@ def test_create_fail_on_base_value_type():
     try:
         Unit("a", base_value="a", dimensions=(mass/time))
     except UnitParseError:
-        yield assert_true, True
+        assert_true(True)
     else:
-        yield assert_true, False
+        assert_true(False)
 
 #
 # End init tests
@@ -299,11 +298,11 @@ def test_string_representation():
     speed = pc / Myr
     dimensionless = Unit()
 
-    yield assert_true, str(pc) == "pc"
-    yield assert_true, str(Myr) == "Myr"
-    yield assert_true, str(speed) == "pc/Myr"
-    yield assert_true, repr(speed) == "pc/Myr"
-    yield assert_true, str(dimensionless) == "dimensionless"
+    assert_true(str(pc) == "pc")
+    assert_true(str(Myr) == "Myr")
+    assert_true(str(speed) == "pc/Myr")
+    assert_true(repr(speed) == "pc/Myr")
+    assert_true(str(dimensionless) == "dimensionless")
 
 #
 # Start operation tests
@@ -329,9 +328,9 @@ def test_multiplication():
     # Mul operation
     u3 = u1 * u2
 
-    yield assert_true, u3.expr == msun_sym * pc_sym
-    yield assert_allclose_units, u3.base_value, msun_cgs * pc_cgs, 1e-12
-    yield assert_true, u3.dimensions == mass * length
+    assert_true(u3.expr == msun_sym * pc_sym)
+    assert_allclose_units(u3.base_value, msun_cgs * pc_cgs, 1e-12)
+    assert_true(u3.dimensions == mass * length)
 
     # Pow and Mul operations
     u4 = Unit("pc**2")
@@ -339,9 +338,9 @@ def test_multiplication():
 
     u6 = u4 * u5
 
-    yield assert_true, u6.expr == pc_sym**2 * msun_sym * s_sym
-    yield assert_allclose_units, u6.base_value, pc_cgs**2 * msun_cgs, 1e-12
-    yield assert_true, u6.dimensions == length**2 * mass * time
+    assert_true(u6.expr == pc_sym**2 * msun_sym * s_sym)
+    assert_allclose_units(u6.base_value, pc_cgs**2 * msun_cgs, 1e-12)
+    assert_true(u6.dimensions == length**2 * mass * time)
 
 
 def test_division():
@@ -363,9 +362,9 @@ def test_division():
 
     u3 = u1 / u2
 
-    yield assert_true, u3.expr == pc_sym / (km_sym * s_sym)
-    yield assert_allclose_units, u3.base_value, pc_cgs / km_cgs, 1e-12
-    yield assert_true, u3.dimensions == 1 / time
+    assert_true(u3.expr == pc_sym / (km_sym * s_sym))
+    assert_allclose_units(u3.base_value, pc_cgs / km_cgs, 1e-12)
+    assert_true(u3.dimensions == 1 / time)
 
 
 def test_power():
@@ -382,13 +381,13 @@ def test_power():
 
     u2 = u1**2
 
-    yield assert_true, u2.dimensions == u1_dims**2
-    yield assert_allclose_units, u2.base_value, (pc_cgs**2 * mK_cgs**4)**2, 1e-12
+    assert_true(u2.dimensions == u1_dims**2)
+    assert_allclose_units(u2.base_value, (pc_cgs**2 * mK_cgs**4)**2, 1e-12)
 
     u3 = u1**(-1.0/3)
 
-    yield assert_true, u3.dimensions == nsimplify(u1_dims**(-1.0/3))
-    yield assert_allclose_units, u3.base_value, (pc_cgs**2 * mK_cgs**4)**(-1.0/3), 1e-12
+    assert_true(u3.dimensions == nsimplify(u1_dims**(-1.0/3)))
+    assert_allclose_units(u3.base_value, (pc_cgs**2 * mK_cgs**4)**(-1.0/3), 1e-12)
 
 
 def test_equality():
@@ -399,7 +398,7 @@ def test_equality():
     u1 = Unit("km * s**-1")
     u2 = Unit("m * ms**-1")
 
-    yield assert_true, u1 == u2
+    assert_true(u1 == u2)
 
 #
 # End operation tests.
@@ -417,21 +416,20 @@ def test_base_equivalent():
     u2 = Unit("g * cm**-3")
     u3 = u1.get_base_equivalent()
 
-    yield assert_true, u2.expr == u3.expr
-    yield assert_true, u2 == u3
+    assert_true(u2.expr == u3.expr)
+    assert_true(u2 == u3)
 
-    yield assert_allclose_units, u1.base_value, Msun_cgs / Mpc_cgs**3, 1e-12
-    yield assert_true, u2.base_value == 1
-    yield assert_true, u3.base_value == 1
+    assert_allclose_units(u1.base_value, Msun_cgs / Mpc_cgs**3, 1e-12)
+    assert_true(u2.base_value == 1)
+    assert_true(u3.base_value == 1)
 
     mass_density = mass / length**3
 
-    yield assert_true, u1.dimensions == mass_density
-    yield assert_true, u2.dimensions == mass_density
-    yield assert_true, u3.dimensions == mass_density
+    assert_true(u1.dimensions == mass_density)
+    assert_true(u2.dimensions == mass_density)
+    assert_true(u3.dimensions == mass_density)
 
-    yield assert_allclose_units, get_conversion_factor(u1, u3)[0], \
-        Msun_cgs / Mpc_cgs**3, 1e-12
+    assert_allclose_units(get_conversion_factor(u1, u3)[0], Msun_cgs / Mpc_cgs**3, 1e-12)
 
 def test_is_code_unit():
     ds = fake_random_ds(64, nprocs=1)
@@ -442,12 +440,12 @@ def test_is_code_unit():
     u5 = Unit('code_mass*g', registry=ds.unit_registry)
     u6 = Unit('g/cm**3')
 
-    yield assert_true, u1.is_code_unit
-    yield assert_true, u2.is_code_unit
-    yield assert_true, u3.is_code_unit
-    yield assert_true, u4.is_code_unit
-    yield assert_true, not u5.is_code_unit
-    yield assert_true, not u6.is_code_unit
+    assert_true(u1.is_code_unit)
+    assert_true(u2.is_code_unit)
+    assert_true(u3.is_code_unit)
+    assert_true(u4.is_code_unit)
+    assert_true(not u5.is_code_unit)
+    assert_true(not u6.is_code_unit)
 
 def test_temperature_offsets():
     u1 = Unit('degC')
@@ -487,22 +485,22 @@ def test_latitude_longitude():
     lat = unit_symbols.lat
     lon = unit_symbols.lon
     deg = unit_symbols.deg
-    yield assert_equal, lat.units.base_offset, 90.0
-    yield assert_equal, (deg*90.0).in_units("lat").value, 0.0
-    yield assert_equal, (deg*180).in_units("lat").value, -90.0
-    yield assert_equal, (lat*0.0).in_units("deg"), deg*90.0
-    yield assert_equal, (lat*-90).in_units("deg"), deg*180
+    assert_equal(lat.units.base_offset, 90.0)
+    assert_equal((deg*90.0).in_units("lat").value, 0.0)
+    assert_equal((deg*180).in_units("lat").value, -90.0)
+    assert_equal((lat*0.0).in_units("deg"), deg*90.0)
+    assert_equal((lat*-90).in_units("deg"), deg*180)
 
-    yield assert_equal, lon.units.base_offset, -180.0
-    yield assert_equal, (deg*0.0).in_units("lon").value, -180.0
-    yield assert_equal, (deg*90.0).in_units("lon").value, -90.0
-    yield assert_equal, (deg*180).in_units("lon").value, 0.0
-    yield assert_equal, (deg*360).in_units("lon").value, 180.0
+    assert_equal(lon.units.base_offset, -180.0)
+    assert_equal((deg*0.0).in_units("lon").value, -180.0)
+    assert_equal((deg*90.0).in_units("lon").value, -90.0)
+    assert_equal((deg*180).in_units("lon").value, 0.0)
+    assert_equal((deg*360).in_units("lon").value, 180.0)
 
-    yield assert_equal, (lon*-180.0).in_units("deg"), deg*0.0
-    yield assert_equal, (lon*-90.0).in_units("deg"), deg*90.0
-    yield assert_equal, (lon*0.0).in_units("deg"), deg*180.0
-    yield assert_equal, (lon*180.0).in_units("deg"), deg*360
+    assert_equal((lon*-180.0).in_units("deg"), deg*0.0)
+    assert_equal((lon*-90.0).in_units("deg"), deg*90.0)
+    assert_equal((lon*0.0).in_units("deg"), deg*180.0)
+    assert_equal((lon*180.0).in_units("deg"), deg*360)
 
 def test_registry_json():
     reg = UnitRegistry()
@@ -510,3 +508,18 @@ def test_registry_json():
     unserialized_reg = UnitRegistry.from_json(json_reg)
 
     assert_equal(reg.lut, unserialized_reg.lut)
+
+def test_creation_from_ytarray():
+    u1 = Unit(electrostatic_unit)
+    assert_equal(str(u1), 'esu')
+    assert_equal(u1, Unit('esu'))
+    assert_equal(u1, electrostatic_unit.units)
+
+    u2 = Unit(elementary_charge)
+    assert_equal(str(u2), '4.8032056e-10*esu')
+    assert_equal(u2, Unit('4.8032056e-10*esu'))
+    assert_equal(u1, elementary_charge.units)
+
+    assert_equal((u1/u2).base_value, electrostatic_unit/elementary_charge)
+
+    assert_raises(UnitParseError, Unit, [1, 2, 3]*elementary_charge)
