@@ -15,7 +15,7 @@ from __future__ import print_function
 
 import os
 import sys
-import h5py
+from yt.utilities.on_demand_imports import _h5py as h5py
 import numpy as np
 from contextlib import contextmanager
 
@@ -40,8 +40,8 @@ def write_to_gdf(ds, gdf_path, fields=None,
         The yt data to write out.
     gdf_path : string
         The path of the file to output.
-    fields : field or list of fields
-        The fields(s) to write out. If None, defaults to 
+    fields
+        The field or list of fields to write out. If None, defaults to
         ds.field_list.
     data_author : string, optional
         The name of the author who wrote the data. Default: None.
@@ -50,11 +50,13 @@ def write_to_gdf(ds, gdf_path, fields=None,
     dataset_units : dictionary, optional
         A dictionary of (value, unit) tuples to set the default units
         of the dataset. Keys can be:
-            "length_unit"
-            "time_unit"
-            "mass_unit"
-            "velocity_unit"
-            "magnetic_unit"
+
+        * "length_unit"
+        * "time_unit"
+        * "mass_unit"
+        * "velocity_unit"
+        * "magnetic_unit"
+
         If not specified, these will carry over from the parent
         dataset.
     particle_type_name : string, optional
@@ -176,34 +178,36 @@ def _write_fields_to_gdf(ds, fhandle, fields, particle_type_name,
     g = fhandle["data"]
     data_source = ds.all_data()
     citer = data_source.chunks([], "io", local_only=True)
-    for chunk in parallel_objects(citer):
+    for region in parallel_objects(citer):
         # is there a better way to the get the grids on each chunk?
-        for grid in list(ds.index._chunk_io(chunk))[0].objs:
-            for field_name in fields:
+        for chunk in ds.index._chunk_io(region):
+            for grid in chunk.objs:
+                for field_name in fields:
 
-                # sanitize and get the field info object
-                if isinstance(field_name, tuple):
-                    field_name = field_name[1]
-                fi = ds._get_field_info(field_name)
+                    # sanitize and get the field info object
+                    if isinstance(field_name, tuple):
+                        field_name = field_name[1]
+                    fi = ds._get_field_info(field_name)
 
-                # set field parameters, if specified
-                if field_parameters is not None:
-                    for k, v in field_parameters.items():
-                        grid.set_field_parameter(k, v)
+                    # set field parameters, if specified
+                    if field_parameters is not None:
+                        for k, v in field_parameters.items():
+                            grid.set_field_parameter(k, v)
 
-                grid_group = g["grid_%010i" % (grid.id - grid._id_offset)]
-                particles_group = grid_group["particles"]
-                pt_group = particles_group[particle_type_name]
-                # add the field data to the grid group
-                # Check if this is a real field or particle data.
-                grid.get_data(field_name)
-                units = fhandle["field_types"][field_name].attrs["field_units"]
-                if fi.particle_type:  # particle data
-                    dset = pt_group[field_name]
-                    dset[:] = grid[field_name].in_units(units)
-                else:  # a field
-                    dset = grid_group[field_name]
-                    dset[:] = grid[field_name].in_units(units)
+                    grid_group = g["grid_%010i" % (grid.id - grid._id_offset)]
+                    particles_group = grid_group["particles"]
+                    pt_group = particles_group[particle_type_name]
+                    # add the field data to the grid group
+                    # Check if this is a real field or particle data.
+                    grid.get_data(field_name)
+                    units = fhandle[
+                        "field_types"][field_name].attrs["field_units"]
+                    if fi.particle_type:  # particle data
+                        dset = pt_group[field_name]
+                        dset[:] = grid[field_name].in_units(units)
+                    else:  # a field
+                        dset = grid_group[field_name]
+                        dset[:] = grid[field_name].in_units(units)
 
 @contextmanager
 def _get_backup_file(ds):
@@ -341,7 +345,7 @@ def _create_new_gdf(ds, gdf_path, data_author=None, data_comment=None,
         grid_group = g.create_group("grid_%010i" % (grid.id - grid._id_offset))
         # add group for the particles on this grid
         particles_group = grid_group.create_group("particles")
-        pt_group = particles_group.create_group(particle_type_name)
+        particles_group.create_group(particle_type_name)
 
     yield f
     
