@@ -1,7 +1,8 @@
 import numpy as np
 
-from yt.testing import *
-from yt.utilities.lib.CICDeposit import CICSample_3
+from yt.utilities.lib.particle_mesh_operations import \
+    CICSample_3
+from yt.testing import assert_allclose
 
 def setup():
     pass
