@@ -13,7 +13,7 @@ A simple IO staging mechanism
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-import np
+import numpy as np
 from yt.utilities.logger import ytLogger as mylog
 from .parallel_analysis_interface import \
     ProcessorPool, parallel_objects
