@@ -28,8 +28,9 @@ from yt.funcs import \
     ensure_list, iterable
 
 from yt.config import ytcfg
+from yt.data_objects.image_array import ImageArray
 import yt.utilities.logger
-from yt.utilities.lib.QuadTree import \
+from yt.utilities.lib.quad_tree import \
     QuadTree, merge_quadtrees
 from yt.units.yt_array import YTArray
 from yt.units.unit_registry import UnitRegistry
@@ -124,7 +125,7 @@ def enable_parallelism(suppress_logging=False, communicator=None):
     ytcfg["yt","__global_parallel_size"] = str(communicator.size)
     ytcfg["yt","__parallel"] = "True"
     if exe_name == "embed_enzo" or \
-        ("_parallel" in dir(sys) and sys._parallel == True):
+        ("_parallel" in dir(sys) and sys._parallel is True):
         ytcfg["yt","inline"] = "True"
     if communicator.rank > 0:
         if ytcfg.getboolean("yt","LogFile"):
@@ -270,7 +271,7 @@ class ParallelDummy(type):
             if attrname.startswith("_") or attrname in skip:
                 if attrname not in extra: continue
             attr = getattr(cls, attrname)
-            if type(attr) == types.MethodType:
+            if isinstance(attr, types.MethodType):
                 setattr(cls, attrname, parallel_simple_proxy(attr))
 
 def parallel_passthrough(func):
@@ -323,7 +324,7 @@ def parallel_root_only(func):
             try:
                 rv = func(*args, **kwargs)
                 all_clear = 1
-            except Exception as ex:
+            except Exception:
                 traceback.print_last()
                 all_clear = 0
         else:
@@ -433,7 +434,7 @@ def parallel_objects(objects, njobs = 0, storage = None, barrier = True,
 
     Parameters
     ----------
-    objects : iterable
+    objects : Iterable
         The list of objects to dispatch to different processors.
     njobs : int
         How many jobs to spawn.  By default, one job will be dispatched for
@@ -539,7 +540,7 @@ def parallel_ring(objects, generator_func, mutable = False):
 
     Parameters
     ----------
-    objects : iterable
+    objects : Iterable
         The list of objects to operate on.
     generator_func : callable
         This function will be called on each object, and the results yielded.
@@ -675,7 +676,8 @@ class Communicator(object):
         self._distributed = comm is not None and self.comm.size > 1
 
     def __del__(self):
-        self.comm.Free()
+        if self.comm is not None:
+            self.comm.Free()
     """
     This is an interface specification providing several useful utility
     functions for analyzing something in parallel.
@@ -794,15 +796,25 @@ class Communicator(object):
             if self.comm.rank == root:
                 if isinstance(data, YTArray):
                     info = (data.shape, data.dtype, str(data.units), data.units.registry.lut)
+                    if isinstance(data, ImageArray):
+                        info += ('ImageArray',)
+                    else:
+                        info += ('YTArray',)
                 else:
                     info = (data.shape, data.dtype)
             else:
                 info = ()
             info = self.comm.bcast(info, root=root)
             if self.comm.rank != root:
-                if len(info) == 4:
+                if len(info) == 5:
                     registry = UnitRegistry(lut=info[3], add_default_symbols=False)
-                    data = YTArray(np.empty(info[0], dtype=info[1]), info[2], registry=registry)
+                    if info[-1] == "ImageArray":
+                        data = ImageArray(np.empty(info[0], dtype=info[1]),
+                                          input_units=info[2],
+                                          registry=registry)
+                    else:
+                        data = YTArray(np.empty(info[0], dtype=info[1]), 
+                                       info[2], registry=registry)
                 else:
                     data = np.empty(info[0], dtype=info[1])
             mpi_type = get_mpi_type(info[1])
@@ -917,7 +929,7 @@ class Communicator(object):
 
     def get_filename(self, prefix, rank=None):
         if not self._distributed: return prefix
-        if rank == None:
+        if rank is None:
             return "%s_%04i" % (prefix, self.comm.rank)
         else:
             return "%s_%04i" % (prefix, rank)
@@ -1008,6 +1020,10 @@ class Communicator(object):
         # communicate type and shape and optionally units
         if isinstance(arr, YTArray):
             unit_metadata = (str(arr.units), arr.units.registry.lut)
+            if isinstance(arr, ImageArray):
+                unit_metadata += ('ImageArray',)
+            else:
+                unit_metadata += ('YTArray',)
         else:
             unit_metadata = ()
         self.comm.send((arr.dtype.str, arr.shape) + unit_metadata, dest=dest, tag=tag)
@@ -1020,9 +1036,13 @@ class Communicator(object):
         if ne is None and dt is None:
             return self.comm.recv(source=source, tag=tag)
         arr = np.empty(ne, dtype=dt)
-        if len(metadata) == 4:
+        if len(metadata) == 5:
             registry = UnitRegistry(lut=metadata[3], add_default_symbols=False)
-            arr = YTArray(arr, metadata[2], registry=registry)
+            if metadata[-1] == "ImageArray":
+                arr = ImageArray(arr, input_units=metadata[2],
+                                 registry=registry)
+            else:
+                arr = YTArray(arr, metadata[2], registry=registry)
         tmp = arr.view(self.__tocast)
         self.comm.Recv([tmp, MPI.CHAR], source=source, tag=tag)
         return arr
@@ -1041,7 +1061,10 @@ class Communicator(object):
         if isinstance(send, YTArray):
             # We assume send.units is consitent with the units
             # on the receiving end.
-            recv = YTArray(recv, send.units)
+            if isinstance(send, ImageArray):
+                recv = ImageArray(recv, input_units=send.units)
+            else:
+                recv = YTArray(recv, send.units)
         recv[offset:offset+send.size] = send[:]
         dtr = send.dtype.itemsize / tmp_send.dtype.itemsize # > 1
         roff = [off * dtr for off in offsets]
@@ -1246,7 +1269,7 @@ class ParallelAnalysisInterface(object):
                 if f in xyzfactors[nextdim]:
                     cuts.append([nextdim, f])
                     topop = xyzfactors[nextdim].index(f)
-                    temp = xyzfactors[nextdim].pop(topop)
+                    xyzfactors[nextdim].pop(topop)
                     lastdim = nextdim
                     break
                 nextdim = (nextdim + 1) % 3
