@@ -1,33 +1,34 @@
 import numpy as np
-from yt.utilities.lib.CICDeposit import CICSample_3
-from yt.funcs import *
-from yt.units.yt_array import uconcatenate, YTArray
+from yt.utilities.lib.particle_mesh_operations import \
+    CICSample_3
+from yt.funcs import get_pbar
+from yt.units.yt_array import uconcatenate
 from yt.extern.six import string_types
 
 class ParticleGenerator(object):
 
-    default_fields = [("io", "particle_position_x"),
-                      ("io", "particle_position_y"),
-                      ("io", "particle_position_z")]
-    
-    def __init__(self, ds, num_particles, field_list):
+    def __init__(self, ds, num_particles, field_list, ptype="io"):
         """
         Base class for generating particle fields which may be applied to
         streams. Normally this would not be called directly, since it doesn't
         really do anything except allocate memory. Takes a *ds* to serve as the
         basis for determining grids, the number of particles *num_particles*,
-        and a list of fields, *field_list*.
+        a list of fields, *field_list*, and the particle type *ptype*, which
+        has a default value of "io". 
         """
         self.ds = ds
         self.num_particles = num_particles
-        self.field_list = [("io",fd) if isinstance(fd,string_types) else fd
+        self.field_list = [(ptype, fd) if isinstance(fd, string_types) else fd
                            for fd in field_list]
-        self.field_list.append(("io", "particle_index"))
+        self.field_list.append((ptype, "particle_index"))
         self.field_units = dict(
-          (('io', 'particle_position_%s' % ax), 'code_length')
+          ((ptype, 'particle_position_%s' % ax), 'code_length')
           for ax in 'xyz')
-        self.field_units['io', 'particle_index'] = ''
-        
+        self.field_units[ptype, 'particle_index'] = ''
+        self.ptype = ptype
+
+        self._set_default_fields()
+
         try:
             self.posx_index = self.field_list.index(self.default_fields[0])
             self.posy_index = self.field_list.index(self.default_fields[1])
@@ -35,35 +36,40 @@ class ParticleGenerator(object):
         except:
             raise KeyError("You must specify position fields: " +
                            " ".join(["particle_position_%s" % ax for ax in "xyz"]))
-        self.index_index = self.field_list.index(("io", "particle_index"))
-        
+        self.index_index = self.field_list.index((ptype, "particle_index"))
+
         self.num_grids = self.ds.index.num_grids
-        self.NumberOfParticles = np.zeros((self.num_grids), dtype='int64')
+        self.NumberOfParticles = np.zeros(self.num_grids, dtype='int64')
         self.ParticleGridIndices = np.zeros(self.num_grids + 1, dtype='int64')
-        
+
         self.num_fields = len(self.field_list)
-        
+
         self.particles = np.zeros((self.num_particles, self.num_fields),
                                   dtype='float64')
 
+    def _set_default_fields(self):
+        self.default_fields = [(self.ptype, "particle_position_x"),
+                               (self.ptype, "particle_position_y"),
+                               (self.ptype, "particle_position_z")]
+
     def has_key(self, key):
         """
         Check to see if *key* is in the particle field list.
         """
         return key in self.field_list
-            
+
     def keys(self):
         """
         Return the list of particle fields.
         """
         return self.field_list
-    
+
     def __getitem__(self, key):
         """
         Get the field associated with key.
         """
         return self.particles[:,self.field_list.index(key)]
-    
+
     def __setitem__(self, key, val):
         """
         Sets a field to be some other value. Note that we assume
@@ -71,7 +77,7 @@ class ParticleGenerator(object):
         make sure the setting of the field is consistent with this.
         """
         self.particles[:,self.field_list.index(key)] = val[:]
-                
+
     def __len__(self):
         """
         The number of particles
@@ -94,17 +100,17 @@ class ParticleGenerator(object):
             else:
                 tr[field] = self.particles[start:end, fi]
         return tr
-    
-    def _setup_particles(self,x,y,z,setup_fields=None):
+
+    def _setup_particles(self, x, y, z, setup_fields=None):
         """
         Assigns grids to particles and sets up particle positions. *setup_fields* is
         a dict of fields other than the particle positions to set up. 
         """
-        particle_grids, particle_grid_inds = self.ds.index._find_points(x,y,z)
+        particle_grids, particle_grid_inds = self.ds.index._find_points(x, y, z)
         idxs = np.argsort(particle_grid_inds)
-        self.particles[:,self.posx_index] = x[idxs]
-        self.particles[:,self.posy_index] = y[idxs]
-        self.particles[:,self.posz_index] = z[idxs]
+        self.particles[:, self.posx_index] = x[idxs]
+        self.particles[:, self.posy_index] = y[idxs]
+        self.particles[:, self.posz_index] = z[idxs]
         self.NumberOfParticles = np.bincount(particle_grid_inds.astype("intp"),
                                              minlength=self.num_grids)
         if self.num_grids > 1:
@@ -114,20 +120,20 @@ class ParticleGenerator(object):
             self.ParticleGridIndices[1] = self.NumberOfParticles.squeeze()
         if setup_fields is not None:
             for key, value in setup_fields.items():
-                field = ("io",key) if isinstance(key, string_types) else key
+                field = (self.ptype, key) if isinstance(key, string_types) else key
                 if field not in self.default_fields:
                     self.particles[:,self.field_list.index(field)] = value[idxs]
-    
+
     def assign_indices(self, function=None, **kwargs):
         """
         Assign unique indices to the particles. The default is to just use
         numpy.arange, but any function may be supplied with keyword arguments.
         """
-        if function is None :
-            self.particles[:,self.index_index] = np.arange((self.num_particles))
-        else :
-            self.particles[:,self.index_index] = function(**kwargs)
-            
+        if function is None:
+            self.particles[:, self.index_index] = np.arange(self.num_particles)
+        else:
+            self.particles[:, self.index_index] = function(**kwargs)
+
     def map_grid_fields_to_particles(self, mapping_dict):
         r"""
         For the fields in  *mapping_dict*, map grid fields to the particles
@@ -163,36 +169,28 @@ class ParticleGenerator(object):
 
     def apply_to_stream(self, clobber=False):
         """
-        Apply the particles to a stream dataset. If particles already exist,
-        and clobber=False, do not overwrite them, but add the new ones to them. 
+        Apply the particles to a grid-based stream dataset. If particles 
+        already exist, and clobber=False, do not overwrite them, but add 
+        the new ones to them.
         """
         grid_data = []
-        for i,g in enumerate(self.ds.index.grids):
+        for i, g in enumerate(self.ds.index.grids):
             data = {}
-            if clobber :
-                data["number_of_particles"] = self.NumberOfParticles[i]
-            else :
-                data["number_of_particles"] = self.NumberOfParticles[i] + \
-                                              g.NumberOfParticles
+            number_of_particles = self.NumberOfParticles[i]
+            if not clobber:
+                number_of_particles += g.NumberOfParticles
             grid_particles = self.get_for_grid(g)
-            for field in self.field_list :
-                if data["number_of_particles"] > 0:
-                    # We have particles in this grid
-                    if g.NumberOfParticles > 0 and not clobber:
-                        # Particles already exist
-                        if field in self.ds.field_list:
-                            # This field already exists
-                            prev_particles = g[field]
-                        else:
-                            # This one doesn't, set the previous particles' field
-                            # values to zero
-                            prev_particles = np.zeros((g.NumberOfParticles))
-                            prev_particles = self.ds.arr(prev_particles,
-                                input_units = self.field_units[field])
-                        data[field] = uconcatenate((prev_particles,
-                                                    grid_particles[field]))
+            for field in self.field_list:
+                if number_of_particles > 0:
+                    if g.NumberOfParticles > 0 and not clobber and \
+                        field in self.ds.field_list:
+                        # We have particles in this grid, we're not
+                        # overwriting them, and the field is in the field
+                        # list already
+                        data[field] = uconcatenate([g[field],
+                                                    grid_particles[field]])
                     else:
-                        # Particles do not already exist or we're clobbering
+                        # Otherwise, simply add the field in
                         data[field] = grid_particles[field]
                 else:
                     # We don't have particles in this grid
@@ -202,7 +200,7 @@ class ParticleGenerator(object):
 
 class FromListParticleGenerator(ParticleGenerator):
 
-    def __init__(self, ds, num_particles, data):
+    def __init__(self, ds, num_particles, data, ptype="io"):
         r"""
         Generate particle fields from array-like lists contained in a dict.
 
@@ -214,6 +212,8 @@ class FromListParticleGenerator(ParticleGenerator):
             The number of particles in the dict.
         data : dict of NumPy arrays
             The particle fields themselves.
+        ptype : string, optional
+            The particle type for these particle fields. Default: "io"
 
         Examples
         --------
@@ -232,10 +232,10 @@ class FromListParticleGenerator(ParticleGenerator):
             x = data.pop("particle_position_x")
             y = data.pop("particle_position_y")
             z = data.pop("particle_position_z")
-        elif ("io","particle_position_x") in data:
-            x = data.pop(("io", "particle_position_x"))
-            y = data.pop(("io", "particle_position_y"))
-            z = data.pop(("io", "particle_position_z"))
+        elif (ptype,"particle_position_x") in data:
+            x = data.pop((ptype, "particle_position_x"))
+            y = data.pop((ptype, "particle_position_y"))
+            z = data.pop((ptype, "particle_position_z"))
 
         xcond = np.logical_or(x < ds.domain_left_edge[0],
                               x >= ds.domain_right_edge[0])
@@ -249,13 +249,14 @@ class FromListParticleGenerator(ParticleGenerator):
         if np.any(cond):
             raise ValueError("Some particles are outside of the domain!!!")
 
-        ParticleGenerator.__init__(self, ds, num_particles, field_list)
-        self._setup_particles(x,y,z,setup_fields=data)
-        
+        super(FromListParticleGenerator, self).__init__(ds, num_particles, 
+                                                        field_list, ptype=ptype)
+        self._setup_particles(x, y, z, setup_fields=data)
+
 class LatticeParticleGenerator(ParticleGenerator):
 
     def __init__(self, ds, particles_dims, particles_left_edge,
-                 particles_right_edge, field_list):
+                 particles_right_edge, field_list, ptype="io"):
         r"""
         Generate particles in a lattice arrangement. 
 
@@ -271,7 +272,9 @@ class LatticeParticleGenerator(ParticleGenerator):
              The 'right-most' ending positions of the lattice.
         field_list : list of strings
              A list of particle fields
-             
+        ptype : string, optional
+            The particle type for these particle fields. Default: "io"
+
         Examples
         --------
         >>> dims = (128,128,128)
@@ -292,33 +295,34 @@ class LatticeParticleGenerator(ParticleGenerator):
         xmax = particles_right_edge[0]
         ymax = particles_right_edge[1]
         zmax = particles_right_edge[2]
-        DLE = ds.domain_left_edge.in_units("code_length").ndarray_view()
-        DRE = ds.domain_right_edge.in_units("code_length").ndarray_view()
+        DLE = ds.domain_left_edge.in_units("code_length").d
+        DRE = ds.domain_right_edge.in_units("code_length").d
 
         xcond = (xmin < DLE[0]) or (xmax >= DRE[0])
         ycond = (ymin < DLE[1]) or (ymax >= DRE[1])
         zcond = (zmin < DLE[2]) or (zmax >= DRE[2])
         cond = xcond or ycond or zcond
 
-        if cond :
+        if cond:
             raise ValueError("Proposed bounds for particles are outside domain!!!")
 
-        ParticleGenerator.__init__(self, ds, num_x*num_y*num_z, field_list)
+        super(LatticeParticleGenerator, self).__init__(ds, num_x*num_y*num_z, 
+                                                       field_list, ptype=ptype)
 
         dx = (xmax-xmin)/(num_x-1)
         dy = (ymax-ymin)/(num_y-1)
         dz = (zmax-zmin)/(num_z-1)
-        inds = np.indices((num_x,num_y,num_z))
+        inds = np.indices((num_x, num_y, num_z))
         xpos = inds[0]*dx + xmin
         ypos = inds[1]*dy + ymin
         zpos = inds[2]*dz + zmin
-        
+
         self._setup_particles(xpos.flat[:], ypos.flat[:], zpos.flat[:])
-        
+
 class WithDensityParticleGenerator(ParticleGenerator):
 
     def __init__(self, ds, data_source, num_particles, field_list,
-                 density_field="density"):
+                 density_field="density", ptype="io"):
         r"""
         Generate particles based on a density field.
 
@@ -335,7 +339,9 @@ class WithDensityParticleGenerator(ParticleGenerator):
         density_field : string, optional
             A density field which will serve as the distribution function for the
             particle positions. Theoretically, this could be any 'per-volume' field. 
-            
+        ptype : string, optional
+            The particle type for these particle fields. Default: "io"
+
         Examples
         --------
         >>> sphere = ds.sphere(ds.domain_center, 0.5)
@@ -347,7 +353,8 @@ class WithDensityParticleGenerator(ParticleGenerator):
         >>>                                          fields, density_field='Dark_Matter_Density')
         """
 
-        ParticleGenerator.__init__(self, ds, num_particles, field_list)
+        super(WithDensityParticleGenerator, self).__init__(ds, num_particles,
+                                                           field_list, ptype=ptype)
 
         num_cells = len(data_source["x"].flat)
         max_mass = (data_source[density_field]*
@@ -356,22 +363,22 @@ class WithDensityParticleGenerator(ParticleGenerator):
         all_x = []
         all_y = []
         all_z = []
-        
+
         pbar = get_pbar("Generating Particles", num_particles)
         tot_num_accepted = int(0)
-        
+
         while num_particles_left > 0:
 
             m = np.random.uniform(high=1.01*max_mass,
                                   size=num_particles_left)
             idxs = np.random.random_integers(low=0, high=num_cells-1,
                                              size=num_particles_left)
-            m_true = (data_source[density_field]*
+            m_true = (data_source[density_field] *
                       data_source["cell_volume"]).flat[idxs]
             accept = m <= m_true
             num_accepted = accept.sum()
             accepted_idxs = idxs[accept]
-            
+
             xpos = data_source["x"].flat[accepted_idxs] + \
                    np.random.uniform(low=-0.5, high=0.5, size=num_accepted) * \
                    data_source["dx"].flat[accepted_idxs]
@@ -396,5 +403,5 @@ class WithDensityParticleGenerator(ParticleGenerator):
         y = uconcatenate(all_y)
         z = uconcatenate(all_z)
 
-        self._setup_particles(x,y,z)
-        
+        self._setup_particles(x, y, z)
+
