@@ -18,18 +18,18 @@ from yt.frontends.ytdata.utilities import \
 from yt.funcs import \
     get_output_filename, \
     mylog, \
-    ensure_list
+    ensure_list, \
+    deprecate
 from .volume_rendering.api import off_axis_projection
 from .fixed_resolution_filters import apply_filter, filter_registry
 from yt.data_objects.image_array import ImageArray
 from yt.utilities.lib.pixelization_routines import \
-    pixelize_cylinder, pixelize_off_axis_cartesian
+    pixelize_cylinder
 from yt.utilities.lib.api import add_points_to_greyscale_image
 from yt.frontends.stream.api import load_uniform_grid
 
 import numpy as np
 import weakref
-import re
 import types
 
 class FixedResolutionBuffer(object):
@@ -51,9 +51,8 @@ class FixedResolutionBuffer(object):
     Parameters
     ----------
     data_source : :class:`yt.data_objects.construction_data_containers.YTQuadTreeProj` or :class:`yt.data_objects.selection_data_containers.YTSlice`
-        This is the source to be pixelized, which can be a projection or a
-        slice.  (For cutting planes, see
-        `yt.visualization.fixed_resolution.ObliqueFixedResolutionBuffer`.)
+        This is the source to be pixelized, which can be a projection, slice or
+        cutting plane.
     bounds : sequence of floats
         Bounds are the min and max in the image plane that we want our
         image to cover.  It's in the order of (xmin, xmax, ymin, ymax),
@@ -67,12 +66,6 @@ class FixedResolutionBuffer(object):
         This can be true or false, and governs whether the pixelization
         will span the domain boundaries.
 
-    See Also
-    --------
-    :class:`yt.visualization.fixed_resolution.ObliqueFixedResolutionBuffer` : A similar object,
-                                                     used for cutting
-                                                     planes.
-
     Examples
     --------
     To make a projection and then several images, you can generate a
@@ -161,38 +154,6 @@ class FixedResolutionBuffer(object):
             if f not in exclude and f[0] not in self.data_source.ds.particle_types:
                 self[f]
 
-    def _is_ion( self, fname ):
-        p = re.compile("_p[0-9]+_")
-        result = False
-        if p.search( fname ) is not None:
-            result = True
-        return result
-
-    def _ion_to_label( self, fname ):
-        pnum2rom = {
-            "0":"I", "1":"II", "2":"III", "3":"IV", "4":"V",
-            "5":"VI", "6":"VII", "7":"VIII", "8":"IX", "9":"X",
-            "10":"XI", "11":"XII", "12":"XIII", "13":"XIV", "14":"XV",
-            "15":"XVI", "16":"XVII", "17":"XVIII", "18":"XIX", "19":"XX"}
-
-        p = re.compile("_p[0-9]+_")
-        m = p.search( fname )
-        if m is not None:
-            pstr = m.string[m.start()+1:m.end()-1]
-            segments = fname.split("_")
-            for i,s in enumerate(segments):
-                segments[i] = s.capitalize()
-                if s == pstr:
-                    ipstr = i
-            element = segments[ipstr-1]
-            roman = pnum2rom[pstr[1:]]
-            label = element + '\ ' + roman + '\ ' + \
-                '\ '.join(segments[ipstr+1:])
-        else:
-            label = fname
-        return label
-
-
     def _get_info(self, item):
         info = {}
         ftype, fname = field = self.data_source._determine_fields(item)[0]
@@ -216,18 +177,7 @@ class FixedResolutionBuffer(object):
         except AttributeError:
             pass
 
-        info['label'] = finfo.display_name
-        if info['label'] is None:
-            if self._is_ion( fname ):
-                fname = self._ion_to_label( fname )
-                info['label'] = r'$\rm{'+fname+r'}$'
-                info['label'] = r'$\rm{'+fname.replace('_','\ ')+r'}$'
-            else:
-                info['label'] = r'$\rm{'+fname+r'}$'
-                info['label'] = r'$\rm{'+fname.replace('_','\ ').title()+r'}$'
-        elif info['label'].find('$') == -1:
-            info['label'] = info['label'].replace(' ','\ ')
-            info['label'] = r'$\rm{'+info['label']+r'}$'
+        info['label'] = finfo.get_latex_display_name()
 
         return info
 
@@ -288,6 +238,44 @@ class FixedResolutionBuffer(object):
         dpy = (self.bounds[3]-self.bounds[2])/self.buff_size[1]
         return distance/dpy
 
+    def set_unit(self, field, unit, equivalency=None, equivalency_kwargs=None):
+        """Sets a new unit for the requested field
+
+        parameters
+        ----------
+        field : string or field tuple
+           The name of the field that is to be changed.
+
+        unit : string or Unit object
+           The name of the new unit.
+
+        equivalency : string, optional
+           If set, the equivalency to use to convert the current units to
+           the new requested unit. If None, the unit conversion will be done
+           without an equivelancy
+
+        equivalency_kwargs : string, optional
+           Keyword arguments to be passed to the equivalency. Only used if
+           ``equivalency`` is set.
+        """
+        if equivalency_kwargs is None:
+            equivalency_kwargs = {}
+        field = self.data_source._determine_fields(field)[0]
+        if equivalency is None:
+            self[field].convert_to_units(unit)
+        else:
+            equiv_array = self[field].to_equivalent(
+                unit, equivalency, **equivalency_kwargs)
+            # equiv_array isn't necessarily an ImageArray. This is an issue
+            # inherent to the way the unit system handles YTArray
+            # sublcasses and I don't see how to modify the unit system to
+            # fix this. Instead, we paper over this issue and hard code
+            # that equiv_array is an ImageArray
+            self[field] = ImageArray(
+                equiv_array, equiv_array.units, equiv_array.units.registry,
+                self[field].info)
+
+
     def export_hdf5(self, filename, fields = None):
         r"""Export a set of fields to a set of HDF5 datasets.
 
@@ -330,7 +318,7 @@ class FixedResolutionBuffer(object):
             the length units that the coordinates are written in, default 'cm'.
         """
 
-        from yt.utilities.fits_image import FITSImageData
+        from yt.visualization.fits_image import FITSImageData
 
         if fields is None:
             fields = list(self.data.keys())
@@ -494,6 +482,11 @@ class FixedResolutionBuffer(object):
             self.__dict__['apply_' + filtername] = \
                 types.MethodType(filt, self)
 
+class ObliqueFixedResolutionBuffer(FixedResolutionBuffer):
+    @deprecate("FixedResolutionBuffer")
+    def __init__(self, *args, **kwargs):
+        super(ObliqueFixedResolutionBuffer, self).__init__(*args, **kwargs)
+
 class CylindricalFixedResolutionBuffer(FixedResolutionBuffer):
     """
     This object is a subclass of
@@ -515,41 +508,13 @@ class CylindricalFixedResolutionBuffer(FixedResolutionBuffer):
 
     def __getitem__(self, item) :
         if item in self.data: return self.data[item]
-        buff = pixelize_cylinder(self.data_source["r"], self.data_source["dr"],
-                                 self.data_source["theta"], self.data_source["dtheta"],
-                                 self.buff_size, self.data_source[item].astype("float64"),
-                                 self.radius)
+        buff = np.zeros(self.buff_size, dtype="f8")
+        pixelize_cylinder(buff, self.data_source["r"], self.data_source["dr"],
+                          self.data_source["theta"], self.data_source["dtheta"],
+                          self.data_source[item].astype("float64"), self.radius)
         self[item] = buff
         return buff
 
-class ObliqueFixedResolutionBuffer(FixedResolutionBuffer):
-    """
-    This object is a subclass of
-    :class:`yt.visualization.fixed_resolution.FixedResolutionBuffer`
-    that supports non-aligned input data objects, primarily cutting planes.
-    """
-    def __getitem__(self, item):
-        if item in self.data: return self.data[item]
-        indices = np.argsort(self.data_source['dx'])[::-1]
-        bounds = []
-        for b in self.bounds:
-            if hasattr(b, "in_units"):
-                b = float(b.in_units("code_length"))
-            bounds.append(b)
-        buff = pixelize_off_axis_cartesian(
-                               self.data_source['x'],   self.data_source['y'],   self.data_source['z'],
-                               self.data_source['px'],  self.data_source['py'],
-                               self.data_source['pdx'], self.data_source['pdy'], self.data_source['pdz'],
-                               self.data_source.center, self.data_source._inv_mat, indices,
-                               self.data_source[item],
-                               self.buff_size[0], self.buff_size[1],
-                               bounds).transpose()
-        ia = ImageArray(buff, input_units=self.data_source[item].units,
-                        info=self._get_info(item))
-        self[item] = ia
-        return ia
-
-
 class OffAxisProjectionFixedResolutionBuffer(FixedResolutionBuffer):
     """
     This object is a subclass of
@@ -639,24 +604,31 @@ class ParticleImageBuffer(FixedResolutionBuffer):
 
         # splat particles
         buff = np.zeros(self.buff_size)
+        buff_mask = np.zeros(self.buff_size).astype('int')
         add_points_to_greyscale_image(buff,
+                                      buff_mask,
                                       px[mask],
                                       py[mask],
                                       splat_vals)
+        # remove values in no-particle region
+        buff[buff_mask==0] = np.nan
         ia = ImageArray(buff, input_units=data.units,
                         info=self._get_info(item))
 
         # divide by the weight_field, if needed
         if weight_field is not None:
             weight_buff = np.zeros(self.buff_size)
+            weight_buff_mask = np.zeros(self.buff_size).astype('int')
             add_points_to_greyscale_image(weight_buff,
+                                          weight_buff_mask,
                                           px[mask],
                                           py[mask],
                                           weight_data[mask])
             weight_array = ImageArray(weight_buff,
                                       input_units=weight_data.units,
                                       info=self._get_info(item))
-
+            # remove values in no-particle region
+            weight_buff[weight_buff_mask==0] = np.nan
             locs = np.where(weight_array > 0)
             ia[locs] /= weight_array[locs]
 
