@@ -14,41 +14,44 @@ rendering.
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-from yt.funcs import mylog
-from yt.data_objects.profiles import BinnedProfile1D
-from yt.visualization.volume_rendering.api import ColorTransferFunction
-from yt.visualization._mpl_imports import FigureCanvasAgg
-from matplotlib.figure import Figure
-from yt.extern.six import BytesIO
+import matplotlib
 import numpy as np
 
+from distutils.version import LooseVersion
+
+from yt.funcs import mylog
+from yt.data_objects.profiles import create_profile
+from yt.visualization.volume_rendering.transfer_functions import \
+    ColorTransferFunction
+from yt.extern.six import BytesIO
+
 
 class TransferFunctionHelper(object):
+    r"""A transfer function helper.
 
+    This attempts to help set up a good transfer function by finding
+    bounds, handling linear/log options, and displaying the transfer
+    function combined with 1D profiles of rendering quantity.
+
+    Parameters
+    ----------
+    ds: A Dataset instance
+        A static output that is currently being rendered. This is used to
+        help set up data bounds.
+
+    Notes
+    -----
+    """
+ 
     profiles = None
 
     def __init__(self, ds):
-        r"""A transfer function helper.
-
-        This attempts to help set up a good transfer function by finding
-        bounds, handling linear/log options, and displaying the transfer
-        function combined with 1D profiles of rendering quantity.
-
-        Parameters
-        ----------
-        ds: A Dataset instance
-            A static output that is currently being rendered. This is used to
-            help set up data bounds.
-
-        Notes
-        -----
-        """
         self.ds = ds
         self.field = None
         self.log = False
         self.tf = None
         self.bounds = None
-        self.grey_opacity = True
+        self.grey_opacity = False
         self.profiles = {}
 
     def set_bounds(self, bounds=None):
@@ -64,7 +67,8 @@ class TransferFunctionHelper(object):
             in the dataset.  This can be slow for very large datasets.
         """
         if bounds is None:
-            bounds = self.ds.all_data().quantities.extrema(self.field)
+            bounds = self.ds.h.all_data().quantities['Extrema'](self.field, non_zero=True)
+            bounds = [b.ndarray_view() for b in bounds]
         self.bounds = bounds
 
         # Do some error checking.
@@ -83,6 +87,8 @@ class TransferFunctionHelper(object):
         field: string
             The field to be rendered.
         """
+        if field != self.field:
+            self.log = self.ds._get_field_info(field).take_log
         self.field = field
 
     def set_log(self, log):
@@ -97,8 +103,6 @@ class TransferFunctionHelper(object):
             Sets whether the transfer function should use log or linear space.
         """
         self.log = log
-        self.ds.index
-        self.ds._get_field_info(self.field).take_log = log
 
     def build_transfer_function(self):
         """
@@ -117,18 +121,35 @@ class TransferFunctionHelper(object):
         """
         if self.bounds is None:
             mylog.info('Calculating data bounds. This may take a while.' +
-                       '  Set the .bounds to avoid this.')
+                       '  Set the TranferFunctionHelper.bounds to avoid this.')
             self.set_bounds()
 
         if self.log:
             mi, ma = np.log10(self.bounds[0]), np.log10(self.bounds[1])
         else:
             mi, ma = self.bounds
+
         self.tf = ColorTransferFunction((mi, ma),
                                         grey_opacity=self.grey_opacity,
                                         nbins=512)
         return self.tf
 
+    def setup_default(self):
+        """Setup a default colormap
+
+        Creates a ColorTransferFunction including 10 gaussian layers whose
+        colors smaple the 'spectral' colormap. Also attempts to scale the
+        transfer function to produce a natural contrast ratio.
+
+        """
+        if LooseVersion(matplotlib.__version__) < LooseVersion('2.0.0'):
+            colormap_name = 'spectral'
+        else:
+            colormap_name = 'nipy_spectral'
+        self.tf.add_layers(10, colormap=colormap_name)
+        factor = self.tf.funcs[-1].y.size / self.tf.funcs[-1].y.sum()
+        self.tf.funcs[-1].y *= 2*factor
+
     def plot(self, fn=None, profile_field=None, profile_weight=None):
         """
         Save the current transfer function to a bitmap, or display
@@ -146,8 +167,11 @@ class TransferFunctionHelper(object):
         If fn is None, will return an image to an IPython notebook.
 
         """
+        from yt.visualization._mpl_imports import FigureCanvasAgg
+        from matplotlib.figure import Figure
         if self.tf is None:
             self.build_transfer_function()
+            self.setup_default()
         tf = self.tf
         if self.log:
             xfunc = np.logspace
@@ -176,11 +200,12 @@ class TransferFunctionHelper(object):
             except KeyError:
                 self.setup_profile(profile_field, profile_weight)
                 prof = self.profiles[self.field]
-            if profile_field not in prof.keys():
-                prof.add_fields([profile_field], fractional=False,
-                                weight=profile_weight)
+            try:
+                prof[profile_field]
+            except KeyError:
+                prof.add_fields([profile_field])
             # Strip units, if any, for matplotlib 1.3.1
-            xplot = np.array(prof[self.field])
+            xplot = np.array(prof.x)
             yplot = np.array(prof[profile_field]*tf.funcs[3].y.max() /
                              prof[profile_field].max())
             ax.plot(xplot, yplot, color='w', linewidth=3)
@@ -205,11 +230,9 @@ class TransferFunctionHelper(object):
     def setup_profile(self, profile_field=None, profile_weight=None):
         if profile_field is None:
             profile_field = 'cell_volume'
-        prof = BinnedProfile1D(self.ds.all_data(), 128, self.field,
-                               self.bounds[0], self.bounds[1],
-                               log_space=self.log,
-                               end_collect=False)
-        prof.add_fields([profile_field], fractional=False,
-                        weight=profile_weight)
+        prof = create_profile(self.ds.all_data(), self.field, profile_field,
+                              n_bins=128, extrema={self.field: self.bounds},
+                              weight_field=profile_weight,
+                              logs = {self.field: self.log})
         self.profiles[self.field] = prof
         return
